// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: Image.cc,v 1.2 2005/01/28 23:55:09 brendt Exp $

*/

#ifndef _Image_cc
#define _Image_cc

#include <math.h>

#if !defined(INCLUDE_TEMPLATE_DEFINITIONS)
#include "Image.h"
#endif

//NB: See pg. 363 of The Design and Evolution of C++


template <class T, class R>
Image<T, R>::Image(const Subimage<T, R>& subimage) :
  ProtoVector<T, R, ImageSize>(ImageSize(subimage.width,subimage.height)) 
{
  unsigned int i,j;
  for (j = 0; j < subimage.height; j++)
    for (i = 0; i < subimage.width; i++)
      operator()(i,j)  = subimage.image(subimage.x + i,
					subimage.y + j);
}

template <class T, class R>
Image<T, R>& Image<T, R>::operator=(const Subimage<T, R>& subimage)
{
  unsigned int i,j;
  if (&subimage.image == this) //image is assigned to subregion of
    {			       //itself
      Image<T, R> temp(ImageSize(subimage.width, subimage.height));
      for (j = 0; j < subimage.height; j++)
	for (i = 0; i < subimage.width; i++)
	  temp(i,j) = subimage.image(subimage.x + i, subimage.y + j);
      ProtoVector<T, R, ImageSize>::destroy();
      swap(temp, *this);
      return *this;
    }
  
  ASSERT_FATAL(
      (ProtoVector<T, R, ImageSize>::reconstruct(ImageSize(subimage.width,
							   subimage.height)))
		); 
  
  for (j = 0; j < subimage.height; j++)
    for (i = 0; i < subimage.width; i++)
      operator()(i,j)  = subimage.image(subimage.x + i, subimage.y + j);
  return *this;
}


template <class T, class R>
Subimage<T, R> Image<T, R>::operator()(unsigned int x, unsigned int y,
			   unsigned int w, unsigned int h) const
{
  assert( x+w <= iwidth() && y+h <= iheight() );
  Subimage<T, R> subimage(*this, x, y, w, h);
  return subimage;
}


template <class T, class R>
Subimage<T, R> Image<T, R>::row(unsigned int r) const
{
  assert( r < iheight() );
  Subimage<T, R> subimage(*this, 0, r, iwidth(), 1);
  return subimage;
}

template <class T, class R>
Subimage<T, R> Image<T, R>::col(unsigned int c) const
{
  assert( c < iwidth() );
  Subimage<T, R> subimage(*this, c, 0, 1, iheight());
  return subimage;
}


template <class T, class R>
void Image<T, R>::reset(const ImageSize& is)
{
  ProtoVector<T, R, ImageSize>::destroy();
  ASSERT_FATAL( construct(is) );
}




/*

 Note: Operation in range 0 .. 7, consisting of 3 bits
         bit 0: reflection along main diagonal
         bit 1: horizontal reflection
         bit 2: vertical reflection

*/ 

template <class T, class R>
Image<T, R>& Image<T, R>::isometry(const Image<T, R>& image, unsigned char op)
{
 ASSERT_WARNING(op < 8);

  ImageSize isosize = image.size();
  if (op & 1 != 0) //Check transposition bit
    isosize.transpose();

  ASSERT_FATAL( (ProtoVector<T, R, ImageSize>::size() == isosize) );
  
  unsigned int i,j;
  switch (op) {
  case 0 : for (j = 0; j < image.iheight(); j++) //Identity
    for (i = 0; i < image.iwidth(); i++)
      operator()(i,j) = image(i,j);
    break;
  case 5 : for (j = 0; j < image.iheight(); j++) //90 degree clockwise
    for (i = 0; i < image.iwidth(); i++)
      operator()(i,j) = image(j, image.iwidth()-1-i);
    break;
  case 6 : for (j = 0; j < image.iheight(); j++) //180 degree clockwise
    for (i = 0; i < image.iwidth(); i++)
      operator()(i,j) = image(image.iwidth()-1-i,image.iheight()-1-j);
    break;
  case 3 : for (j = 0; j < image.iheight(); j++) //270 degree clockwise
    for (i = 0; i < image.iwidth(); i++)
      operator()(i,j) = image(image.iheight()-1-j, i);
    break;
  case 2 : for (j = 0; j < image.iheight(); j++) //horizontal reflection of 0
    for (i = 0; i < image.iwidth(); i++)
      operator()(i,j) = image(i,image.iheight()-1-j);
    break;
  case 7 : for (j = 0; j < image.iheight(); j++) //horizontal reflection of 1
    for (i = 0; i < image.iwidth(); i++)
      operator()(i,j) = image(image.iheight()-1-j,image.iwidth()-1-i);
    break;
  case 4 : for (j = 0; j < image.iheight(); j++) //horizontal reflection of 2
    for (i = 0; i < image.iwidth(); i++)
      operator()(i,j) = image(image.iwidth()-1-i,j);
    break;
  case 1 : for (j = 0; j < image.iheight(); j++) //horizontal reflection of 3
    for (i = 0; i < image.iwidth(); i++)
      operator()(i,j) = image(j,i);
    break;
  }
  return *this;
}




template <class T, class R>
bool Image<T, R>::native_write(char* filename) const
{
  unsigned int size[2];
  size[0] = iwidth();
  size[1] = iheight();

  ofstream ofs(filename, IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  if (!ofs.write((char *)size, sizeof(size)))
    {
      ofs.close();
      return false;
    }
  bool status = ProtoVector<T, R, ImageSize>::raw_write(ofs);
  ofs.close();
  return status;
}

template <class T, class R>
bool Image<T, R>::native_read(char* filename)
{
  unsigned int size[2];

  ifstream ifs(filename, IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;

  if (!ifs.read((char *)size, sizeof(size)))
    {
      ifs.close();
      return false;
    }
  else
    reset(ImageSize(size[0],size[1]));

  bool status = ProtoVector<T, R, ImageSize>::raw_read(ifs);
  ifs.close();
  return status;
}


template <class T, class R>
Image<T, R>& Image<T, R>::subsample(const Image<T, R>& im,
				    unsigned int k, unsigned int l)
{
  unsigned int nwidth = im.iwidth()/k;
  unsigned int nheight = im.iheight()/l;
  ASSERT_FATAL( (ProtoVector<T, R, ImageSize>::size() == ImageSize(nwidth, nheight)) );
  unsigned int i, j;
  for (j = 0; j < nheight; j++)
  for (i = 0; i < nwidth; i++)
    operator()(i,j) = im(k*i, l*j);
  return *this;
}

template <class T, class R>
bool Image<T, R>::load(const ImageFileInterface<T, R>& imfi)
{
  return imfi.load(*this);
}

template <class T, class R>
bool Image<T, R>::save(const ImageFileInterface<T, R>& imfi) const
{
  return imfi.save(*this);
}


template <class T, class R>
ostream& operator<<(ostream &s, const Image<T, R>& im)
{
  unsigned int imax = im.iwidth();
  unsigned int jmax = im.iheight();
  unsigned int fieldw = s.width();
  unsigned int fieldp = s.precision();
  unsigned int i,j;
  for (j = 0; j < jmax; j++)
    {
      for (i = 0; i < imax; i++)
	s << setw(fieldw) << setprecision(fieldp) << numeric(im(i,j));
      s << endl;
    }
  return s;
}

#endif
