// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: Image.h,v 1.3 2005/09/05 20:31:14 brendt Exp $

 */
  
 
#ifndef _Image_h
#define _Image_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
#else
#include <iostream.h>
#include <fstream.h>
#endif

#include "ImageSize.h"
#include "ProtoVector.h"
#include "Subimage.h"

//! unit="Image and Vector Types"
//! module="Image Support Classes"
//! level = 2

template <class T, class R>
class ImageRef;
template <class T, class R>
class ImageFileInterface;

template <class T, class R>
class Image;

template <class T, class R>
inline ImageRef<T, R> operator+(const Image<T, R>& u, const Image<T, R>& v);

template <class T, class R>
inline ImageRef<T, R> operator+(const Image<T, R>& u, T k);

template <class T, class R>
inline ImageRef<T, R> operator-(const Image<T, R>& u, const Image<T, R>& v);

template <class T, class R>
inline ImageRef<T, R> operator-(const Image<T, R>& u, T k);

template <class T, class R>
inline ImageRef<T, R> operator-(T k, const Image<T, R>& u);

template <class T, class R>
inline ImageRef<T, R> operator*(const Image<T, R>& u, R k);

template <class T, class R>
inline ImageRef<T, R> operator*(R k, const Image<T, R>& u);

template <class T, class R>
inline ImageRef<T, R> operator/(const Image<T, R>& u, R k);

template <class T, class R>
inline ImageRef<T, R> abs(const Image<T, R>& u);

template <class T, class R>
inline ImageRef<T, R> clip(const Image<T, R>& u, T lower, T upper);

template <class T, class R>
inline ImageRef<T, R> isometry(const Image<T, R>& image, unsigned char op);

template <class T, class R>
inline ImageRef<T, R> subsample(const Image<T, R>& im,
       		         unsigned int k=2, unsigned int l=2);

template <class T, class R>
ostream& operator<<(ostream &s, const Image<T, R>& im);

//: Template image class providing basic image operations. 
// Template image class having template parameters 
//T - vector element type eg. int
//R - return type for norms etc.
template <class T, class R>
class Image : public ProtoVector<T, R, ImageSize> {
protected:
  int construct(const ImageSize& is);
  int construct(unsigned int w, unsigned int h);
  T* get_pv_ptr(void) const;
  void set_pv_ptr(T* p) MTCONST;
  void set_size(const ImageSize& ssize);
  Image(const ImageSize& init_size, T* init_ptr);
  
public:
  Image(unsigned int w = 0, unsigned int h = 0);
  Image(unsigned int w, unsigned int h, T pixel);
  Image(const ImageSize& size);
  //Construct given vector size and element initialiser
  Image(const ImageSize& size, T pixel);

  //Cloning copy constructor/type conversions (other types here as well)
#ifdef NO_MEMBER_TEMPLATES
  Image(const Image<unsigned char, real>& ip);
  Image(const Image<unsigned short, real>& ip);
  Image(const Image<float, float>& ip);
#else
  Image(const Image<T, R>& ip);
  template <class P, class Q>
    Image(const Image<P, Q>& ip);
#endif

  //Cloning assignment
  const Image<T, R>& operator=(const Image<T, R>& ip);

  //Reference passing copy constructor
  Image(MTCONST ImageRef<T, R>& r);
  //Reference passing assignment operator
  Image<T, R>& operator=(MTCONST ImageRef<T, R>& r);

  //Image dimension  access
  unsigned int iwidth() const { 
    return ProtoVector<T, R, ImageSize>::size().iwidth(); }
  unsigned int iheight() const { 
    return ProtoVector<T, R, ImageSize>::size().iheight(); }

  //Destroy image and reallocate image of requested size
  void reset(const ImageSize& is);
  
  //Pixel access
  T& operator()(unsigned int x, unsigned int y) const;
  T& operator()(const PixelIndex& pi) const;

  //Arithmetic operations
  friend FRIEND_INLINE ImageRef<T, R> 
         TMPLT_FRND_SPCL(operator+)(const Image<T, R>& u, 
				    const Image<T, R>& v);
  friend FRIEND_INLINE ImageRef<T, R> 
         TMPLT_FRND_SPCL(operator+)(const Image<T, R>& u, T k);
  friend FRIEND_INLINE ImageRef<T, R> 
         TMPLT_FRND_SPCL(operator-)(const Image<T, R>& u,
				    const Image<T, R>& v);
  friend FRIEND_INLINE ImageRef<T, R> 
         TMPLT_FRND_SPCL(operator-)(const Image<T, R>& u, T k);
  friend FRIEND_INLINE ImageRef<T, R> 
         TMPLT_FRND_SPCL(operator-)(T k, const Image<T, R>& u);
  friend FRIEND_INLINE ImageRef<T, R> 
         TMPLT_FRND_SPCL(operator*)(const Image<T, R>& u, R k);
  friend FRIEND_INLINE ImageRef<T, R> 
         TMPLT_FRND_SPCL(operator*)(R k, const Image<T, R>& u);
  friend FRIEND_INLINE ImageRef<T, R> 
         TMPLT_FRND_SPCL(operator/)(const Image<T, R>& u, R k);

#if defined(REQUIRE_UNSHADOW_KLUDGE)
  template<class K, class L>
  friend FRIEND_INLINE ImageRef<K, L> abs(const Image<K, L>& u);
  template<class K, class L>
  friend FRIEND_INLINE ImageRef<K, L> 
         clip(const Image<K, L>& u, K lower, K upper);
#else
  friend FRIEND_INLINE ImageRef<T, R> UNSHADOW(abs)(const Image<T, R>& u);
  friend FRIEND_INLINE ImageRef<T, R> 
         UNSHADOW(clip)(const Image<T, R>& u, T lower, T upper);
#endif

  //Compound assignments
  Image<T, R>& operator+=(const Image<T, R>& u);
  Image<T, R>& operator+=(T k);
  Image<T, R>& operator-=(const Image<T, R>& u);
  Image<T, R>& operator-=(T k);
  Image<T, R>& operator*=(T k);
  Image<T, R>& operator/=(T k);
 
  //Sub-image
  //Constructor creating image from a subimage
  Image(const Subimage<T, R>& subimage);
  //Assignment operator setting image from a subimage. Note that
  //memory is re-allocated if subimage size does not match existing
  //image size
  Image<T, R>& operator=(const Subimage<T, R>& subimage);
  //Select subimage of image
  Subimage<T, R> operator()(unsigned int x, unsigned int y,
			    unsigned int w, unsigned int h) const;
  Subimage<T, R> row(unsigned int r) const;
  Subimage<T, R> col(unsigned int c) const;
  friend class Subimage<T, R>;
 
  //Rotation and reflection of *this image. 
  //Operation in range 0 .. 7, consisting of 3 bits
  //bit 0: reflection along main diagonal
  //bit 1: horizontal reflection
  //bit 2: vertical reflection
  Image<T, R>& isometry(const Image<T, R>& image, unsigned char op);

#if defined(REQUIRE_UNSHADOW_KLUDGE)
  template<class K, class L>
  friend FRIEND_INLINE ImageRef<K, L> 
         isometry(const Image<K, L>& image, unsigned char op);
#else
  friend FRIEND_INLINE ImageRef<T, R> 
         UNSHADOW(isometry)(const Image<T, R>& image, unsigned char op);
#endif

  //Subsample image im and place result in *this. Arguments are
  //subsampling ratios in horizontal and vertical directions
  Image<T, R>& subsample(const Image<T, R>& im,
			 unsigned int k=2, unsigned int l=2);

#if defined(REQUIRE_UNSHADOW_KLUDGE)
  template<class K, class L>
  friend FRIEND_INLINE ImageRef<K, L> 
         subsample(const Image<K, L>& im, unsigned int k, unsigned int l);
#else
  //Return subsampled version of image im
  friend FRIEND_INLINE ImageRef<T, R> 
         UNSHADOW(subsample)(const Image<T, R>& im,
			     unsigned int k, unsigned int l);
#endif

  //File access
  bool native_write(char* filename) const;
  bool native_read(char* filename);

  //Stream output in ASCII
  friend ostream& TMPLT_FRND_SPCL(operator<<)(ostream &s, 
					      const Image<T, R>& im);


  bool load(const ImageFileInterface<T, R>& imfi);
  bool save(const ImageFileInterface<T, R>& imfi) const;

  friend class ImageRef<T, R>;
};


template <class T, class R>
inline int Image<T, R>::construct(const ImageSize& is)
{
  return ProtoVector<T, R, ImageSize>::construct(is);
}

template <class T, class R>
inline int Image<T, R>::construct(unsigned int w, unsigned int h)
{
  return construct(ImageSize(w,h));
}

template <class T, class R>
inline T* Image<T, R>::get_pv_ptr(void) const
{
  return ProtoVector<T, R, ImageSize>::get_pv_ptr();
}

template <class T, class R>
inline void Image<T, R>::set_pv_ptr(T* p) MTCONST
{
  ProtoVector<T, R, ImageSize>::set_pv_ptr(p);
}

template <class T, class R>
inline void Image<T, R>::set_size(const ImageSize& ssize)
{
  ProtoVector<T, R, ImageSize>::set_size(ssize);
}

template <class T, class R>
inline Image<T, R>::Image(const ImageSize& init_size, T* init_ptr) :
  ProtoVector<T, R, ImageSize>(init_size, init_ptr) {}

template <class T, class R>
inline Image<T, R>::Image(unsigned int w, unsigned int h) :
  ProtoVector<T, R, ImageSize>(ImageSize(w, h)) {}

template <class T, class R>
inline Image<T, R>::Image(unsigned int w, unsigned int h, T pixel) :
  ProtoVector<T, R, ImageSize>(ImageSize(w, h), pixel) {}

template <class T, class R>
inline Image<T, R>::Image(const ImageSize& size) :
  ProtoVector<T, R, ImageSize>(size) {}

  
template <class T, class R>
inline Image<T, R>::Image(const ImageSize& size, T pixel) :
  ProtoVector<T, R, ImageSize>(size, pixel) {}

#ifdef NO_MEMBER_TEMPLATES
template <class T, class R>
inline Image<T, R>::Image(const Image<unsigned char, real>& ip) :
  ProtoVector<T, R, ImageSize>(ip) {}

template <class T, class R>
inline Image<T, R>::Image(const Image<unsigned short, real>& ip) :
  ProtoVector<T, R, ImageSize>(ip) {}

template <class T, class R>
inline Image<T, R>::Image(const Image<float, float>& ip) :
  ProtoVector<T, R, ImageSize>(ip) {}
#else
template <class T, class R>
inline Image<T, R>::Image(const Image<T, R>& ip) :
  ProtoVector<T, R, ImageSize>(ip) {}

template <class T, class R>
  template <class P, class Q>
inline Image<T, R>::Image(const Image<P, Q>& ip) :
  ProtoVector<T, R, ImageSize>(ip) {}
#endif


template <class T, class R>
inline const Image<T, R>& Image<T, R>::operator=(const Image<T, R>& ip)
{
  ProtoVector<T, R, ImageSize>::operator=(ip);
  return *this;
}


template <class T, class R>
inline Image<T, R>::Image(MTCONST ImageRef<T, R>& r) :
  ProtoVector<T, R, ImageSize>(r.size(), r.get_pv_ptr())
{
  r.set_pv_ptr(0);
}

template <class T, class R>
inline Image<T, R>& Image<T, R>::operator=(MTCONST ImageRef<T, R>& r) 
{
  ProtoVector<T, R, ImageSize>::destroy();
  set_size(r.size());
  set_pv_ptr(r.get_pv_ptr());
  r.set_pv_ptr(0);
  return *this;
}

template <class T, class R>
inline T& Image<T, R>::operator()(unsigned int x, unsigned int y) const
{
  assert( x < iwidth() && y < iheight() );
  return pv_elt(x + iwidth() * y);
}

template <class T, class R>
inline T& Image<T, R>::operator()(const PixelIndex& pi) const
{
  assert( (ProtoVector<T, R, ImageSize>::size().withinbounds(pi)) );
  return pv_elt(pi.x + iwidth() * pi.y);
}

template <class T, class R>
inline ImageRef<T, R> operator+(const Image<T, R>& u,
				const Image<T, R>& v)
{
  Image<T, R> vecsum(u.size());
  vecsum.sum(u, v);
  return vecsum;
}

template <class T, class R>
inline ImageRef<T, R> operator+(const Image<T, R>& u, T k)
{
  Image<T, R> vecsum(u.size());
  vecsum.sum(u, k);
  return vecsum;
}

template <class T, class R>
inline ImageRef<T, R> operator-(const Image<T, R>& u,
				const Image<T, R>& v)
{
  Image<T, R> vecdif(u.size());
  vecdif.dif(u, v);
  return vecdif;
}

template <class T, class R>
inline ImageRef<T, R> operator-(const Image<T, R>& u, T k)
{
  Image<T, R> vecdif(u.size());
  vecdif.dif(u, k);
  return vecdif;
}

template <class T, class R>
inline ImageRef<T, R> operator-(T k, const Image<T, R>& u)
{
  Image<T, R> vecdif(u.size());
  vecdif.dif(k, u);
  return vecdif;
}

template <class T, class R>
inline ImageRef<T, R> operator*(const Image<T, R>& u, R k)
{
  Image<T, R> vecprod(u.size());
  vecprod.prod(u, k);
  return vecprod;
}

template <class T, class R>
inline ImageRef<T, R> operator*(R k, const Image<T, R>& u)
{
  return u * k;
}

template <class T, class R>
inline ImageRef<T, R> operator/(const Image<T, R>& u, R k)
{
  return u * (1.0f/k); 
}

template <class T, class R>
inline ImageRef<T, R> abs(const Image<T, R>& u)
{
  Image<T, R> vecabs(u.size());
  vecabs.abs(u);
  return vecabs;
}

template <class T, class R>
inline ImageRef<T, R> clip(const Image<T, R>& u, T lower, T upper)
{
  Image<T, R> vecclip(u.size());
  vecclip.clip(u, lower, upper);
  return vecclip;
}
  

template <class T, class R>
inline Image<T, R>& Image<T, R>::operator+=(const Image<T, R>& u)
{
  sum(*this, u);
  return *this;
}

template <class T, class R>
inline Image<T, R>& Image<T, R>::operator+=(T k)
{
  sum(*this, k);
  return *this;
}

template <class T, class R>
inline Image<T, R>& Image<T, R>::operator-=(const Image<T, R>& u)
{
  dif(*this, u);
  return *this;
}

template <class T, class R>
inline Image<T, R>& Image<T, R>::operator-=(T k)
{
  dif(*this, k);
  return *this;
}

template <class T, class R>
inline Image<T, R>& Image<T, R>::operator*=(T k)
{
  prod(*this, k);
  return *this;
}

template <class T, class R>
inline Image<T, R>& Image<T, R>::operator/=(T k)
{
  prod(*this, 1.0/k);
  return *this;
}

template <class T, class R>
inline ImageRef<T, R> isometry(const Image<T, R>& image, unsigned char op)
{
  ImageSize isosize = image.size();
  if (op & 1 != 0) //Check transposition bit
    isosize.transpose();
  Image<T, R> imiso(isosize);
  imiso.isometry(image, op);
  return imiso;
}
  
template <class T, class R>
inline ImageRef<T, R> subsample(const Image<T, R>& u,
				unsigned int k, unsigned int l)
{
  Image<T, R> imsubs(u.iwidth()/k, u.iheight()/l);
  imsubs.subsample(u, k, l);
  return imsubs;
}


//: Template class transporting a reference to an Image<T,R> 
template <class T, class R>
class ImageRef : public Image<T, R>{
public:
  ImageRef(MTCONST Image<T, R>& v) :
    Image<T, R>(v.size(), v.get_pv_ptr()) 
  { 
    v.set_pv_ptr(0);
  }

  friend class Image<T, R>;
};


#include "ImageFile.h"
//: Template class interface to image file mechanism
template <class T, class R>
class ImageFileInterface {
public:
  virtual ~ImageFileInterface() {}
  virtual bool load(Image<T, R>& im) const = 0;
  virtual bool save(const Image<T, R>& im) const = 0;
};


#if defined(INCLUDE_TEMPLATE_DEFINITIONS)
#include "Image.cc"
#endif

#endif
