// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: ImageIterate.h,v 1.1 2003/02/01 17:46:23 brendt Exp $
  
*/

#ifndef _ImageIterate_h
#define _ImageIterate_h

#include "ImageSize.h"

//! unit="Image and Vector Types"
//! module="Image Iterators"
//! level = 1


//: Abstract interface for image iterators
class ImageIterate : public PixelIndex {
public:
  //: State of the iterator state machine
  enum State {initial, running, final, stop};

protected:
  ImageSize size;
  State iistate;

public:
  ImageIterate(const ImageSize& is);
  ImageIterate(unsigned int w, unsigned int h);
  virtual ~ImageIterate(void) {}
  virtual void reset(void);
  // Return current state
  State state(void) const { return iistate; }
  // Move to next pixel and return state
  virtual State next(void) = 0;
  // Pre-increment
  PixelIndex operator++(void);
  // Post-increment
  PixelIndex operator++(int);
};


inline ImageIterate::ImageIterate(const ImageSize& is)
  {
    size = is;
    reset();
  }
inline ImageIterate::ImageIterate(unsigned int w, unsigned int h)
  {
    size = ImageSize(w, h);
    reset();
  }

inline PixelIndex ImageIterate::operator++(void) 
  {
    next();
    return (PixelIndex)*this;
  }

inline PixelIndex ImageIterate::operator++(int)
  {
    PixelIndex pi = (PixelIndex)*this;
    next();
    return pi;
  }


//! level = 2

//: Raster scan
class RasterIterate : public ImageIterate {
public:
  RasterIterate(const ImageSize& is) :
    ImageIterate(is) {}
  RasterIterate(unsigned int w, unsigned int h) :
    ImageIterate(w, h) {}
              
  State next(void);
};


//! level = 2
//: Scan left and right along alternating rows
class AlternatingRasterIterate : public ImageIterate {
public:
  // Additional state information
  enum ARState {forwards, backwards};
private:
  ARState arstate;
public:
  AlternatingRasterIterate(const ImageSize& is) :
    ImageIterate(is) { arstate = forwards; }
  AlternatingRasterIterate(unsigned int w, unsigned int h) :
    ImageIterate(w, h) { arstate = forwards; }
              
  State next(void);
  // Return additional state information
  ARState pathdrn(void) const { return arstate; }
};


//! level = 2
//: Zig-zag scan as in JPEG scan of DCT coefficients
class ZigzagIterate : public ImageIterate {
public:
  // Additional state information
  enum ZState {horizontal, vertical, downdiagonal, updiagonal};
private:
  ZState zstate;
public:
  ZigzagIterate(const ImageSize& is) :
    ImageIterate(is) { zstate = horizontal; }
  ZigzagIterate(unsigned int w, unsigned int h) :
    ImageIterate(w, h) { zstate = horizontal; }

  State next(void);
  // Return additional state information
  ZState pathdrn(void) const { return zstate; }
};
#endif
