// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/
  
/*

  $Id: ImageLib.h,v 1.1 2003/02/01 17:46:23 brendt Exp $
  
*/

//Filename: ImageLib.h
//Description: Common declarations for entire image class library


#ifndef _ImageLib_h
#define _ImageLib_h

#include "imlibconf.h"  //NB: change the way this is included

#include <assert.h>
#include <stdlib.h> //required for exit
#include <math.h>


/*****************************************************************************/
//Compiler dependence

#ifndef PI
#define PI M_PI
#endif

#ifndef HAVE_BOOL
enum bool {false, true};
#endif

#ifdef HAVE_MUTABLE
#define MUTABLE mutable
#define MTCONST const
#else
#define MUTABLE
#define MTCONST
#endif

#ifdef REQUIRE_FRIEND_INLINE
#define FRIEND_INLINE inline
#else
#define FRIEND_INLINE
#endif

#ifdef HAVE_EXPLICIT_TEMPLATE_INSTANTIATION
#define EXPLICIT_TEMPLATE_INSTANTIATION 1
#else
#define INCLUDE_TEMPLATE_DEFINITIONS 1
#endif

#ifndef HAVE_MEMBER_TEMPLATES
#define NO_MEMBER_TEMPLATES 1
#endif

#ifdef NEW_TEMPLATE_FRIEND_SYNTAX
#define TMPLT_FRND_SPCL(x) x<>
#else
#define TMPLT_FRND_SPCL(x) x
#endif

#ifdef ACCEPTS_PARENTHESISED_SCOPE_OPERATOR
#ifdef NEW_TEMPLATE_FRIEND_SYNTAX
#define UNSHADOW(x) (::x<>)
#else
#define UNSHADOW(x) (::x)
#endif
#else
#ifdef NEW_TEMPLATE_FRIEND_SYNTAX
#define UNSHADOW(x) x<>
#else
#define UNSHADOW(x) x
#endif 
#endif

#if defined(STD_NAME_LOOKUP) && \
   !defined(ACCEPTS_PARENTHESISED_SCOPE_OPERATOR) && \
    defined(ACCEPTS_FRIEND_SHADOWING_WORKAROUND)
#define REQUIRE_UNSHADOW_KLUDGE 1
#endif

#ifdef HAVE_IOS_BIN
#ifdef NEW_HEADER_NAMES
#define IOS_IN std::ios::in | std::ios::bin
#define IOS_OUT std::ios::out | std::ios::bin
#else
#define IOS_IN ios::in | ios::bin
#define IOS_OUT ios::out | ios::bin
#endif
#else
#ifdef NEW_HEADER_NAMES
#define IOS_IN std::ios::in
#define IOS_OUT std::ios::out
#else
#define IOS_IN ios::in
#define IOS_OUT ios::out
#endif
#endif

#ifdef HAVE_IOSTREAM_IS_OPEN
#define STREAM_OPEN(s) (s.is_open())
#else
#define STREAM_OPEN(s) (s)
#endif

#if defined(__GNUC__)
#if (__GNUC__< 2) || (__GNUC__==2 && __GNUC_MINOR__ < 8)
#error "Compile with a g++ release at least as recent as 2.8.0"
#endif 
#endif

/*****************************************************************************/
//Compile options

#if defined(HAVE_NUMERIC_LIMITS) && defined(ENABLE_OVERFLOW_CHECKING)
#define OVERFLOW_CHECKING 1  //Check for overflow in pixel arithmetic
#endif

#if defined(ENABLE_ARG_CHECKING)
#define ARG_CHECKING 1  //Check function argument bounds
#endif

#if defined(REAL_TYPE)
typedef REAL_TYPE real;
#else
typedef float real;
#endif

/*****************************************************************************/
//Auxiliary functions

template<class T> inline const T& min(const T& a, const T& b)
{
  return (a < b) ? a : b;
}

template<class T> inline const T& max(const T& a, const T& b)
{
  return (a > b) ? a : b;
}

template<class T> inline T abs(T a)
{
  return (a >= 0) ? a : -a;
}

template<> inline unsigned char abs(unsigned char a)
{
  return a;
}

template<> inline unsigned short abs(unsigned short a)
{
  return a;
}

template<> inline unsigned int abs(unsigned int a)
{
  return a;
}

template<> inline unsigned long abs(unsigned long a)
{
  return a;
}

template<class T> inline void swap(T& a, T& b)
{
  T tmp = a;
  a = b;
  b = tmp;
}


inline unsigned int rnmod(int m, unsigned int n)
{
  return (m < 0) ? ( ( (-m)%n ) ? (n - ((-m) % n) ):0 ) : (m % n);
}

float nearest_int(float a);

/*****************************************************************************/
//Error handling

#ifdef __GNUC__
#define ASSERT_FATAL(expression)\
        if (!(expression)) { cerr << "Assertion " << #expression\
	            << " failed in function " << __PRETTY_FUNCTION__\
                    << "\nat line " << __LINE__\
		    << " in file " << __FILE__ << endl;\
	exit(1); }
#else
#define ASSERT_FATAL(expression)\
        if (!(expression)) { cerr << "Assertion " << #expression\
	            << " failed\nat line " << __LINE__\
		    << " in file " << __FILE__ << endl;\
	exit(1); }
#endif



#ifdef __GNUC__
#define ASSERT_WARNING(expression)\
        if (!(expression)) { cerr << "Assertion " << #expression\
	            << " failed in function " << __PRETTY_FUNCTION__\
                    << "\nat line " << __LINE__\
		    << " in file " << __FILE__ << endl; }
#else
#define ASSERT_WARNING(expression)\
        if (!(expression)) { cerr << "Assertion " << #expression\
	            << " failed\nat line " << __LINE__\
		    << " in file " << __FILE__ << endl; }
#endif



#endif
