// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 19958 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: ImageSize.h,v 1.1 2003/02/01 17:46:23 brendt Exp $
 
*/

#ifndef _ImageSize_h
#define _ImageSize_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
using std::ostream;
#else
#include <iostream.h>
#endif


//! unit="Image and Vector Types"
//! module="Image Support Classes"
//! level = 1


//: Pixel position within an image
// PixelIndex represents the position of a pixel within an image

struct PixelIndex {
  unsigned int x;
  unsigned int y;

  //Set x and y coordinates
  void set(unsigned int sx = 0, unsigned int sy = 0);
};

inline void PixelIndex::set(unsigned int sx, unsigned int sy)
{
  x = sx;
  y = sy;
}


//: Image dimensions
// ImageSize represents the width and height of an image. Horizontal
// dimensions range between 0 and width-1, and vertical dimensions
// range between 0 and height-1
class ImageSize {
private:
  unsigned int width;   /* x indices range from 0 .. width-1 */
  unsigned int height;  /* y indices range from 0 .. height-1 */
public:
  //Construct null ImageSize
  ImageSize(void) { reset(); }
  //Construct ImageSize with specified width and height
  ImageSize(unsigned int w, unsigned int h) { width = w; height = h; }
  //Construct ImageSize given another ImageSize
  ImageSize(const ImageSize& is);
  //Reset dimensions to zero width and height
  void reset() { width = 0; height = 0; }
  //Return the corresponding number of pixels
  unsigned int elements() const { return width*height; }
  // Return the image width
  unsigned int iwidth() const { return width; }
  // Return the image height
  unsigned int iheight() const { return height; }
  // Equality test
  bool operator==(const ImageSize& is) const;
  // Inequality test
  bool operator!=(const ImageSize& is) const;
  // Test for null dimensions (zero width and height)
  bool null() { return bool(width == 0 && height == 0); }
  // Transpose the dimensions (width become height and vice versa)
  void transpose();
  // Test whether a PixelIndex lies within the bounds of the image dimensions
  bool withinbounds(const PixelIndex& pi) const;
  // Test whether a PixelIndex lies on any of the image boundaries
  bool boundary(const PixelIndex& pi) const;
  // Test whether a PixelIndex lies on the left side boundary
  bool leftboundary(const PixelIndex& pi) const;
  // Test whether a PixelIndex lies on the top boundary
  bool topboundary(const PixelIndex& pi) const;
  // Test whether a PixelIndex lies on the right side boundary
  bool rightboundary(const PixelIndex& pi) const;
  // Test whether a PixelIndex lies on the bottom boundary
  bool bottomboundary(const PixelIndex& pi) const;

  //Write text description of image size (width x height) to a stream
  friend ostream& operator<<(ostream &s, const ImageSize& is);
};

inline ImageSize::ImageSize(const ImageSize& is)
{ 
  width = is.width;
  height = is.height;
}

inline bool ImageSize::operator==(const ImageSize& is) const
{
  return ((width == is.width) && (height == is.height))?true:false;
}

inline bool ImageSize::operator!=(const ImageSize& is) const
{
  return bool(!operator==(is));
}
  
inline void ImageSize::transpose()
{
  unsigned int tmp = width;
  width = height;
  height = tmp;
}

inline bool ImageSize::withinbounds(const PixelIndex& pi) const
{
  return bool(pi.x < iwidth() && pi.y < iheight());
}

inline bool ImageSize::boundary(const PixelIndex& pi) const
{
  return bool(leftboundary(pi) || rightboundary(pi) ||
	      topboundary(pi)  || bottomboundary(pi));
}

inline bool ImageSize::leftboundary(const PixelIndex& pi) const
{
  return bool(pi.x == 0);
}

inline bool ImageSize::topboundary(const PixelIndex& pi) const
{
  return bool(pi.y == 0);
}

inline bool ImageSize::rightboundary(const PixelIndex& pi) const
{
  return bool(pi.x == iwidth() - 1);
}

inline bool ImageSize::bottomboundary(const PixelIndex& pi) const
{
  return bool(pi.y == iheight() - 1);
}


inline ostream& operator<<(ostream &s, const ImageSize& is)
{
  s << is.iwidth() << " x " << is.iheight();
  return s;
}
#endif
