// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: ProtoVector.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $

  
*/

#ifndef _ProtoVector_cc
#define _ProtoVector_cc

#include <math.h>

#include "ImageLib.h"

#if !defined(INCLUDE_TEMPLATE_DEFINITIONS)
#include "ProtoVector.h"
#endif

#ifdef OVERFLOW_CHECKING
#include <limits>
#endif

// Conversion functions for use in stream operations for derived types
template<class T> static inline T numeric(T a)
{
  return a;
}

static inline unsigned short numeric(unsigned char a)
{
  return (unsigned short)a;
}


template <class T, class R, class S>
int ProtoVector<T, R, S>::cmpT(const void * ap, const void * bp)
{
  T a = *(T *)ap;
  T b = *(T *)bp;

  if (a > b)
    return 1;
  if (a < b)
    return -1;
  return 0;
}


template <class T, class R, class S>
bool ProtoVector<T, R, S>::construct(const S& init_size)
{
  vsize = init_size;
  if (init_size.elements() > 0)
    {
      if ((pv_ptr = new T [init_size.elements()]) == 0)
	return false;  // memory could not be allocated
      else
	return true;
    }
  else
    {
      pv_ptr = 0;
      return true;
    }
}


template <class T, class R, class S>
bool ProtoVector<T, R, S>::reconstruct(const S& new_size)
{
  if (pv_ptr != 0)
    {
      if (vsize != new_size)
	{
	  destroy();
          return construct(new_size);
	}
      else
	return true;
    }
  else
    return construct(new_size);
}


template <class T, class R, class S>
ProtoVector<T, R, S>::ProtoVector(const S& init_size)
{
  ASSERT_FATAL( construct(init_size) );
}


template <class T, class R, class S>
ProtoVector<T, R, S>::ProtoVector(const S& init_size, T elt)
{
  ASSERT_FATAL( construct(init_size) );
  for (unsigned int i = 0; i < init_size.elements(); i++)
    pv_ptr[i] = elt;
}


#ifdef NO_MEMBER_TEMPLATES
template <class T, class R, class S>
ProtoVector<T, R, S>::ProtoVector(const ProtoVector<unsigned char,real,S>& v)
{
  ASSERT_FATAL( construct(v.size()) );  //Cloning required
  for (unsigned int i = 0; i < v.size().elements(); i++)
      pv_ptr[i] = (T)v.pv_elt(i);
}

template <class T, class R, class S>
ProtoVector<T,R,S>::ProtoVector(const ProtoVector<unsigned short,real,S>& v)
{
  ASSERT_FATAL( construct(v.size()) );  //Cloning required
  for (unsigned int i = 0; i < v.size().elements(); i++)
      pv_ptr[i] = (T)v.pv_elt(i);
}

template <class T, class R, class S>
ProtoVector<T, R, S>::ProtoVector(const ProtoVector<float, float, S>& v)
{
  ASSERT_FATAL( construct(v.size()) );  //Cloning required
  for (unsigned int i = 0; i < v.size().elements(); i++)
      pv_ptr[i] = (T)v.pv_elt(i);
}
#else
template <class T, class R, class S>
ProtoVector<T, R, S>::ProtoVector(const ProtoVector<T, R, S>& v)
{
  ASSERT_FATAL( construct(v.size()) );  //Cloning required
  for (unsigned int i = 0; i < v.size().elements(); i++)
      pv_ptr[i] = v.pv_ptr[i];
}

template <class T, class R, class S>
 template <class P, class Q>
ProtoVector<T, R, S>::ProtoVector(const ProtoVector<P, Q, S>& v)
{
  ASSERT_FATAL( construct(v.size()) );  //Cloning required
  for (unsigned int i = 0; i < v.size().elements(); i++)
      pv_ptr[i] = (T)v.pv_elt(i);
}
#endif


template <class T, class R, class S>
ProtoVector<T, R, S>& ProtoVector<T, R, S>::operator=(const ProtoVector<T, R, S>& v)
{
  if (&v == this)
    return *this;
  ASSERT_FATAL( reconstruct(v.size()) );
  for (unsigned int i = 0; i < v.size().elements(); i++)
    pv_ptr[i] = v.pv_ptr[i];
  return *this;
}


template <class T, class R, class S>
void swap(ProtoVector<T, R, S>& u, ProtoVector<T, R, S>& v)
{
  S tmp_size = u.vsize;
  u.vsize = v.vsize;
  v.vsize = tmp_size;
  T *tmp_ptr = u.pv_ptr;
  u.pv_ptr = v.pv_ptr;
  v.pv_ptr = tmp_ptr;
}


template <class T, class R, class S>
bool ProtoVector<T, R, S>::operator==(ProtoVector<T, R, S>& u) const
{
  ASSERT_FATAL( size() == u.size() );
  unsigned int i;
  for (i = 0; i < size().elements(); i++)
    if (pv_ptr[i] != u.pv_ptr[i])
      return false;
  return true;
}



template <class T, class R, class S>
ProtoVector<T, R, S>&
ProtoVector<T, R, S>::sum(const ProtoVector<T, R, S>& u,
			  const ProtoVector<T, R, S>& v)
{
  ASSERT_FATAL( (u.size() == v.size()) &&
		(v.size() == size()) );
    
  unsigned int i;
#ifdef OVERFLOW_CHECKING
  T half_sum;
  T max_bound = numeric_limits<T>::max/2;
  T min_bound = numeric_limits<T>::min/2;
#endif
  for (i = 0; i < size().elements(); i++)
#ifdef OVERFLOW_CHECKING
    {
      half_sum = u.pv_ptr[i]/2 + v.pv_ptr[i]/2;
      if (half_sum > max_bound)
	pv_ptr[i] = ProtoVector<T, R, S>::elt_max;  //Overflow
      else
	if (half_sum < min_bound)
	  pv_ptr[i] = ProtoVector<T, R, S>::elt_min;  //Underflow
	else
	  pv_ptr[i] = u.pv_ptr[i] + v.pv_ptr[i];
    }
#else
  pv_ptr[i] = u.pv_ptr[i] + v.pv_ptr[i];
#endif
  return *this;
}


template <class T, class R, class S>
ProtoVector<T, R, S>&
ProtoVector<T, R, S>::dif(const ProtoVector<T, R, S>& u,
			  const ProtoVector<T, R, S>& v)
{
  ASSERT_FATAL( (u.size() == v.size()) &&
		(v.size() == size()) );
  
  unsigned int i;

#ifdef OVERFLOW_CHECKING
  T half_dif;
  T max_bound = numeric_limits<T>::max/2;
  T min_bound = numeric_limits<T>::min/2;
#endif
  for (i = 0; i < size().elements(); i++)
#ifdef OVERFLOW_CHECKING
    {
      half_dif = u.pv_ptr[i]/2 - v.pv_ptr[i]/2;
      if (half_dif > max_bound)
	pv_ptr[i] = ProtoVector<T, R, S>::elt_max;  //Overflow
      else
	if (half_dif < min_bound)
	  pv_ptr[i] = ProtoVector<T, R, S>::elt_min;  //Underflow
	else
	  pv_ptr[i] = u.pv_ptr[i] - v.pv_ptr[i];
    }
#else
  pv_ptr[i] = u.pv_ptr[i] - v.pv_ptr[i];
#endif
  return *this;
}



template <class T, class R, class S>
ProtoVector<T, R, S>&
ProtoVector<T, R, S>::sum(const ProtoVector<T, R, S>& u, T k)
{
  ASSERT_FATAL( u.size()  == size() );
 
  unsigned int i;
  
#ifdef OVERFLOW_CHECKING
  T half_sum;
  T half_k = k/2;
  T max_bound = numeric_limits<T>::max/2;
  T min_bound = numeric_limits<T>::min/2;
#endif
  for (i = 0; i < size().elements(); i++)
#ifdef OVERFLOW_CHECKING
    {
      half_sum = u.pv_ptr[i]/2 + half_k;
      if (half_sum > max_bound)
	pv_ptr[i] = ProtoVector<T, R, S>::elt_max;  //Overflow
      else
	if (half_sum < min_bound)
	  pv_ptr[i] = ProtoVector<T, R, S>::elt_min;  //Underflow
	else
	  pv_ptr[i] = u.pv_ptr[i] + k;
    }
#else
  pv_ptr[i] = u.pv_ptr[i] + k;
#endif
  return *this;
}



template <class T, class R, class S>
ProtoVector<T, R, S>&
ProtoVector<T, R, S>::dif(const ProtoVector<T, R, S>& u, T k)
{
  ASSERT_FATAL( u.size()  == size() );
  
  unsigned int i;

#ifdef OVERFLOW_CHECKING
  T half_dif;
  T half_k = k/2;
  T max_bound = numeric_limits<T>::max/2;
  T min_bound = numeric_limits<T>::min/2;
#endif
  
  for (i = 0; i < size().elements(); i++)
#ifdef OVERFLOW_CHECKING
    {
      half_dif = u.pv_ptr[i]/2 - half_k;
      if (half_dif > max_bound)
	pv_ptr[i] = ProtoVector<T, R, S>::elt_max;  //Overflow
      else
	if (half_dif < min_bound)
	  pv_ptr[i] = ProtoVector<T, R, S>::elt_min;  //Underflow
	else
	  pv_ptr[i] =  u.pv_ptr[i] - k;
    }
#else
  pv_ptr[i] = u.pv_ptr[i] - k;
#endif
  return *this;
}



template <class T, class R, class S>
ProtoVector<T, R, S>&
ProtoVector<T, R, S>::dif(T k, const ProtoVector<T, R, S>& u)
{
  ASSERT_FATAL( u.size()  == size() );
  
  unsigned int i;

#ifdef OVERFLOW_CHECKING
  T half_dif;
  T half_k = k/2;
  T max_bound = numeric_limits<T>::max/2;
  T min_bound = numeric_limits<T>::min/2;
#endif
  
  for (i = 0; i < size().elements(); i++)
#ifdef OVERFLOW_CHECKING
    {
      half_dif = half_k - u.pv_ptr[i]/2;
      if (half_dif > max_bound)
	pv_ptr[i] = ProtoVector<T, R, S>::elt_max;  //Overflow
      else
	if (half_dif < min_bound)
	  pv_ptr[i] = ProtoVector<T, R, S>::elt_min;  //Underflow
	else
	  pv_ptr[i] =  k - u.pv_ptr[i];
    }
#else
  pv_ptr[i] = k - u.pv_ptr[i];
#endif
  return *this;
}


template <class T, class R, class S>
ProtoVector<T, R, S>&
ProtoVector<T, R, S>::prod(const ProtoVector<T, R, S>& u, R k)
{
  ASSERT_FATAL( u.size()  == size() );
  
  unsigned int i;
 
#ifdef OVERFLOW_CHECKING
  T max_bound = numeric_limits<T>::max/k;
  T min_bound = numeric_limits<T>::min/k;
  T pixel;
#endif
  
  for (i = 0; i < size().elements(); i++)
#ifdef OVERFLOW_CHECKING
    {
      pixel = u.pv_ptr[i];
      if (pixel > max_bound)
	pv_ptr[i] = ProtoVector<T, R, S>::elt_max;  //Overflow
      else
	if (pixel < min_bound)
	  pv_ptr[i] = ProtoVector<T, R, S>::elt_min;  //Underflow
	else
	  pv_ptr[i] = pixel * k;
    }
#else
  pv_ptr[i] = (T)(u.pv_ptr[i] * k);
#endif
  return *this;
}


template <class T, class R, class S>
ProtoVector<T, R, S>&
ProtoVector<T, R, S>::abs(const ProtoVector<T, R, S>& u)
{
  ASSERT_FATAL( u.size() == size() );

  for (unsigned int i = 0; i < size().elements(); i++)
    pv_elt(i) = ::abs(u.pv_elt(i));
  return *this;
}


template <class T, class R, class S>
ProtoVector<T, R, S>&
ProtoVector<T, R, S>::clip(const ProtoVector<T, R, S>& u,
			   T lower, T upper)
{
  ASSERT_FATAL( u.size() == size() );

  for (unsigned int i = 0; i < size().elements(); i++)
    {
      if (u.pv_elt(i) < lower)
	pv_elt(i) = lower;
      else
	if (u.pv_elt(i) > upper)
	  pv_elt(i) = upper;
      else
	pv_elt(i) = u.pv_elt(i);
    }
  return *this;
}



template <class T, class R, class S>
R ProtoVector<T, R, S>::normL1() const
{
  R total = 0;
  unsigned int i;
  
  for (i = 0; i < size().elements(); i++)
      total += (R)fabs(pv_ptr[i]);
  
  return total;
}


template <class T, class R, class S>
R ProtoVector<T, R, S>::normL2() const
{
  R pixel, total = 0;
  unsigned int i;
  
  for (i = 0; i < size().elements(); i++)
    {
      pixel = (R)pv_ptr[i];
      total += pixel*pixel;
    }
  return sqrt(total);
}


template <class T, class R, class S>
R ProtoVector<T, R, S>::normLp(R p) const
{
  R pixel, total = 0;
  unsigned int i;
 
  for (i = 0; i < size().elements(); i++)
    {
      pixel = (R)pv_ptr[i];
      total += pow(pixel, (R)p);
    }
  return pow(total, 1.0/((R)p));
}


template <class T, class R, class S>
R inproduct(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v)
{
  ASSERT_WARNING(u.size() == v.size());
  
  R total = 0;
  unsigned int elts = min(u.size().elements(), v.size().elements());
  unsigned int i;

  for (i = 0; i < elts; i++)
    total += (R)u.pv_ptr[i] * (R)v.pv_ptr[i];
  return total;
}


template <class T, class R, class S>
R MSE(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v)
{
  ASSERT_WARNING(u.size() == v.size());
  
  R total = 0, term;
  unsigned int elts = min(u.size().elements(), v.size().elements());
  unsigned int i;

  for (i = 0; i < elts; i++)
    {
      term = (R)u.pv_ptr[i] - (R)v.pv_ptr[i];
      total += term*term;
    }
  return total/R(u.size().elements());
}


template <class T, class R, class S>
R ProtoVector<T, R, S>::sum() const
{
  R total = 0;
  unsigned int i;
 
  for (i = 0; i < size().elements(); i++)
      total += (R)pv_ptr[i];

  return total;
}


template <class T, class R, class S>
R ProtoVector<T, R, S>::sum2() const
{
  R sample, total = 0;
  unsigned int i;
 
  for (i = 0; i < size().elements(); i++)
    {
      sample = (R)pv_ptr[i];
      total += sample*sample;
    }

  return total;
}



template <class T, class R, class S>
R ProtoVector<T, R, S>::variance() const
{
  R sample, total = 0, m = mean();
  unsigned int i;
 
  for (i = 0; i < size().elements(); i++)
    {
      sample = (R)pv_ptr[i] - m;
      total += sample*sample;
    }

  return total/R(size().elements());
}


template <class T, class R, class S>
R ProtoVector<T, R, S>::origin_moment(unsigned int k) const
{
  R total = 0;
  unsigned int i;
   
  for (i = 0; i < size().elements(); i++)
    total += pow((R)pv_ptr[i], (R)k);
  return total/(R)size().elements();
}


template <class T, class R, class S>
R ProtoVector<T, R, S>::mean_moment(unsigned int k) const
{
  R mean = origin_moment(1);
  R total = 0;
  unsigned int i;
   
  for (i = 0; i < size().elements(); i++)
    total += pow((R)pv_ptr[i] - mean, (R)k);
  return total/(R)size().elements();
}


template <class T, class R, class S>
T ProtoVector<T, R, S>::min() const
{
  unsigned int i;
  T minval = pv_ptr[0];
  for (i = 1; i < size().elements(); i++)
    if (pv_ptr[i] < minval)
      minval = pv_ptr[i];
  return minval;
}


template <class T, class R, class S>
T ProtoVector<T, R, S>::max() const
{
  unsigned int i;
  T maxval = pv_ptr[0];
  for (i = 1; i < size().elements(); i++)
    if (pv_ptr[i] > maxval)
      maxval = pv_ptr[i];
  return maxval;
}



template <class T, class R, class S>
T ProtoVector<T, R, S>::rank(unsigned int k) const
{
  ASSERT_FATAL(k < size().elements());

  T *sortarray = new T [size().elements()];
  unsigned int i;
  for (i = 0; i < size().elements(); i++)
    sortarray[i] = pv_ptr[i];
  qsort(sortarray, size().elements(), sizeof(T), cmpT);
  T ret = sortarray[k];
  delete [] sortarray;
  return ret;
}



template <class T, class R, class S>
bool ProtoVector<T, R, S>::raw_write(ofstream& ofs) const
{
  if (!ofs.write((char *)&pv_elt(0), vsize.elements()*sizeof(T)))
    return false;
  else
    return true;
}


template <class T, class R, class S>
bool ProtoVector<T, R, S>::raw_read(ifstream& ifs)
{
  if (!ifs.read((char *)&pv_elt(0), vsize.elements()*sizeof(T)))
    return false;
  else
    return true;
}


template <class T, class R, class S>
bool ProtoVector<T, R, S>::raw_write(char* filename) const
{
  ofstream ofs(filename, IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  bool status = raw_write(ofs);
  ofs.close();
  return status;
}


template <class T, class R, class S>
bool ProtoVector<T, R, S>::raw_read(char* filename)
{
  ifstream ifs(filename, IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;

  bool status = raw_read(ifs);
  ifs.close();
  return status;
}

#endif 
