// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: ProtoVector.h,v 1.1 2003/02/01 17:46:23 brendt Exp $

 */

#ifndef _ProtoVector_h
#define _ProtoVector_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
using std::ofstream;
using std::ifstream;
using std::endl;
using std::cerr;
#else
#include <iostream.h>
#include <fstream.h>
#endif


//! unit="Image and Vector Types"
//! module="Support Classes"
//! level = 1

template <class T, class R, class S>
class ProtoVector;

template <class T, class R, class S>
void swap(ProtoVector<T, R, S>& u, ProtoVector<T, R, S>& v);

template <class T, class R, class S>
R inproduct(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v);

template <class T, class R, class S>
R MSE(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v);

template <class T, class R, class S>
inline R RMSE(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v);

template <class T, class R, class S>
inline R SNR(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v);

template <class T, class R, class S>
inline R PSNR(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v);


//: Class providing basic vector space operations.
// Template class providing operations common to vectors and images
// Template parameters: T - vector element type eg. int
//                      R - return type for norms etc.
//                      S - dimension specifying class eg. ImageSize
template <class T, class R, class S>
class ProtoVector {
private:
  S vsize;     /* vector dimensions */
  MUTABLE T* pv_ptr;   /* pointer to data */

  //Order on elements required for qsort in rank() method
  //Arguments ap and bp are pointers to scalars a and b; the function
  //returns -1 if a < b, o if a == b, and 1 if a > b
  static int cmpT(const void *ap, const void *bp);
  
protected:
  //Allocate memory for vector of specified size. Returns false on
  //failure, true on success. No memory allocated if none required for
  //init_size, in which case method returns successfully
  bool construct(const S& init_size);
  //Re-allocates memory for vector if specified size differs
  //from existing size (in which case previous contents deleted). 
  //Returns false on failure, true on success.
  bool reconstruct(const S& new_size);
  //De-allocate memory
  void destroy();

  /* Routines for private data access by derived classes */

  //Return pointer to array containing vector data
  T* get_pv_ptr(void) const { return pv_ptr; }
  //Set pointer to array containing vector data
  void set_pv_ptr(T* p) MTCONST { pv_ptr = p; }
  //Set recorded vector size without changing actual data
  void set_size(const S& ssize) { vsize = ssize; }
  //Construct given a size and a block of data
  ProtoVector(const S&init_size, T* init_ptr);  

public:
  //Construct given desired size
  ProtoVector(const S& init_size);
  //Construct given desired vsize and initial value
  ProtoVector(const S& init_size, T elt);
  ~ProtoVector() {  destroy(); }
  
  //Copy constructor and conversions between types with different T
#ifdef NO_MEMBER_TEMPLATES
  ProtoVector(const ProtoVector<unsigned char, real, S>& ip);
  ProtoVector(const ProtoVector<unsigned short, real, S>& ip);
  ProtoVector(const ProtoVector<float, float, S>& ip);
#else
  ProtoVector(const ProtoVector<T, R, S>& v);
  template <class P, class Q>
   ProtoVector(const ProtoVector<P, Q, S>& v);
#endif

  //Assignment operator
  ProtoVector<T, R, S>& operator=(const ProtoVector<T, R, S>& v);

  //Return reference to vector element
  T& pv_elt(unsigned int i) const { return pv_ptr[i]; }

  //Return object vsize
  const S& size() const { return vsize; }
  //Swap internals of two objects
  friend void TMPLT_FRND_SPCL(swap)(ProtoVector<T, R, S>& u, 
				    ProtoVector<T, R, S>& v);
  
  //Equality check
  bool operator==(ProtoVector<T, R, S>& u) const;
  //Inquality check
  bool operator!=(ProtoVector<T, R, S>& u) const {return !(*this == u);}


  //Add two vectors (*this is the destination of the sum)
  ProtoVector<T, R, S>& sum(const ProtoVector<T, R, S>& u,
			    const ProtoVector<T, R, S>& v);
  //Subtract one vector from another
  ProtoVector<T, R, S>& dif(const ProtoVector<T, R, S>& u,
			    const ProtoVector<T, R, S>& v); 
  //Add a scalar to all elements of a vector
  ProtoVector<T, R, S>& sum(const ProtoVector<T, R, S>& u, T k);
  //Subtract a scalar from all elements of  a vector
  ProtoVector<T, R, S>& dif(const ProtoVector<T, R, S>& u, T k);
  //Subtract  all elements of  a vector from a scalar
  ProtoVector<T, R, S>& dif(T k, const ProtoVector<T, R, S>& u);
  //Multiply a vector by a scalar
  ProtoVector<T, R, S>& prod(const ProtoVector<T, R, S>& u, R k);
  //Absolute value of all elements
  ProtoVector<T, R, S>& abs(const ProtoVector<T, R, S>& u);
  //Clip range of elements
  ProtoVector<T, R, S>& clip(const ProtoVector<T, R, S>& u,
			     T lower, T upper);
  

  //Inner product
  friend R TMPLT_FRND_SPCL(inproduct)(const ProtoVector<T, R, S>& u,
				      const ProtoVector<T, R, S>& v);


  //L_1 norm
  R normL1() const;
  //L_2 norm
  R normL2() const;
  //L_p norm
  R normLp(R p) const;

  //Mean Square Error between two vectors
  friend R TMPLT_FRND_SPCL(MSE)(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v);
  //Root Mean Square Error between two vectors
  friend FRIEND_INLINE R TMPLT_FRND_SPCL(RMSE)(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v);
  //Root Mean Square Error between two vectors
  friend FRIEND_INLINE R TMPLT_FRND_SPCL(SNR)(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v);
  //Peak Signal to Noise Ratio between signal u and signal with noise v
  friend FRIEND_INLINE R TMPLT_FRND_SPCL(PSNR)(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v);

  
  //Sum of elements in vector
  R sum() const;
  //Sum of squares of elements in a vector
  R sum2() const;
  //Deterministic mean of vector elements
  R mean() const { return sum()/real(size().elements()); }
  //Deterministic variance of vector elements
  R variance() const;
  //Deterministic moment about the origin
  R origin_moment(unsigned int k) const;
  //Deterministic moment about the mean
  R mean_moment(unsigned int k) const;

  
  //Minimum element in vector
  T min() const;
  //Maximum element in vector
  T max() const;
  //Element of rank k in vector
  T rank(unsigned int k) const;


  //Write file in raw format. Returns true on success, false on failure
  bool raw_write(ofstream& ofs) const;
  bool raw_write(char* filename) const;
  //Read file in raw format (vector on disk assumed same size as *this)
  bool raw_read(ifstream& ifs);
  bool raw_read(char* filename);

};

template <class T, class R, class S>
inline void ProtoVector<T, R, S>::destroy()
{
  delete [] pv_ptr;
}

template <class T, class R, class S>
inline ProtoVector<T, R, S>::ProtoVector(const S&init_size, T* init_ptr)
{
  set_size(init_size);
  set_pv_ptr(init_ptr);
}


template <class T, class R, class S>
inline R RMSE(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v)
{
  return sqrt(MSE(u,v));
}

template <class T, class R, class S>
inline R SNR(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v)
{
  return 10.0*log10(u.variance()/MSE(u,v));
}

template <class T, class R, class S>
inline R PSNR(const ProtoVector<T, R, S>& u, const ProtoVector<T, R, S>& v)
{
  return 20.0*log10( ((real)u.max() - (real)u.min())/RMSE(u,v) );
}


#if defined(INCLUDE_TEMPLATE_DEFINITIONS)
#include "ProtoVector.cc"
#endif

#endif
