// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

 $Id: Vector.cc,v 1.2 2005/01/28 23:55:09 brendt Exp $

*/

#ifndef _Vector_cc
#define _Vector_cc

#if !defined(INCLUDE_TEMPLATE_DEFINITIONS)
#include "Vector.h"
#endif

template <class T, class R>
Vector<T, R>::Vector(const Subimage<T, R>& subimage) :
  ProtoVector<T, R, VectorSize>(VectorSize(::max(subimage.width,
					         subimage.height)))
{
  assert(::min(subimage.width, subimage.height) == 1);
  if (subimage.height == 1) // Row vector
    {
      for (unsigned int i = 0; i < vlength(); i++)
	operator()(i) = subimage.image(subimage.x+i, subimage.y);
    }
  else
    {
      for (unsigned int i = 0; i < vlength(); i++)
	operator()(i) = subimage.image(subimage.x, subimage.y+i);
    }
}



template <class T, class R>
Vector<T, R>::Vector(const Subvector<T, R>& subvector) :
  ProtoVector<T, R, VectorSize>(VectorSize(subvector.width)) 
{
  unsigned int i;
    for (i = 0; i < subvector.width; i++)
      operator()(i)  = subvector.vector(subvector.x + i);
}


template <class T, class R>
Vector<T, R>& Vector<T, R>::operator=(const Subvector<T, R>& subvector)
{
  unsigned int i;
  if (&subvector.vector == this) //vector is assigned to subregion of
    {			       //itself
      Vector<T, R> temp(VectorSize(subvector.width));
	for (i = 0; i < subvector.width; i++)
	  temp(i) = subvector.vector(subvector.x + i);
	ProtoVector<T, R, VectorSize>::destroy();
      swap(temp, *this);
      return *this;
    }
  
  ASSERT_FATAL(
    (ProtoVector<T, R, VectorSize>::reconstruct(VectorSize(subvector.width)))
		); 
  
    for (i = 0; i < subvector.width; i++)
      operator()(i)  = subvector.vector(subvector.x + i);
  return *this;
}


template <class T, class R>
Subvector<T, R> Vector<T, R>::operator()(unsigned int x, unsigned int w) const
{
  assert( x+w <= vlength() );
  Subvector<T, R> subvector(*this, x, w);
  return subvector;
}



template <class T, class R>
void Vector<T, R>::reset(const VectorSize& vs)
{
  ProtoVector<T, R, VectorSize>::destroy();
  ASSERT_FATAL( (ProtoVector<T, R, VectorSize>::construct(vs)) );
}


template <class T, class R>
ImageRef<T, R> rowimage(const Vector<T, R>& u)
{
  Image<T, R> ri(u.vlength(), 1);
  for (unsigned int i = 0; i < u.vlength(); i++)
    ri(i, 0) = u(i);
  return ri;
}

template <class T, class R>
ImageRef<T, R> colimage(const Vector<T, R>& u)
{
  Image<T, R> ci(1, u.vlength());
  for (unsigned int i = 0; i < u.vlength(); i++)
    ci(0,i) = u(i);
  return ci;
}



template <class T, class R>
Vector<T, R>& Vector<T, R>::subsample(const Vector<T, R>& u, unsigned int k)
{
  unsigned int nlength = u.vlength()/k;
  ASSERT_FATAL( vlength() == nlength );
  for (unsigned int i = 0; i < nlength; i++)
    operator()(i) = u(k*i);
  return *this;
}


template <class T, class R>
R Vector<T, R>::integral(unsigned int first, unsigned int last)
{
  R total = (ProtoVector<T, R, VectorSize>::pv_elt(first) + 
             ProtoVector<T, R, VectorSize>::pv_elt(last))/2;
  unsigned int i;
  for (i = first+1; i < last; i++)
    total += ProtoVector<T, R, VectorSize>::pv_elt(i);
  return total;
}


template <class T, class R>
Vector<T, R>& Vector<T, R>::concatenate(const Vector<T, R>& u,
					const Vector<T, R>& v)
{
  ASSERT_FATAL( vlength() == u.vlength() + v.vlength() ); 
  unsigned int i;
  for (i = 0; i < u.vlength(); i++)
    operator()(i) = u(i);
  for (i = 0; i < v.vlength(); i++)
    operator()(i+u.vlength()) = v(i);

  return *this;
}


template <class T, class R>
bool Vector<T, R>::native_write(char* filename) const
{
  unsigned int size;
  size = vlength();

  ofstream ofs(filename, IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  if (!ofs.write((char *)&size, sizeof(size)))
    {
      ofs.close();
      return false;
    }
  bool status = ProtoVector<T, R, VectorSize>::raw_write(ofs);
  ofs.close();
  return status;
}


template <class T, class R>
bool Vector<T, R>::native_read(char* filename)
{
  unsigned int size;

  ifstream ifs(filename, IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;

  if (!ifs.read((char *)&size, sizeof(size)))
    {
      ifs.close();
      return false;
    }
  else
    reset(VectorSize(size));

  bool status = ProtoVector<T, R, VectorSize>::raw_read(ifs);
  ifs.close();
  return status;
}

template <class T, class R>
bool Vector<T, R>::load(const VectorFileInterface<T, R>& vfi)
{
  return vfi.load(*this);
}

template <class T, class R>
bool Vector<T, R>::save(const VectorFileInterface<T, R>& vfi) const
{
  return vfi.save(*this);
}

template <class T, class R>
ostream& operator<<(ostream &s, const Vector<T, R>& v)
{
  unsigned int fieldw = s.width();
  unsigned int fieldp = s.precision();
  if (s.fill() == '\n')
    {
      s.fill(' ');
      for (unsigned int i = 0; i < v.vlength(); i++)
	s << setw(fieldw) << setprecision(fieldp) << numeric(v(i)) << endl;
    }
  else
    {
      for (unsigned int i = 0; i < v.vlength(); i++)
	s << setw(fieldw) << setprecision(fieldp) << numeric(v(i));
    }
  return s;
}

#endif
