// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: Vector.h,v 1.3 2005/09/05 20:31:14 brendt Exp $
  
*/

#ifndef _Vector_h
#define _Vector_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
#include <iomanip>
using std::ofstream;
using std::ifstream;
using std::endl;
using std::cerr;
using std::setw;
using std::setprecision;
#else
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#endif

#include "ProtoVector.h"
#include "Subvector.h"
#include "VectorSize.h"

#include "Image.h" //Required for row/col conversions

//! unit="Image and Vector Types"
//! module="Vector Support Classes"
//! level = 2

template <class T, class R>
class VectorRef;

template <class T, class R>
class VectorFileInterface;

template <class T, class R>
class Vector;

template <class T, class R>
inline VectorRef<T, R> operator+(const Vector<T, R>& u, const Vector<T, R>& v);

template <class T, class R>
inline VectorRef<T, R> operator+(const Vector<T, R>& u, T k);

template <class T, class R>
inline VectorRef<T, R> operator-(const Vector<T, R>& u, const Vector<T, R>& v);

template <class T, class R>
inline VectorRef<T, R> operator-(const Vector<T, R>& u, T k);

template <class T, class R>
inline VectorRef<T, R> operator-(T k, const Vector<T, R>& u);

template <class T, class R>
inline VectorRef<T, R> operator*(const Vector<T, R>& u, R k);

template <class T, class R>
inline VectorRef<T, R> operator*(R k, const Vector<T, R>& u);

template <class T, class R>
inline VectorRef<T, R> operator/(const Vector<T, R>& u, R k);

template <class T, class R>
inline VectorRef<T, R> abs(const Vector<T, R>& u);

template <class T, class R>
inline VectorRef<T, R> clip(const Vector<T, R>& u, T lower, T upper);

template <class T, class R>
ImageRef<T, R> rowimage(const Vector<T, R>& u);

template <class T, class R>
ImageRef<T, R> colimage(const Vector<T, R>& u);

template <class T, class R>
inline VectorRef<T, R> subsample(const Vector<T, R>& u, unsigned int k = 2);

template <class T, class R>
inline VectorRef<T, R> concatenate(const Vector<T, R>& u, const Vector<T, R>& v);

template <class T, class R>
ostream& operator<<(ostream &s, const Vector<T, R>& v);


//: Template class of Euclidean vectors
// Template vector class with parameters 
//T - vector element type eg. int
//R - return type for norms etc.
template <class T, class R>
class Vector : public ProtoVector<T, R, VectorSize> {
protected:
  T* get_pv_ptr(void) const;
  void set_pv_ptr(T* p) MTCONST;
  void set_size(const VectorSize& ssize);
  Vector(const VectorSize& init_size, T* init_ptr);
 
public:
  //Construct given vector length
  Vector(unsigned int n = 0);
  //Construct given vector length and element initialiser
  Vector(unsigned int n, T element);
  //Construct given vector size 
  Vector(const VectorSize& vs);
  //Construct given vector size and element initialiser
  Vector(const VectorSize& vs, T element);

  //Cloning copy constructor/type conversions (other types here as  well)
#ifdef NO_MEMBER_TEMPLATES
  Vector(const Vector<unsigned char, real>& ip);
  Vector(const Vector<unsigned short, real>& ip);
  Vector(const Vector<float, float>& ip);
#else
  Vector(const Vector<T, R>& ip);
  template <class P, class Q>
    Vector(const Vector<P, Q>& ip);
#endif  

  //Cloning assignment operator
  const Vector<T, R>& operator=(const Vector<T, R>& ip);


  //Reference passing copy constructor
  Vector(MTCONST VectorRef<T, R>& r);
  //Reference passing assignment operator
  Vector<T, R>& operator=(MTCONST VectorRef<T, R>& r);
  
  //Construct vector from subimage
  Vector(const Subimage<T, R>& subimage);


  //Construct vector from subvector
  Vector(const Subvector<T, R>& subvector);
  //Assign vector from subvector. Note that memory is re-allocated if
  //subvector size does not match existing  vector size
  Vector<T, R>& operator=(const Subvector<T, R>& subvector);
  //Extract subvector from vector
  Subvector<T, R> operator()(unsigned int x, unsigned int w) const;

  friend class Subvector<T, R>;
  

  //Destroy vector and reallocate vector of requested size
  void reset(const VectorSize& vs);
  //Return vector length
  unsigned int vlength() const { 
    return ProtoVector<T, R, VectorSize>::size().elements(); }
  //Return reference to element n of vector
  T& operator()(unsigned int n) const;
  // a b c --> a b c a b c
  T& modindex(int n) const;
  // a b c --> a b c b
  T& oddsymindex(int n) const;
  // a b c --> a b c c b a
  T& evensymindex(int n) const;


  //Arithmetic operations
  friend FRIEND_INLINE VectorRef<T, R> 
         TMPLT_FRND_SPCL(operator+)(const Vector<T, R>& u,
				    const Vector<T, R>& v);
  friend FRIEND_INLINE VectorRef<T, R> 
         TMPLT_FRND_SPCL(operator+)(const Vector<T, R>& u, T k);
  friend FRIEND_INLINE VectorRef<T, R> 
         TMPLT_FRND_SPCL(operator-)(const Vector<T, R>& u,
				   const Vector<T, R>& v);
  friend FRIEND_INLINE VectorRef<T, R> 
         TMPLT_FRND_SPCL(operator-)(const Vector<T, R>& u, T k);
  friend FRIEND_INLINE VectorRef<T, R> 
         TMPLT_FRND_SPCL(operator-)(T k, const Vector<T, R>& u);
  friend FRIEND_INLINE VectorRef<T, R> 
         TMPLT_FRND_SPCL(operator*)(const Vector<T, R>& u, R k);
  friend FRIEND_INLINE VectorRef<T, R> 
         TMPLT_FRND_SPCL(operator*)(R k, const Vector<T, R>& u);
  friend FRIEND_INLINE VectorRef<T, R> 
         TMPLT_FRND_SPCL(operator/)(const Vector<T, R>& u, R k);

#if defined(REQUIRE_UNSHADOW_KLUDGE)
  template<class K, class L>
  friend FRIEND_INLINE VectorRef<K, L> abs(const Vector<K, R>& u);
  template<class K, class L>
  friend FRIEND_INLINE VectorRef<K, L> 
         clip(const Vector<K, L>& u, K lower, K upper);
#else
  friend FRIEND_INLINE VectorRef<T, R> UNSHADOW(abs)(const Vector<T, R>& u);
  friend FRIEND_INLINE VectorRef<T, R> 
         UNSHADOW(clip)(const Vector<T, R>& u, T lower, T upper);
#endif

  //Compound assignments
  Vector<T, R>& operator+=(const Vector<T, R>& u);
  Vector<T, R>& operator+=(T k);
  Vector<T, R>& operator-=(const Vector<T, R>& u);
  Vector<T, R>& operator-=(T k);
  Vector<T, R>& operator*=(R k);
  Vector<T, R>& operator/=(R k);
 

  //Convert to image row/col
  friend ImageRef<T, R> TMPLT_FRND_SPCL(rowimage)(const Vector<T, R>& u);
  friend ImageRef<T, R> TMPLT_FRND_SPCL(colimage)(const Vector<T, R>& u);

  //Subsample vector u and place result in *this. Argument k is
  //subsampling ratio
  Vector<T, R>& subsample(const Vector<T, R>& u, unsigned int k = 2);
  //Concatenate two vectors, place result in *this
  Vector<T, R>& concatenate(const Vector<T, R>& u, const Vector<T, R>& v);
  //Integrate vector between bounds
  R integral(unsigned int first, unsigned int last);
  //Integrate entire vector
  R integral() { return integral(0, vlength()-1); }

#if defined(REQUIRE_UNSHADOW_KLUDGE)
  template<class K, class L>
  friend FRIEND_INLINE VectorRef<K, L> 
         subsample(const Vector<K, L>& u, unsigned int k);
  template<class K, class L>
  friend FRIEND_INLINE VectorRef<K, L> 
         concatenate(const Vector<K, L>& u, const Vector<K, L>& v);
#else
   //Return subsampled version of vector v
  friend FRIEND_INLINE VectorRef<T, R> 
         UNSHADOW(subsample)(const Vector<T, R>& u, unsigned int k);
  //Return concatenation of two vectors
  friend FRIEND_INLINE VectorRef<T, R> 
         UNSHADOW(concatenate)(const Vector<T, R>& u,
			       const Vector<T, R>& v);
#endif

  //Write file in native format
  bool native_write(char* filename) const;
  //Read file in native format
  bool native_read(char* filename);

  
  //Stream output
  friend ostream& TMPLT_FRND_SPCL(operator<< )(ostream &s, 
					      const Vector<T, R>& v);


  bool load(const VectorFileInterface<T, R>& vfi);
  bool save(const VectorFileInterface<T, R>& vfi) const;

  friend class VectorRef<T, R>;
};


template <class T, class R>
inline T* Vector<T, R>::get_pv_ptr(void) const
{
  return ProtoVector<T, R, VectorSize>::get_pv_ptr();
}

template <class T, class R>
inline void Vector<T, R>::set_pv_ptr(T* p) MTCONST
{
  ProtoVector<T, R, VectorSize>::set_pv_ptr(p);
}

template <class T, class R>
inline void Vector<T, R>::set_size(const VectorSize& ssize)
{
  ProtoVector<T, R, VectorSize>::set_size(ssize);
}

template <class T, class R>
inline Vector<T, R>::Vector(const VectorSize& init_size, T* init_ptr) :
  ProtoVector<T, R, VectorSize>(init_size, init_ptr) {}



template <class T, class R>
inline Vector<T, R>::Vector(unsigned int n) :
  ProtoVector<T, R, VectorSize>(VectorSize(n)) {}

template <class T, class R>
inline Vector<T, R>::Vector(unsigned int n, T element) :
  ProtoVector<T, R, VectorSize>(VectorSize(n), element) {}

template <class T, class R>
inline Vector<T, R>::Vector(const VectorSize& vs) :
  ProtoVector<T, R, VectorSize>(vs) {}

template <class T, class R>
inline Vector<T, R>::Vector(const VectorSize& vs, T element) :
  ProtoVector<T, R, VectorSize>(vs, element) {}

#ifdef NO_MEMBER_TEMPLATES
template <class T, class R>
inline Vector<T, R>::Vector(const Vector<unsigned char, real>& ip) :
  ProtoVector<T, R, VectorSize>(ip) {}

template <class T, class R>
inline Vector<T, R>::Vector(const Vector<unsigned short, real>& ip) :
  ProtoVector<T, R, VectorSize>(ip) {}

template <class T, class R>
inline Vector<T, R>::Vector(const Vector<float, float>& ip) :
  ProtoVector<T, R, VectorSize>(ip) {}
#else
template <class T, class R>
inline Vector<T, R>::Vector(const Vector<T, R>& ip) :
  ProtoVector<T, R, VectorSize>(ip) {}

template <class T, class R>
  template <class P, class Q>
inline Vector<T, R>::Vector(const Vector<P, Q>& ip) :
  ProtoVector<T, R, VectorSize>(ip) {}
#endif

template <class T, class R>
inline const Vector<T, R>& Vector<T, R>::operator=(const Vector<T, R>& ip)
{
  ProtoVector<T, R, VectorSize>::operator=(ip);
  return *this;
}



template <class T, class R>
inline Vector<T, R>::Vector(MTCONST VectorRef<T, R>& r) :
  ProtoVector<T, R, VectorSize>(r.size(), r.get_pv_ptr())
{
  r.set_pv_ptr(0);
}

template <class T, class R>
inline Vector<T, R>& Vector<T, R>::operator=(MTCONST VectorRef<T, R>& r) 
{
  ProtoVector<T, R, VectorSize>::destroy();
  set_size(r.size());
  set_pv_ptr(r.get_pv_ptr());
  r.set_pv_ptr(0);
  return *this;
}


template <class T, class R>
inline T& Vector<T, R>::operator()(unsigned int n) const
{
  ASSERT_WARNING( n < vlength() );
  return ProtoVector<T, R, VectorSize>::pv_elt(n);
}

template <class T, class R>
inline T& Vector<T, R>::modindex(int n) const
{
  return pv_elt(rnmod(n,vlength()));
}

template <class T, class R>
inline T& Vector<T, R>::oddsymindex(int n) const
{
  unsigned int idx = rnmod(n, 2*vlength()-2);
  if (idx >= vlength())
    idx = 2*vlength() - 2 - idx;
  return ProtoVector<T, R, VectorSize>::pv_elt(idx);
}

template <class T, class R>
inline T& Vector<T, R>::evensymindex(int n) const
{
  unsigned int idx = rnmod(n, 2*vlength());
  if (idx >= vlength())
    idx = 2*vlength() - 2 - idx + 1;
  return ProtoVector<T, R, VectorSize>::pv_elt(idx);
}

template <class T, class R>
inline VectorRef<T, R> operator+(const Vector<T, R>& u,
				 const Vector<T, R>& v)
{
  Vector<T, R> vecsum(u.vlength());
  vecsum.sum(u, v);
  return vecsum;
}

template <class T, class R>
inline VectorRef<T, R> operator+(const Vector<T, R>& u, T k)
{
  Vector<T, R> vecsum(u.vlength());
  vecsum.sum(u, k);
  return vecsum;
}

template <class T, class R>
inline VectorRef<T, R> operator-(const Vector<T, R>& u,
				 const Vector<T, R>& v)
{
  Vector<T, R> vecdif(u.vlength());
  vecdif.dif(u, v);
  return vecdif;
}

template <class T, class R>
inline VectorRef<T, R> operator-(const Vector<T, R>& u, T k)
{
  Vector<T, R> vecdif(u.vlength());
  vecdif.dif(u, k);
  return vecdif;
}

template <class T, class R>
inline VectorRef<T, R> operator-(T k, const Vector<T, R>& u)
{
  Vector<T, R> vecdif(u.vlength());
  vecdif.dif(k, u);
  return vecdif;
}

template <class T, class R>
inline VectorRef<T, R> operator*(const Vector<T, R>& u, R k)
{
  Vector<T, R> vecprod(u.vlength());
  vecprod.prod(u, k);
  return vecprod;
}

template <class T, class R>
inline VectorRef<T, R> operator*(R k, const Vector<T, R>& u)
{
  return u * k;
}

template <class T, class R>
inline VectorRef<T, R> operator/(const Vector<T, R>& u, R k)
{
  return u * (1.0f/k); 
}

template <class T, class R>
inline VectorRef<T, R> abs(const Vector<T, R>& u)
{
  Vector<T, R> vecabs(u.vlength());
  vecabs.abs(u);
  return vecabs;
}

template <class T, class R>
inline VectorRef<T, R> clip(const Vector<T, R>& u, T lower, T upper)
{
  Vector<T, R> vecclip(u.vlength());
  vecclip.clip(u, lower, upper);
  return vecclip;
}
  

template <class T, class R>
inline Vector<T, R>& Vector<T, R>::operator+=(const Vector<T, R>& u)
{
  sum(*this, u);
  return *this;
}

template <class T, class R>
inline Vector<T, R>& Vector<T, R>::operator+=(T k)
{
  sum(*this, k);
  return *this;
}

template <class T, class R>
inline Vector<T, R>& Vector<T, R>::operator-=(const Vector<T, R>& u)
{
  dif(*this, u);
  return *this;
}

template <class T, class R>
inline  Vector<T, R>& Vector<T, R>::operator-=(T k)
{
  dif(*this, k);
  return *this;
}

template <class T, class R>
inline Vector<T, R>& Vector<T, R>::operator*=(R k)
{
  prod(*this, k);
  return *this;
}

template <class T, class R>
inline  Vector<T, R>& Vector<T, R>::operator/=(R k)
{
  prod(*this, 1.0/k);
  return *this;
}

template <class T, class R>
inline VectorRef<T, R> subsample(const Vector<T, R>& u, unsigned int k)
{
  Vector<T, R> vecsubs(u.vlength()/k);
  vecsubs.subsample(u, k);
  return vecsubs;
}

template <class T, class R>
inline VectorRef<T, R> concatenate(const Vector<T, R>& u,
				   const Vector<T, R>& v)
{
  Vector<T, R> veccon(u.vlength() + v.vlength());
  veccon.concatenate(u, v);
  return veccon;
}


//: Template class transporting a reference to a Vector<T,R> 
template <class T, class R>
class VectorRef : public Vector<T, R> {
public:
  VectorRef(MTCONST Vector<T, R>& v) :
    Vector<T, R>(v.size(), v.get_pv_ptr())
  {
    v.set_pv_ptr(0);
  }

  friend class Vector<T, R>;
};


//: Template class interface to vector file mechanism
#include "VectorFile.h"
template <class T, class R>
class VectorFileInterface  {
public:
  virtual ~VectorFileInterface() {}
  virtual bool load(Vector<T, R>& im) const = 0;
  virtual bool save(const Vector<T, R>& im) const = 0;
};

#if defined(INCLUDE_TEMPLATE_DEFINITIONS)
#include "Vector.cc"
#endif

#endif
