// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

#include <math.h>

#include "ImageLib.h"
#include "GaussianSQ.h"


GaussianSQ::GaussianSQ(float imu, float isigma, const QRate& qr) :
  ScalarQuantiser(qr)
{
  reset_shape(imu, isigma);
  reset_rate(qr);
}

GaussianSQ::~GaussianSQ()
{
  delete [] xk;
}

void GaussianSQ::reset_shape(float imu, float isigma)
{
  mu = imu;
  sigma = isigma;
}


void GaussianSQ::reset_rate(const QRate& qr)
{
  uRate = qr.Rate();
  ASSERT_FATAL(uRate <= Rmax);
  if (uRate > 0)
    uRightLevels = 1 << (uRate-1);
  else
    uRightLevels = 1;
  xk = new float [uRightLevels];
  xk[0] = 0.0;
  unsigned int i;
  for (i = 1; i < uRightLevels; i++)
    xk[i] = (yk[uRate][i-1] + yk[uRate][i])/2.0;
}

int GaussianSQ::Index(float x) const
{
  float stdx = (x - mu)/sigma;
  float pstdx = fabs(stdx);
  unsigned int i = 1;
  while (pstdx > xk[i] && i < uRightLevels)
    i++;
  return (stdx < 0.0)?(uRightLevels-i):(i-1+uRightLevels);
}

float GaussianSQ::Reconstruction(unsigned int idx) const
{
  ASSERT_FATAL(idx < 2*uRightLevels);
  float sign = (idx < uRightLevels)?-1:+1;
  int rcnst = (sign < 0)?(uRightLevels - idx - 1):(idx - uRightLevels);
  return sigma*sign*yk[uRate][rcnst] + mu;
}

//These values as in S.P. LLoyd, 1982, Least Squares ....
const float GaussianSQ::yk0[] = { 0.0000 };
const float GaussianSQ::yk1[] = { 0.7979 };
const float GaussianSQ::yk2[] = { 0.4528, 1.5104 };
const float GaussianSQ::yk3[] = { 0.2451, 0.7560, 1.3439, 2.1520 };
const float GaussianSQ::yk4[] = { 0.1284, 0.3880, 0.6568, 0.9423,
				  1.2562, 1.6181, 2.0690, 2.7326 };
//Remaining values computed using OptQuantG - some inaccuracy
const float GaussianSQ::yk5[] = { 0.0659,  0.1981,  0.3314,  0.4667, 
				  0.6049,  0.7471,  0.8946,  1.0488, 
				  1.2118,  1.3864,  1.5762,  1.7872, 
				  2.0287,  2.3178,  2.6911,  3.2607 };
const float GaussianSQ::yk6[] = { 0.0334,  0.1003,  0.1673,  0.2346, 
				  0.3022,  0.3703,  0.4390,  0.5083, 
				  0.5785,  0.6497,  0.7220,  0.7955, 
				  0.8705,  0.9472,  1.0258,  1.1065, 
				  1.1898,  1.2758,  1.3652,  1.4583, 
				  1.5559,  1.6586,  1.7676,  1.8840, 
				  2.0097,  2.1469,  2.2990,  2.4714, 
				  2.6723,  2.9175,  3.2405,  3.7441 };
const float GaussianSQ::yk7[] = { 0.0172,  0.0516,  0.0861,  0.1205, 
				  0.1550,  0.1896,  0.2242,  0.2589, 
				  0.2937,  0.3286,  0.3636,  0.3988, 
				  0.4341,  0.4695,  0.5051,  0.5410, 
				  0.5770,  0.6132,  0.6497,  0.6865, 
				  0.7235,  0.7608,  0.7984,  0.8364, 
				  0.8748,  0.9135,  0.9527,  0.9923, 
				  1.0325,  1.0731,  1.1143,  1.1560, 
				  1.1985,  1.2416,  1.2854,  1.3300, 
				  1.3755,  1.4219,  1.4692,  1.5177, 
				  1.5673,  1.6182,  1.6705,  1.7243, 
				  1.7797,  1.8370,  1.8963,  1.9579, 
				  2.0220,  2.0889,  2.1591,  2.2329, 
				  2.3110,  2.3940,  2.4829,  2.5788, 
				  2.6832,  2.7984,  2.9275,  3.0751, 
				  3.2492,  3.4639,  3.7504,  4.2038 };
//Re-run this one!
const float GaussianSQ::yk8[] = { 0.0085,  0.0254,  0.0424,  0.0594, 
				  0.0763,  0.0933,  0.1103,  0.1273, 
				  0.1443,  0.1613,  0.1784,  0.1954, 
				  0.2125,  0.2296,  0.2467,  0.2639, 
				  0.2810,  0.2982,  0.3155,  0.3327, 
				  0.3500,  0.3673,  0.3847,  0.4021, 
				  0.4195,  0.4370,  0.4545,  0.4721, 
				  0.4897,  0.5074,  0.5251,  0.5429, 
				  0.5608,  0.5787,  0.5966,  0.6146, 
				  0.6327,  0.6509,  0.6691,  0.6874, 
				  0.7058,  0.7242,  0.7428,  0.7614, 
				  0.7801,  0.7989,  0.8178,  0.8368, 
				  0.8559,  0.8751,  0.8944,  0.9138, 
				  0.9333,  0.9530,  0.9727,  0.9926, 
				  1.0127,  1.0328,  1.0531,  1.0736, 
				  1.0941,  1.1149,  1.1358,  1.1569, 
				  1.1781,  1.1995,  1.2211,  1.2429, 
				  1.2649,  1.2870,  1.3094,  1.3320, 
				  1.3549,  1.3779,  1.4013,  1.4248, 
				  1.4487,  1.4728,  1.4972,  1.5218, 
				  1.5468,  1.5721,  1.5978,  1.6238, 
				  1.6502,  1.6769,  1.7041,  1.7316, 
				  1.7597,  1.7881,  1.8171,  1.8466, 
				  1.8766,  1.9072,  1.9383,  1.9702, 
				  2.0027,  2.0359,  2.0698,  2.1046, 
				  2.1402,  2.1768,  2.2143,  2.2529, 
				  2.2927,  2.3337,  2.3760,  2.4198, 
				  2.4652,  2.5124,  2.5615,  2.6127, 
				  2.6663,  2.7226,  2.7819,  2.8446, 
				  2.9114,  2.9827,  3.0595,  3.1428, 
				  3.2341,  3.3353,  3.4494,  3.5808, 
				  3.7367,  3.9305,  4.1912,  4.6078 };
const float GaussianSQ::yk9[] = { 0.0042,  0.0127,  0.0212,  0.0297, 
				  0.0382,  0.0466,  0.0551,  0.0636, 
				  0.0721,  0.0806,  0.0891,  0.0976, 
				  0.1061,  0.1146,  0.1231,  0.1316, 
				  0.1401,  0.1486,  0.1571,  0.1656, 
				  0.1741,  0.1827,  0.1912,  0.1997, 
				  0.2083,  0.2168,  0.2253,  0.2339, 
				  0.2425,  0.2510,  0.2596,  0.2682, 
				  0.2768,  0.2854,  0.2940,  0.3026, 
				  0.3112,  0.3198,  0.3284,  0.3371, 
				  0.3457,  0.3544,  0.3630,  0.3717, 
				  0.3804,  0.3891,  0.3978,  0.4065, 
				  0.4152,  0.4239,  0.4327,  0.4414, 
				  0.4502,  0.4590,  0.4678,  0.4766, 
				  0.4854,  0.4942,  0.5030,  0.5119, 
				  0.5208,  0.5296,  0.5385,  0.5474, 
				  0.5564,  0.5653,  0.5742,  0.5832, 
				  0.5922,  0.6012,  0.6102,  0.6192, 
				  0.6283,  0.6374,  0.6464,  0.6555, 
				  0.6647,  0.6738,  0.6829,  0.6921, 
				  0.7013,  0.7105,  0.7198,  0.7290, 
				  0.7383,  0.7476,  0.7569,  0.7663, 
				  0.7756,  0.7850,  0.7944,  0.8038, 
				  0.8133,  0.8228,  0.8323,  0.8418, 
				  0.8514,  0.8609,  0.8706,  0.8802, 
				  0.8899,  0.8995,  0.9093,  0.9190, 
				  0.9288,  0.9386,  0.9484,  0.9583, 
				  0.9682,  0.9781,  0.9881,  0.9981, 
				  1.0081,  1.0182,  1.0283,  1.0384, 
				  1.0486,  1.0588,  1.0690,  1.0793, 
				  1.0896,  1.1000,  1.1103,  1.1208, 
				  1.1313,  1.1418,  1.1523,  1.1629, 
				  1.1736,  1.1843,  1.1950,  1.2058, 
				  1.2166,  1.2275,  1.2384,  1.2494, 
				  1.2604,  1.2715,  1.2826,  1.2938, 
				  1.3050,  1.3163,  1.3277,  1.3391, 
				  1.3505,  1.3621,  1.3736,  1.3853, 
				  1.3970,  1.4088,  1.4206,  1.4325, 
				  1.4445,  1.4565,  1.4686,  1.4808, 
				  1.4931,  1.5054,  1.5178,  1.5303, 
				  1.5429,  1.5555,  1.5683,  1.5811, 
				  1.5940,  1.6070,  1.6201,  1.6332, 
				  1.6465,  1.6599,  1.6733,  1.6869, 
				  1.7006,  1.7144,  1.7283,  1.7423, 
				  1.7564,  1.7706,  1.7850,  1.7995, 
				  1.8141,  1.8288,  1.8437,  1.8587, 
				  1.8738,  1.8891,  1.9045,  1.9201, 
				  1.9358,  1.9517,  1.9678,  1.9841, 
				  2.0005,  2.0171,  2.0338,  2.0508, 
				  2.0680,  2.0853,  2.1029,  2.1207, 
				  2.1387,  2.1570,  2.1755,  2.1942, 
				  2.2132,  2.2325,  2.2520,  2.2718, 
				  2.2920,  2.3124,  2.3332,  2.3542, 
				  2.3757,  2.3975,  2.4197,  2.4423, 
				  2.4653,  2.4887,  2.5126,  2.5370, 
				  2.5619,  2.5873,  2.6133,  2.6399, 
				  2.6671,  2.6950,  2.7236,  2.7529, 
				  2.7830,  2.8140,  2.8460,  2.8789, 
				  2.9128,  2.9479,  2.9843,  3.0220, 
				  3.0612,  3.1020,  3.1446,  3.1892, 
				  3.2360,  3.2852,  3.3373,  3.3926, 
				  3.4515,  3.5147,  3.5829,  3.6572, 
				  3.7389,  3.8299,  3.9328,  4.0518, 
				  4.1937,  4.3710,  4.6110,  4.9972};
const float GaussianSQ::yk10[] = { 0.0021,  0.0064,  0.0106,  0.0148, 
				   0.0191,  0.0233,  0.0276,  0.0318, 
				   0.0360,  0.0403,  0.0445,  0.0488, 
				   0.0530,  0.0572,  0.0615,  0.0657, 
				   0.0700,  0.0742,  0.0785,  0.0827, 
				   0.0870,  0.0912,  0.0954,  0.0997, 
				   0.1039,  0.1082,  0.1124,  0.1167, 
				   0.1209,  0.1252,  0.1294,  0.1337, 
				   0.1379,  0.1422,  0.1464,  0.1507, 
				   0.1550,  0.1592,  0.1635,  0.1677, 
				   0.1720,  0.1763,  0.1805,  0.1848, 
				   0.1890,  0.1933,  0.1976,  0.2018, 
				   0.2061,  0.2104,  0.2147,  0.2189, 
				   0.2232,  0.2275,  0.2318,  0.2360, 
				   0.2403,  0.2446,  0.2489,  0.2532, 
				   0.2575,  0.2617,  0.2660,  0.2703, 
				   0.2746,  0.2789,  0.2832,  0.2875, 
				   0.2918,  0.2961,  0.3004,  0.3047, 
				   0.3090,  0.3133,  0.3176,  0.3220, 
				   0.3263,  0.3306,  0.3349,  0.3392, 
				   0.3435,  0.3479,  0.3522,  0.3565, 
				   0.3609,  0.3652,  0.3695,  0.3739, 
				   0.3782,  0.3826,  0.3869,  0.3912, 
				   0.3956,  0.4000,  0.4043,  0.4087, 
				   0.4130,  0.4174,  0.4218,  0.4261, 
				   0.4305,  0.4349,  0.4392,  0.4436, 
				   0.4480,  0.4524,  0.4568,  0.4612, 
				   0.4656,  0.4700,  0.4744,  0.4788, 
				   0.4832,  0.4876,  0.4920,  0.4964, 
				   0.5008,  0.5053,  0.5097,  0.5141, 
				   0.5185,  0.5230,  0.5274,  0.5319, 
				   0.5363,  0.5408,  0.5452,  0.5497, 
				   0.5541,  0.5586,  0.5631,  0.5675, 
				   0.5720,  0.5765,  0.5810,  0.5855, 
				   0.5900,  0.5944,  0.5989,  0.6034, 
				   0.6080,  0.6125,  0.6170,  0.6215, 
				   0.6260,  0.6306,  0.6351,  0.6396, 
				   0.6442,  0.6487,  0.6533,  0.6578, 
				   0.6624,  0.6669,  0.6715,  0.6761, 
				   0.6807,  0.6852,  0.6898,  0.6944, 
				   0.6990,  0.7036,  0.7082,  0.7128, 
				   0.7175,  0.7221,  0.7267,  0.7313, 
				   0.7360,  0.7406,  0.7453,  0.7499, 
				   0.7546,  0.7592,  0.7639,  0.7686, 
				   0.7733,  0.7780,  0.7827,  0.7873, 
				   0.7921,  0.7968,  0.8015,  0.8062, 
				   0.8109,  0.8157,  0.8204,  0.8251, 
				   0.8299,  0.8347,  0.8394,  0.8442, 
				   0.8490,  0.8538,  0.8586,  0.8633, 
				   0.8682,  0.8730,  0.8778,  0.8826, 
				   0.8874,  0.8923,  0.8971,  0.9020, 
				   0.9068,  0.9117,  0.9166,  0.9214, 
				   0.9263,  0.9312,  0.9361,  0.9410, 
				   0.9460,  0.9509,  0.9558,  0.9608, 
				   0.9657,  0.9707,  0.9756,  0.9806, 
				   0.9856,  0.9906,  0.9956,  1.0006, 
				   1.0056,  1.0106,  1.0156,  1.0207, 
				   1.0257,  1.0308,  1.0359,  1.0409, 
				   1.0460,  1.0511,  1.0562,  1.0613, 
				   1.0664,  1.0716,  1.0767,  1.0819, 
				   1.0870,  1.0922,  1.0974,  1.1025, 
				   1.1077,  1.1130,  1.1182,  1.1234, 
				   1.1286,  1.1339,  1.1391,  1.1444, 
				   1.1497,  1.1550,  1.1603,  1.1656, 
				   1.1709,  1.1762,  1.1816,  1.1869, 
				   1.1923,  1.1977,  1.2031,  1.2085, 
				   1.2139,  1.2193,  1.2248,  1.2302, 
				   1.2357,  1.2412,  1.2466,  1.2521, 
				   1.2577,  1.2632,  1.2687,  1.2743, 
				   1.2798,  1.2854,  1.2910,  1.2966, 
				   1.3022,  1.3079,  1.3135,  1.3192, 
				   1.3248,  1.3305,  1.3362,  1.3419, 
				   1.3477,  1.3534,  1.3592,  1.3650, 
				   1.3707,  1.3766,  1.3824,  1.3882, 
				   1.3941,  1.3999,  1.4058,  1.4117, 
				   1.4176,  1.4236,  1.4295,  1.4355, 
				   1.4415,  1.4475,  1.4535,  1.4595, 
				   1.4656,  1.4717,  1.4778,  1.4839, 
				   1.4900,  1.4962,  1.5023,  1.5085, 
				   1.5147,  1.5209,  1.5272,  1.5334, 
				   1.5397,  1.5460,  1.5524,  1.5587, 
				   1.5651,  1.5715,  1.5779,  1.5843, 
				   1.5908,  1.5972,  1.6037,  1.6102, 
				   1.6168,  1.6234,  1.6299,  1.6366, 
				   1.6432,  1.6499,  1.6565,  1.6633, 
				   1.6700,  1.6768,  1.6835,  1.6904, 
				   1.6972,  1.7041,  1.7109,  1.7179, 
				   1.7248,  1.7318,  1.7388,  1.7458, 
				   1.7529,  1.7600,  1.7671,  1.7742, 
				   1.7814,  1.7886,  1.7959,  1.8032, 
				   1.8105,  1.8178,  1.8252,  1.8326, 
				   1.8400,  1.8475,  1.8550,  1.8625, 
				   1.8701,  1.8777,  1.8854,  1.8931, 
				   1.9008,  1.9085,  1.9163,  1.9242, 
				   1.9321,  1.9400,  1.9479,  1.9559, 
				   1.9640,  1.9721,  1.9802,  1.9884, 
				   1.9966,  2.0048,  2.0131,  2.0215, 
				   2.0299,  2.0384,  2.0469,  2.0554, 
				   2.0640,  2.0726,  2.0814,  2.0901, 
				   2.0989,  2.1078,  2.1167,  2.1257, 
				   2.1347,  2.1438,  2.1529,  2.1622, 
				   2.1714,  2.1808,  2.1902,  2.1996, 
				   2.2092,  2.2187,  2.2284,  2.2382, 
				   2.2480,  2.2578,  2.2678,  2.2778, 
				   2.2879,  2.2981,  2.3084,  2.3187, 
				   2.3291,  2.3397,  2.3503,  2.3610, 
				   2.3717,  2.3826,  2.3936,  2.4046, 
				   2.4158,  2.4271,  2.4384,  2.4499, 
				   2.4615,  2.4732,  2.4850,  2.4969, 
				   2.5090,  2.5211,  2.5334,  2.5458, 
				   2.5584,  2.5711,  2.5839,  2.5969, 
				   2.6100,  2.6233,  2.6367,  2.6503, 
				   2.6640,  2.6779,  2.6920,  2.7063, 
				   2.7207,  2.7354,  2.7502,  2.7653, 
				   2.7805,  2.7960,  2.8117,  2.8276, 
				   2.8438,  2.8602,  2.8769,  2.8938, 
				   2.9111,  2.9286,  2.9464,  2.9645, 
				   2.9830,  3.0018,  3.0209,  3.0404, 
				   3.0603,  3.0807,  3.1014,  3.1226, 
				   3.1442,  3.1664,  3.1891,  3.2123, 
				   3.2361,  3.2605,  3.2856,  3.3114, 
				   3.3379,  3.3653,  3.3934,  3.4225, 
				   3.4526,  3.4837,  3.5160,  3.5496, 
				   3.5845,  3.6209,  3.6590,  3.6989, 
				   3.7409,  3.7851,  3.8320,  3.8819, 
				   3.9351,  3.9924,  4.0543,  4.1219, 
				   4.1964,  4.2796,  4.3740,  4.4835, 
				   4.6144,  4.7786,  5.0019,  5.3633};
const float GaussianSQ::yk11[] = { 0.0011,  0.0032,  0.0053,  0.0074, 
				   0.0095,  0.0117,  0.0138,  0.0159, 
				   0.0180,  0.0201,  0.0223,  0.0244, 
				   0.0265,  0.0286,  0.0307,  0.0329, 
				   0.0350,  0.0371,  0.0392,  0.0413, 
				   0.0435,  0.0456,  0.0477,  0.0498, 
				   0.0519,  0.0541,  0.0562,  0.0583, 
				   0.0604,  0.0626,  0.0647,  0.0668, 
				   0.0689,  0.0710,  0.0732,  0.0753, 
				   0.0774,  0.0795,  0.0816,  0.0838, 
				   0.0859,  0.0880,  0.0901,  0.0923, 
				   0.0944,  0.0965,  0.0986,  0.1008, 
				   0.1029,  0.1050,  0.1071,  0.1092, 
				   0.1114,  0.1135,  0.1156,  0.1177, 
				   0.1199,  0.1220,  0.1241,  0.1262, 
				   0.1284,  0.1305,  0.1326,  0.1348, 
				   0.1369,  0.1390,  0.1411,  0.1433, 
				   0.1454,  0.1475,  0.1496,  0.1518, 
				   0.1539,  0.1560,  0.1582,  0.1603, 
				   0.1624,  0.1645,  0.1667,  0.1688, 
				   0.1709,  0.1731,  0.1752,  0.1773, 
				   0.1795,  0.1816,  0.1837,  0.1858, 
				   0.1880,  0.1901,  0.1922,  0.1944, 
				   0.1965,  0.1986,  0.2008,  0.2029, 
				   0.2051,  0.2072,  0.2093,  0.2115, 
				   0.2136,  0.2157,  0.2179,  0.2200, 
				   0.2221,  0.2243,  0.2264,  0.2286, 
				   0.2307,  0.2328,  0.2350,  0.2371, 
				   0.2393,  0.2414,  0.2435,  0.2457, 
				   0.2478,  0.2500,  0.2521,  0.2542, 
				   0.2564,  0.2585,  0.2607,  0.2628, 
				   0.2650,  0.2671,  0.2693,  0.2714, 
				   0.2735,  0.2757,  0.2778,  0.2800, 
				   0.2821,  0.2843,  0.2864,  0.2886, 
				   0.2907,  0.2929,  0.2950,  0.2972, 
				   0.2993,  0.3015,  0.3036,  0.3058, 
				   0.3079,  0.3101,  0.3123,  0.3144, 
				   0.3166,  0.3187,  0.3209,  0.3230, 
				   0.3252,  0.3273,  0.3295,  0.3317, 
				   0.3338,  0.3360,  0.3381,  0.3403, 
				   0.3425,  0.3446,  0.3468,  0.3490, 
				   0.3511,  0.3533,  0.3554,  0.3576, 
				   0.3598,  0.3619,  0.3641,  0.3663, 
				   0.3684,  0.3706,  0.3728,  0.3750, 
				   0.3771,  0.3793,  0.3815,  0.3836, 
				   0.3858,  0.3880,  0.3902,  0.3923, 
				   0.3945,  0.3967,  0.3989,  0.4010, 
				   0.4032,  0.4054,  0.4076,  0.4098, 
				   0.4119,  0.4141,  0.4163,  0.4185, 
				   0.4207,  0.4228,  0.4250,  0.4272, 
				   0.4294,  0.4316,  0.4338,  0.4360, 
				   0.4382,  0.4403,  0.4425,  0.4447, 
				   0.4469,  0.4491,  0.4513,  0.4535, 
				   0.4557,  0.4579,  0.4601,  0.4623, 
				   0.4645,  0.4667,  0.4689,  0.4711, 
				   0.4733,  0.4755,  0.4777,  0.4799, 
				   0.4821,  0.4843,  0.4865,  0.4887, 
				   0.4909,  0.4931,  0.4953,  0.4975, 
				   0.4997,  0.5019,  0.5042,  0.5064, 
				   0.5086,  0.5108,  0.5130,  0.5152, 
				   0.5174,  0.5197,  0.5219,  0.5241, 
				   0.5263,  0.5285,  0.5308,  0.5330, 
				   0.5352,  0.5374,  0.5396,  0.5419, 
				   0.5441,  0.5463,  0.5486,  0.5508, 
				   0.5530,  0.5552,  0.5575,  0.5597, 
				   0.5619,  0.5642,  0.5664,  0.5687, 
				   0.5709,  0.5731,  0.5754,  0.5776, 
				   0.5799,  0.5821,  0.5843,  0.5866, 
				   0.5888,  0.5911,  0.5933,  0.5956, 
				   0.5978,  0.6001,  0.6023,  0.6046, 
				   0.6068,  0.6091,  0.6113,  0.6136, 
				   0.6159,  0.6181,  0.6204,  0.6226, 
				   0.6249,  0.6272,  0.6294,  0.6317, 
				   0.6340,  0.6362,  0.6385,  0.6408, 
				   0.6430,  0.6453,  0.6476,  0.6498, 
				   0.6521,  0.6544,  0.6567,  0.6590, 
				   0.6612,  0.6635,  0.6658,  0.6681, 
				   0.6704,  0.6726,  0.6749,  0.6772, 
				   0.6795,  0.6818,  0.6841,  0.6864, 
				   0.6887,  0.6910,  0.6933,  0.6956, 
				   0.6979,  0.7002,  0.7025,  0.7048, 
				   0.7071,  0.7094,  0.7117,  0.7140, 
				   0.7163,  0.7186,  0.7209,  0.7232, 
				   0.7255,  0.7279,  0.7302,  0.7325, 
				   0.7348,  0.7371,  0.7395,  0.7418, 
				   0.7441,  0.7464,  0.7488,  0.7511, 
				   0.7534,  0.7557,  0.7581,  0.7604, 
				   0.7627,  0.7651,  0.7674,  0.7698, 
				   0.7721,  0.7744,  0.7768,  0.7791, 
				   0.7815,  0.7838,  0.7862,  0.7885, 
				   0.7909,  0.7932,  0.7956,  0.7979, 
				   0.8003,  0.8027,  0.8050,  0.8074, 
				   0.8097,  0.8121,  0.8145,  0.8168, 
				   0.8192,  0.8216,  0.8240,  0.8263, 
				   0.8287,  0.8311,  0.8335,  0.8359, 
				   0.8382,  0.8406,  0.8430,  0.8454, 
				   0.8478,  0.8502,  0.8526,  0.8550, 
				   0.8574,  0.8597,  0.8621,  0.8645, 
				   0.8670,  0.8694,  0.8718,  0.8742, 
				   0.8766,  0.8790,  0.8814,  0.8838, 
				   0.8862,  0.8886,  0.8911,  0.8935, 
				   0.8959,  0.8983,  0.9008,  0.9032, 
				   0.9056,  0.9080,  0.9105,  0.9129, 
				   0.9153,  0.9178,  0.9202,  0.9227, 
				   0.9251,  0.9276,  0.9300,  0.9325, 
				   0.9349,  0.9374,  0.9398,  0.9423, 
				   0.9447,  0.9472,  0.9497,  0.9521, 
				   0.9546,  0.9571,  0.9595,  0.9620, 
				   0.9645,  0.9670,  0.9694,  0.9719, 
				   0.9744,  0.9769,  0.9794,  0.9819, 
				   0.9843,  0.9868,  0.9893,  0.9918, 
				   0.9943,  0.9968,  0.9993,  1.0018, 
				   1.0043,  1.0068,  1.0094,  1.0119, 
				   1.0144,  1.0169,  1.0194,  1.0219, 
				   1.0245,  1.0270,  1.0295,  1.0321, 
				   1.0346,  1.0371,  1.0397,  1.0422, 
				   1.0447,  1.0473,  1.0498,  1.0524, 
				   1.0549,  1.0575,  1.0600,  1.0626, 
				   1.0652,  1.0677,  1.0703,  1.0729, 
				   1.0754,  1.0780,  1.0806,  1.0831, 
				   1.0857,  1.0883,  1.0909,  1.0935, 
				   1.0961,  1.0987,  1.1012,  1.1038, 
				   1.1064,  1.1090,  1.1116,  1.1143, 
				   1.1169,  1.1195,  1.1221,  1.1247, 
				   1.1273,  1.1299,  1.1326,  1.1352, 
				   1.1378,  1.1405,  1.1431,  1.1457, 
				   1.1484,  1.1510,  1.1537,  1.1563, 
				   1.1590,  1.1616,  1.1643,  1.1669, 
				   1.1696,  1.1722,  1.1749,  1.1776, 
				   1.1803,  1.1829,  1.1856,  1.1883, 
				   1.1910,  1.1937,  1.1963,  1.1990, 
				   1.2017,  1.2044,  1.2071,  1.2098, 
				   1.2125,  1.2153,  1.2180,  1.2207, 
				   1.2234,  1.2261,  1.2289,  1.2316, 
				   1.2343,  1.2370,  1.2398,  1.2425, 
				   1.2453,  1.2480,  1.2508,  1.2535, 
				   1.2563,  1.2590,  1.2618,  1.2646, 
				   1.2673,  1.2701,  1.2729,  1.2757, 
				   1.2784,  1.2812,  1.2840,  1.2868, 
				   1.2896,  1.2924,  1.2952,  1.2980, 
				   1.3008,  1.3036,  1.3064,  1.3093, 
				   1.3121,  1.3149,  1.3177,  1.3206, 
				   1.3234,  1.3263,  1.3291,  1.3319, 
				   1.3348,  1.3376,  1.3405,  1.3434, 
				   1.3462,  1.3491,  1.3520,  1.3549, 
				   1.3577,  1.3606,  1.3635,  1.3664, 
				   1.3693,  1.3722,  1.3751,  1.3780, 
				   1.3809,  1.3838,  1.3867,  1.3897, 
				   1.3926,  1.3955,  1.3985,  1.4014, 
				   1.4043,  1.4073,  1.4102,  1.4132, 
				   1.4162,  1.4191,  1.4221,  1.4251, 
				   1.4280,  1.4310,  1.4340,  1.4370, 
				   1.4400,  1.4430,  1.4460,  1.4490, 
				   1.4520,  1.4550,  1.4580,  1.4611, 
				   1.4641,  1.4671,  1.4702,  1.4732, 
				   1.4762,  1.4793,  1.4823,  1.4854, 
				   1.4885,  1.4915,  1.4946,  1.4977, 
				   1.5008,  1.5039,  1.5070,  1.5101, 
				   1.5132,  1.5163,  1.5194,  1.5225, 
				   1.5256,  1.5287,  1.5319,  1.5350, 
				   1.5382,  1.5413,  1.5445,  1.5476, 
				   1.5508,  1.5539,  1.5571,  1.5603, 
				   1.5635,  1.5667,  1.5699,  1.5731, 
				   1.5763,  1.5795,  1.5827,  1.5859, 
				   1.5891,  1.5924,  1.5956,  1.5988, 
				   1.6021,  1.6054,  1.6086,  1.6119, 
				   1.6152,  1.6184,  1.6217,  1.6250, 
				   1.6283,  1.6316,  1.6349,  1.6382, 
				   1.6415,  1.6449,  1.6482,  1.6515, 
				   1.6549,  1.6582,  1.6616,  1.6649, 
				   1.6683,  1.6717,  1.6751,  1.6784, 
				   1.6818,  1.6852,  1.6886,  1.6921, 
				   1.6955,  1.6989,  1.7023,  1.7058, 
				   1.7092,  1.7127,  1.7161,  1.7196, 
				   1.7231,  1.7266,  1.7300,  1.7335, 
				   1.7370,  1.7406,  1.7441,  1.7476, 
				   1.7511,  1.7547,  1.7582,  1.7618, 
				   1.7653,  1.7689,  1.7725,  1.7760, 
				   1.7796,  1.7832,  1.7868,  1.7904, 
				   1.7941,  1.7977,  1.8013,  1.8050, 
				   1.8086,  1.8123,  1.8160,  1.8196, 
				   1.8233,  1.8270,  1.8307,  1.8344, 
				   1.8381,  1.8419,  1.8456,  1.8494, 
				   1.8531,  1.8569,  1.8606,  1.8644, 
				   1.8682,  1.8720,  1.8758,  1.8796, 
				   1.8835,  1.8873,  1.8911,  1.8950, 
				   1.8988,  1.9027,  1.9066,  1.9105, 
				   1.9144,  1.9183,  1.9222,  1.9261, 
				   1.9301,  1.9340,  1.9380,  1.9420, 
				   1.9459,  1.9499,  1.9539,  1.9579, 
				   1.9620,  1.9660,  1.9700,  1.9741, 
				   1.9782,  1.9822,  1.9863,  1.9904, 
				   1.9945,  1.9986,  2.0028,  2.0069, 
				   2.0111,  2.0152,  2.0194,  2.0236, 
				   2.0278,  2.0320,  2.0362,  2.0405, 
				   2.0447,  2.0490,  2.0533,  2.0575, 
				   2.0618,  2.0662,  2.0705,  2.0748, 
				   2.0792,  2.0835,  2.0879,  2.0923, 
				   2.0967,  2.1011,  2.1056,  2.1100, 
				   2.1145,  2.1189,  2.1234,  2.1279, 
				   2.1324,  2.1370,  2.1415,  2.1461, 
				   2.1506,  2.1552,  2.1598,  2.1645, 
				   2.1691,  2.1738,  2.1784,  2.1831, 
				   2.1878,  2.1925,  2.1973,  2.2020, 
				   2.2068,  2.2115,  2.2163,  2.2212, 
				   2.2260,  2.2308,  2.2357,  2.2406, 
				   2.2455,  2.2504,  2.2554,  2.2603, 
				   2.2653,  2.2703,  2.2753,  2.2803, 
				   2.2854,  2.2905,  2.2956,  2.3007, 
				   2.3058,  2.3110,  2.3161,  2.3213, 
				   2.3265,  2.3318,  2.3370,  2.3423, 
				   2.3476,  2.3529,  2.3583,  2.3636, 
				   2.3690,  2.3744,  2.3799,  2.3853, 
				   2.3908,  2.3963,  2.4019,  2.4074, 
				   2.4130,  2.4186,  2.4242,  2.4299, 
				   2.4356,  2.4413,  2.4470,  2.4528, 
				   2.4586,  2.4644,  2.4703,  2.4761, 
				   2.4820,  2.4880,  2.4939,  2.4999, 
				   2.5060,  2.5120,  2.5181,  2.5242, 
				   2.5304,  2.5365,  2.5427,  2.5490, 
				   2.5553,  2.5616,  2.5679,  2.5743, 
				   2.5807,  2.5872,  2.5937,  2.6002, 
				   2.6067,  2.6133,  2.6200,  2.6266, 
				   2.6334,  2.6401,  2.6469,  2.6537, 
				   2.6606,  2.6675,  2.6745,  2.6815, 
				   2.6886,  2.6956,  2.7028,  2.7100, 
				   2.7172,  2.7245,  2.7318,  2.7392, 
				   2.7466,  2.7541,  2.7616,  2.7692, 
				   2.7769,  2.7845,  2.7923,  2.8001, 
				   2.8080,  2.8159,  2.8239,  2.8319, 
				   2.8400,  2.8482,  2.8564,  2.8647, 
				   2.8730,  2.8814,  2.8899,  2.8985, 
				   2.9071,  2.9158,  2.9246,  2.9335, 
				   2.9424,  2.9514,  2.9605,  2.9697, 
				   2.9790,  2.9883,  2.9977,  3.0073, 
				   3.0169,  3.0266,  3.0364,  3.0463, 
				   3.0563,  3.0664,  3.0766,  3.0870, 
				   3.0974,  3.1079,  3.1186,  3.1294, 
				   3.1403,  3.1513,  3.1625,  3.1738, 
				   3.1852,  3.1968,  3.2085,  3.2203, 
				   3.2324,  3.2445,  3.2569,  3.2694, 
				   3.2820,  3.2949,  3.3079,  3.3212, 
				   3.3346,  3.3482,  3.3620,  3.3761, 
				   3.3904,  3.4049,  3.4196,  3.4346, 
				   3.4499,  3.4654,  3.4812,  3.4973, 
				   3.5137,  3.5304,  3.5475,  3.5649, 
				   3.5826,  3.6008,  3.6193,  3.6383, 
				   3.6577,  3.6776,  3.6979,  3.7188, 
				   3.7402,  3.7622,  3.7848,  3.8080, 
				   3.8320,  3.8567,  3.8821,  3.9085, 
				   3.9357,  3.9640,  3.9933,  4.0238, 
				   4.0556,  4.0887,  4.1235,  4.1599, 
				   4.1983,  4.2388,  4.2818,  4.3275, 
				   4.3764,  4.4291,  4.4862,  4.5486, 
				   4.6175,  4.6945,  4.7821,  4.8839, 
				   5.0060,  5.1596,  5.3691,  5.7098 };
const float GaussianSQ::yk12[] = { 0.0005,  0.0016,  0.0026,  0.0037, 
				   0.0048,  0.0058,  0.0069,  0.0079, 
				   0.0090,  0.0101,  0.0111,  0.0122, 
				   0.0132,  0.0143,  0.0154,  0.0164, 
				   0.0175,  0.0185,  0.0196,  0.0207, 
				   0.0217,  0.0228,  0.0238,  0.0249, 
				   0.0260,  0.0270,  0.0281,  0.0292, 
				   0.0302,  0.0313,  0.0323,  0.0334, 
				   0.0345,  0.0355,  0.0366,  0.0376, 
				   0.0387,  0.0398,  0.0408,  0.0419, 
				   0.0429,  0.0440,  0.0451,  0.0461, 
				   0.0472,  0.0482,  0.0493,  0.0504, 
				   0.0514,  0.0525,  0.0535,  0.0546, 
				   0.0557,  0.0567,  0.0578,  0.0588, 
				   0.0599,  0.0610,  0.0620,  0.0631, 
				   0.0641,  0.0652,  0.0663,  0.0673, 
				   0.0684,  0.0694,  0.0705,  0.0716, 
				   0.0726,  0.0737,  0.0748,  0.0758, 
				   0.0769,  0.0779,  0.0790,  0.0801, 
				   0.0811,  0.0822,  0.0832,  0.0843, 
				   0.0854,  0.0864,  0.0875,  0.0885, 
				   0.0896,  0.0907,  0.0917,  0.0928, 
				   0.0939,  0.0949,  0.0960,  0.0970, 
				   0.0981,  0.0992,  0.1002,  0.1013, 
				   0.1023,  0.1034,  0.1045,  0.1055, 
				   0.1066,  0.1077,  0.1087,  0.1098, 
				   0.1108,  0.1119,  0.1130,  0.1140, 
				   0.1151,  0.1162,  0.1172,  0.1183, 
				   0.1193,  0.1204,  0.1215,  0.1225, 
				   0.1236,  0.1247,  0.1257,  0.1268, 
				   0.1278,  0.1289,  0.1300,  0.1310, 
				   0.1321,  0.1332,  0.1342,  0.1353, 
				   0.1363,  0.1374,  0.1385,  0.1395, 
				   0.1406,  0.1417,  0.1427,  0.1438, 
				   0.1449,  0.1459,  0.1470,  0.1480, 
				   0.1491,  0.1502,  0.1512,  0.1523, 
				   0.1534,  0.1544,  0.1555,  0.1566, 
				   0.1576,  0.1587,  0.1598,  0.1608, 
				   0.1619,  0.1629,  0.1640,  0.1651, 
				   0.1661,  0.1672,  0.1683,  0.1693, 
				   0.1704,  0.1715,  0.1725,  0.1736, 
				   0.1747,  0.1757,  0.1768,  0.1779, 
				   0.1789,  0.1800,  0.1811,  0.1821, 
				   0.1832,  0.1843,  0.1853,  0.1864, 
				   0.1874,  0.1885,  0.1896,  0.1906, 
				   0.1917,  0.1928,  0.1938,  0.1949, 
				   0.1960,  0.1970,  0.1981,  0.1992, 
				   0.2002,  0.2013,  0.2024,  0.2034, 
				   0.2045,  0.2056,  0.2067,  0.2077, 
				   0.2088,  0.2099,  0.2109,  0.2120, 
				   0.2131,  0.2141,  0.2152,  0.2163, 
				   0.2173,  0.2184,  0.2195,  0.2205, 
				   0.2216,  0.2227,  0.2237,  0.2248, 
				   0.2259,  0.2269,  0.2280,  0.2291, 
				   0.2302,  0.2312,  0.2323,  0.2334, 
				   0.2344,  0.2355,  0.2366,  0.2376, 
				   0.2387,  0.2398,  0.2409,  0.2419, 
				   0.2430,  0.2441,  0.2451,  0.2462, 
				   0.2473,  0.2483,  0.2494,  0.2505, 
				   0.2516,  0.2526,  0.2537,  0.2548, 
				   0.2558,  0.2569,  0.2580,  0.2591, 
				   0.2601,  0.2612,  0.2623,  0.2634, 
				   0.2644,  0.2655,  0.2666,  0.2676, 
				   0.2687,  0.2698,  0.2709,  0.2719, 
				   0.2730,  0.2741,  0.2752,  0.2762, 
				   0.2773,  0.2784,  0.2794,  0.2805, 
				   0.2816,  0.2827,  0.2837,  0.2848, 
				   0.2859,  0.2870,  0.2880,  0.2891, 
				   0.2902,  0.2913,  0.2923,  0.2934, 
				   0.2945,  0.2956,  0.2966,  0.2977, 
				   0.2988,  0.2999,  0.3009,  0.3020, 
				   0.3031,  0.3042,  0.3053,  0.3063, 
				   0.3074,  0.3085,  0.3096,  0.3106, 
				   0.3117,  0.3128,  0.3139,  0.3149, 
				   0.3160,  0.3171,  0.3182,  0.3193, 
				   0.3203,  0.3214,  0.3225,  0.3236, 
				   0.3246,  0.3257,  0.3268,  0.3279, 
				   0.3290,  0.3300,  0.3311,  0.3322, 
				   0.3333,  0.3344,  0.3354,  0.3365, 
				   0.3376,  0.3387,  0.3398,  0.3408, 
				   0.3419,  0.3430,  0.3441,  0.3452, 
				   0.3463,  0.3473,  0.3484,  0.3495, 
				   0.3506,  0.3517,  0.3527,  0.3538, 
				   0.3549,  0.3560,  0.3571,  0.3582, 
				   0.3592,  0.3603,  0.3614,  0.3625, 
				   0.3636,  0.3647,  0.3657,  0.3668, 
				   0.3679,  0.3690,  0.3701,  0.3712, 
				   0.3722,  0.3733,  0.3744,  0.3755, 
				   0.3766,  0.3777,  0.3788,  0.3798, 
				   0.3809,  0.3820,  0.3831,  0.3842, 
				   0.3853,  0.3864,  0.3874,  0.3885, 
				   0.3896,  0.3907,  0.3918,  0.3929, 
				   0.3940,  0.3951,  0.3961,  0.3972, 
				   0.3983,  0.3994,  0.4005,  0.4016, 
				   0.4027,  0.4038,  0.4049,  0.4059, 
				   0.4070,  0.4081,  0.4092,  0.4103, 
				   0.4114,  0.4125,  0.4136,  0.4147, 
				   0.4158,  0.4168,  0.4179,  0.4190, 
				   0.4201,  0.4212,  0.4223,  0.4234, 
				   0.4245,  0.4256,  0.4267,  0.4278, 
				   0.4289,  0.4300,  0.4310,  0.4321, 
				   0.4332,  0.4343,  0.4354,  0.4365, 
				   0.4376,  0.4387,  0.4398,  0.4409, 
				   0.4420,  0.4431,  0.4442,  0.4453, 
				   0.4464,  0.4475,  0.4486,  0.4497, 
				   0.4508,  0.4518,  0.4529,  0.4540, 
				   0.4551,  0.4562,  0.4573,  0.4584, 
				   0.4595,  0.4606,  0.4617,  0.4628, 
				   0.4639,  0.4650,  0.4661,  0.4672, 
				   0.4683,  0.4694,  0.4705,  0.4716, 
				   0.4727,  0.4738,  0.4749,  0.4760, 
				   0.4771,  0.4782,  0.4793,  0.4804, 
				   0.4815,  0.4826,  0.4837,  0.4848, 
				   0.4859,  0.4870,  0.4881,  0.4892, 
				   0.4903,  0.4914,  0.4926,  0.4937, 
				   0.4948,  0.4959,  0.4970,  0.4981, 
				   0.4992,  0.5003,  0.5014,  0.5025, 
				   0.5036,  0.5047,  0.5058,  0.5069, 
				   0.5080,  0.5091,  0.5102,  0.5113, 
				   0.5125,  0.5136,  0.5147,  0.5158, 
				   0.5169,  0.5180,  0.5191,  0.5202, 
				   0.5213,  0.5224,  0.5235,  0.5246, 
				   0.5258,  0.5269,  0.5280,  0.5291, 
				   0.5302,  0.5313,  0.5324,  0.5335, 
				   0.5346,  0.5358,  0.5369,  0.5380, 
				   0.5391,  0.5402,  0.5413,  0.5424, 
				   0.5435,  0.5447,  0.5458,  0.5469, 
				   0.5480,  0.5491,  0.5502,  0.5513, 
				   0.5525,  0.5536,  0.5547,  0.5558, 
				   0.5569,  0.5580,  0.5592,  0.5603, 
				   0.5614,  0.5625,  0.5636,  0.5647, 
				   0.5659,  0.5670,  0.5681,  0.5692, 
				   0.5703,  0.5715,  0.5726,  0.5737, 
				   0.5748,  0.5759,  0.5770,  0.5782, 
				   0.5793,  0.5804,  0.5815,  0.5827, 
				   0.5838,  0.5849,  0.5860,  0.5871, 
				   0.5883,  0.5894,  0.5905,  0.5916, 
				   0.5928,  0.5939,  0.5950,  0.5961, 
				   0.5973,  0.5984,  0.5995,  0.6006, 
				   0.6018,  0.6029,  0.6040,  0.6051, 
				   0.6063,  0.6074,  0.6085,  0.6096, 
				   0.6108,  0.6119,  0.6130,  0.6142, 
				   0.6153,  0.6164,  0.6175,  0.6187, 
				   0.6198,  0.6209,  0.6221,  0.6232, 
				   0.6243,  0.6255,  0.6266,  0.6277, 
				   0.6289,  0.6300,  0.6311,  0.6323, 
				   0.6334,  0.6345,  0.6357,  0.6368, 
				   0.6379,  0.6391,  0.6402,  0.6413, 
				   0.6425,  0.6436,  0.6447,  0.6459, 
				   0.6470,  0.6481,  0.6493,  0.6504, 
				   0.6516,  0.6527,  0.6538,  0.6550, 
				   0.6561,  0.6572,  0.6584,  0.6595, 
				   0.6607,  0.6618,  0.6629,  0.6641, 
				   0.6652,  0.6664,  0.6675,  0.6687, 
				   0.6698,  0.6709,  0.6721,  0.6732, 
				   0.6744,  0.6755,  0.6767,  0.6778, 
				   0.6789,  0.6801,  0.6812,  0.6824, 
				   0.6835,  0.6847,  0.6858,  0.6870, 
				   0.6881,  0.6893,  0.6904,  0.6915, 
				   0.6927,  0.6938,  0.6950,  0.6961, 
				   0.6973,  0.6984,  0.6996,  0.7007, 
				   0.7019,  0.7030,  0.7042,  0.7053, 
				   0.7065,  0.7076,  0.7088,  0.7100, 
				   0.7111,  0.7123,  0.7134,  0.7146, 
				   0.7157,  0.7169,  0.7180,  0.7192, 
				   0.7203,  0.7215,  0.7227,  0.7238, 
				   0.7250,  0.7261,  0.7273,  0.7284, 
				   0.7296,  0.7308,  0.7319,  0.7331, 
				   0.7342,  0.7354,  0.7366,  0.7377, 
				   0.7389,  0.7400,  0.7412,  0.7424, 
				   0.7435,  0.7447,  0.7458,  0.7470, 
				   0.7482,  0.7493,  0.7505,  0.7517, 
				   0.7528,  0.7540,  0.7552,  0.7563, 
				   0.7575,  0.7587,  0.7598,  0.7610, 
				   0.7622,  0.7633,  0.7645,  0.7657, 
				   0.7668,  0.7680,  0.7692,  0.7703, 
				   0.7715,  0.7727,  0.7739,  0.7750, 
				   0.7762,  0.7774,  0.7785,  0.7797, 
				   0.7809,  0.7821,  0.7832,  0.7844, 
				   0.7856,  0.7868,  0.7879,  0.7891, 
				   0.7903,  0.7915,  0.7926,  0.7938, 
				   0.7950,  0.7962,  0.7974,  0.7985, 
				   0.7997,  0.8009,  0.8021,  0.8032, 
				   0.8044,  0.8056,  0.8068,  0.8080, 
				   0.8092,  0.8103,  0.8115,  0.8127, 
				   0.8139,  0.8151,  0.8163,  0.8174, 
				   0.8186,  0.8198,  0.8210,  0.8222, 
				   0.8234,  0.8246,  0.8257,  0.8269, 
				   0.8281,  0.8293,  0.8305,  0.8317, 
				   0.8329,  0.8341,  0.8353,  0.8364, 
				   0.8376,  0.8388,  0.8400,  0.8412, 
				   0.8424,  0.8436,  0.8448,  0.8460, 
				   0.8472,  0.8484,  0.8496,  0.8508, 
				   0.8520,  0.8532,  0.8544,  0.8556, 
				   0.8568,  0.8580,  0.8591,  0.8603, 
				   0.8615,  0.8627,  0.8639,  0.8651, 
				   0.8663,  0.8676,  0.8688,  0.8700, 
				   0.8712,  0.8724,  0.8736,  0.8748, 
				   0.8760,  0.8772,  0.8784,  0.8796, 
				   0.8808,  0.8820,  0.8832,  0.8844, 
				   0.8856,  0.8868,  0.8880,  0.8892, 
				   0.8905,  0.8917,  0.8929,  0.8941, 
				   0.8953,  0.8965,  0.8977,  0.8989, 
				   0.9001,  0.9014,  0.9026,  0.9038, 
				   0.9050,  0.9062,  0.9074,  0.9087, 
				   0.9099,  0.9111,  0.9123,  0.9135, 
				   0.9147,  0.9160,  0.9172,  0.9184, 
				   0.9196,  0.9208,  0.9221,  0.9233, 
				   0.9245,  0.9257,  0.9269,  0.9282, 
				   0.9294,  0.9306,  0.9318,  0.9331, 
				   0.9343,  0.9355,  0.9367,  0.9380, 
				   0.9392,  0.9404,  0.9417,  0.9429, 
				   0.9441,  0.9453,  0.9466,  0.9478, 
				   0.9490,  0.9503,  0.9515,  0.9527, 
				   0.9540,  0.9552,  0.9564,  0.9577, 
				   0.9589,  0.9601,  0.9614,  0.9626, 
				   0.9639,  0.9651,  0.9663,  0.9676, 
				   0.9688,  0.9701,  0.9713,  0.9725, 
				   0.9738,  0.9750,  0.9763,  0.9775, 
				   0.9787,  0.9800,  0.9812,  0.9825, 
				   0.9837,  0.9850,  0.9862,  0.9875, 
				   0.9887,  0.9900,  0.9912,  0.9925, 
				   0.9937,  0.9949,  0.9962,  0.9975, 
				   0.9987,  1.0000,  1.0012,  1.0025, 
				   1.0037,  1.0050,  1.0062,  1.0075, 
				   1.0087,  1.0100,  1.0112,  1.0125, 
				   1.0138,  1.0150,  1.0163,  1.0175, 
				   1.0188,  1.0201,  1.0213,  1.0226, 
				   1.0238,  1.0251,  1.0264,  1.0276, 
				   1.0289,  1.0302,  1.0314,  1.0327, 
				   1.0340,  1.0352,  1.0365,  1.0378, 
				   1.0390,  1.0403,  1.0416,  1.0428, 
				   1.0441,  1.0454,  1.0467,  1.0479, 
				   1.0492,  1.0505,  1.0517,  1.0530, 
				   1.0543,  1.0556,  1.0568,  1.0581, 
				   1.0594,  1.0607,  1.0620,  1.0632, 
				   1.0645,  1.0658,  1.0671,  1.0684, 
				   1.0696,  1.0709,  1.0722,  1.0735, 
				   1.0748,  1.0761,  1.0774,  1.0786, 
				   1.0799,  1.0812,  1.0825,  1.0838, 
				   1.0851,  1.0864,  1.0877,  1.0890, 
				   1.0902,  1.0915,  1.0928,  1.0941, 
				   1.0954,  1.0967,  1.0980,  1.0993, 
				   1.1006,  1.1019,  1.1032,  1.1045, 
				   1.1058,  1.1071,  1.1084,  1.1097, 
				   1.1110,  1.1123,  1.1136,  1.1149, 
				   1.1162,  1.1175,  1.1188,  1.1201, 
				   1.1214,  1.1227,  1.1241,  1.1254, 
				   1.1267,  1.1280,  1.1293,  1.1306, 
				   1.1319,  1.1332,  1.1345,  1.1359, 
				   1.1372,  1.1385,  1.1398,  1.1411, 
				   1.1424,  1.1437,  1.1451,  1.1464, 
				   1.1477,  1.1490,  1.1503,  1.1517, 
				   1.1530,  1.1543,  1.1556,  1.1570, 
				   1.1583,  1.1596,  1.1609,  1.1623, 
				   1.1636,  1.1649,  1.1663,  1.1676, 
				   1.1689,  1.1702,  1.1716,  1.1729, 
				   1.1742,  1.1756,  1.1769,  1.1783, 
				   1.1796,  1.1809,  1.1823,  1.1836, 
				   1.1849,  1.1863,  1.1876,  1.1890, 
				   1.1903,  1.1916,  1.1930,  1.1943, 
				   1.1957,  1.1970,  1.1984,  1.1997, 
				   1.2011,  1.2024,  1.2038,  1.2051, 
				   1.2065,  1.2078,  1.2092,  1.2105, 
				   1.2119,  1.2132,  1.2146,  1.2159, 
				   1.2173,  1.2187,  1.2200,  1.2214, 
				   1.2227,  1.2241,  1.2254,  1.2268, 
				   1.2282,  1.2295,  1.2309,  1.2323, 
				   1.2336,  1.2350,  1.2364,  1.2377, 
				   1.2391,  1.2405,  1.2418,  1.2432, 
				   1.2446,  1.2460,  1.2473,  1.2487, 
				   1.2501,  1.2515,  1.2528,  1.2542, 
				   1.2556,  1.2570,  1.2583,  1.2597, 
				   1.2611,  1.2625,  1.2639,  1.2653, 
				   1.2666,  1.2680,  1.2694,  1.2708, 
				   1.2722,  1.2736,  1.2750,  1.2764, 
				   1.2777,  1.2791,  1.2805,  1.2819, 
				   1.2833,  1.2847,  1.2861,  1.2875, 
				   1.2889,  1.2903,  1.2917,  1.2931, 
				   1.2945,  1.2959,  1.2973,  1.2987, 
				   1.3001,  1.3015,  1.3029,  1.3043, 
				   1.3057,  1.3071,  1.3086,  1.3100, 
				   1.3114,  1.3128,  1.3142,  1.3156, 
				   1.3170,  1.3184,  1.3199,  1.3213, 
				   1.3227,  1.3241,  1.3255,  1.3270, 
				   1.3284,  1.3298,  1.3312,  1.3327, 
				   1.3341,  1.3355,  1.3369,  1.3384, 
				   1.3398,  1.3412,  1.3427,  1.3441, 
				   1.3455,  1.3470,  1.3484,  1.3498, 
				   1.3513,  1.3527,  1.3541,  1.3556, 
				   1.3570,  1.3585,  1.3599,  1.3613, 
				   1.3628,  1.3642,  1.3657,  1.3671, 
				   1.3686,  1.3700,  1.3715,  1.3729, 
				   1.3744,  1.3758,  1.3773,  1.3787, 
				   1.3802,  1.3816,  1.3831,  1.3846, 
				   1.3860,  1.3875,  1.3889,  1.3904, 
				   1.3919,  1.3933,  1.3948,  1.3963, 
				   1.3977,  1.3992,  1.4007,  1.4021, 
				   1.4036,  1.4051,  1.4066,  1.4080, 
				   1.4095,  1.4110,  1.4125,  1.4139, 
				   1.4154,  1.4169,  1.4184,  1.4199, 
				   1.4213,  1.4228,  1.4243,  1.4258, 
				   1.4273,  1.4288,  1.4303,  1.4318, 
				   1.4333,  1.4347,  1.4362,  1.4377, 
				   1.4392,  1.4407,  1.4422,  1.4437, 
				   1.4452,  1.4467,  1.4482,  1.4497, 
				   1.4512,  1.4528,  1.4543,  1.4558, 
				   1.4573,  1.4588,  1.4603,  1.4618, 
				   1.4633,  1.4648,  1.4664,  1.4679, 
				   1.4694,  1.4709,  1.4724,  1.4740, 
				   1.4755,  1.4770,  1.4785,  1.4801, 
				   1.4816,  1.4831,  1.4846,  1.4862, 
				   1.4877,  1.4892,  1.4908,  1.4923, 
				   1.4938,  1.4954,  1.4969,  1.4985, 
				   1.5000,  1.5015,  1.5031,  1.5046, 
				   1.5062,  1.5077,  1.5093,  1.5108, 
				   1.5124,  1.5139,  1.5155,  1.5170, 
				   1.5186,  1.5202,  1.5217,  1.5233, 
				   1.5248,  1.5264,  1.5280,  1.5295, 
				   1.5311,  1.5327,  1.5342,  1.5358, 
				   1.5374,  1.5389,  1.5405,  1.5421, 
				   1.5437,  1.5452,  1.5468,  1.5484, 
				   1.5500,  1.5516,  1.5531,  1.5547, 
				   1.5563,  1.5579,  1.5595,  1.5611, 
				   1.5627,  1.5643,  1.5659,  1.5675, 
				   1.5691,  1.5707,  1.5723,  1.5739, 
				   1.5755,  1.5771,  1.5787,  1.5803, 
				   1.5819,  1.5835,  1.5851,  1.5867, 
				   1.5883,  1.5899,  1.5916,  1.5932, 
				   1.5948,  1.5964,  1.5980,  1.5997, 
				   1.6013,  1.6029,  1.6045,  1.6062, 
				   1.6078,  1.6094,  1.6111,  1.6127, 
				   1.6143,  1.6160,  1.6176,  1.6192, 
				   1.6209,  1.6225,  1.6242,  1.6258, 
				   1.6275,  1.6291,  1.6308,  1.6324, 
				   1.6341,  1.6357,  1.6374,  1.6390, 
				   1.6407,  1.6424,  1.6440,  1.6457, 
				   1.6474,  1.6490,  1.6507,  1.6524, 
				   1.6540,  1.6557,  1.6574,  1.6591, 
				   1.6607,  1.6624,  1.6641,  1.6658, 
				   1.6675,  1.6691,  1.6708,  1.6725, 
				   1.6742,  1.6759,  1.6776,  1.6793, 
				   1.6810,  1.6827,  1.6844,  1.6861, 
				   1.6878,  1.6895,  1.6912,  1.6929, 
				   1.6946,  1.6963,  1.6980,  1.6998, 
				   1.7015,  1.7032,  1.7049,  1.7066, 
				   1.7084,  1.7101,  1.7118,  1.7135, 
				   1.7153,  1.7170,  1.7187,  1.7205, 
				   1.7222,  1.7239,  1.7257,  1.7274, 
				   1.7292,  1.7309,  1.7327,  1.7344, 
				   1.7362,  1.7379,  1.7397,  1.7414, 
				   1.7432,  1.7449,  1.7467,  1.7485, 
				   1.7502,  1.7520,  1.7538,  1.7555, 
				   1.7573,  1.7591,  1.7609,  1.7626, 
				   1.7644,  1.7662,  1.7680,  1.7698, 
				   1.7716,  1.7734,  1.7751,  1.7769, 
				   1.7787,  1.7805,  1.7823,  1.7841, 
				   1.7859,  1.7877,  1.7895,  1.7914, 
				   1.7932,  1.7950,  1.7968,  1.7986, 
				   1.8004,  1.8022,  1.8041,  1.8059, 
				   1.8077,  1.8095,  1.8114,  1.8132, 
				   1.8150,  1.8169,  1.8187,  1.8206, 
				   1.8224,  1.8242,  1.8261,  1.8279, 
				   1.8298,  1.8316,  1.8335,  1.8354, 
				   1.8372,  1.8391,  1.8409,  1.8428, 
				   1.8447,  1.8465,  1.8484,  1.8503, 
				   1.8522,  1.8540,  1.8559,  1.8578, 
				   1.8597,  1.8616,  1.8635,  1.8654, 
				   1.8673,  1.8692,  1.8711,  1.8730, 
				   1.8749,  1.8768,  1.8787,  1.8806, 
				   1.8825,  1.8844,  1.8863,  1.8882, 
				   1.8902,  1.8921,  1.8940,  1.8959, 
				   1.8979,  1.8998,  1.9017,  1.9037, 
				   1.9056,  1.9076,  1.9095,  1.9115, 
				   1.9134,  1.9154,  1.9173,  1.9193, 
				   1.9212,  1.9232,  1.9252,  1.9271, 
				   1.9291,  1.9311,  1.9330,  1.9350, 
				   1.9370,  1.9390,  1.9410,  1.9430, 
				   1.9449,  1.9469,  1.9489,  1.9509, 
				   1.9529,  1.9549,  1.9569,  1.9589, 
				   1.9610,  1.9630,  1.9650,  1.9670, 
				   1.9690,  1.9710,  1.9731,  1.9751, 
				   1.9771,  1.9792,  1.9812,  1.9832, 
				   1.9853,  1.9873,  1.9894,  1.9914, 
				   1.9935,  1.9955,  1.9976,  1.9997, 
				   2.0017,  2.0038,  2.0059,  2.0079, 
				   2.0100,  2.0121,  2.0142,  2.0163, 
				   2.0184,  2.0205,  2.0225,  2.0246, 
				   2.0267,  2.0289,  2.0310,  2.0331, 
				   2.0352,  2.0373,  2.0394,  2.0415, 
				   2.0437,  2.0458,  2.0479,  2.0501, 
				   2.0522,  2.0543,  2.0565,  2.0586, 
				   2.0608,  2.0629,  2.0651,  2.0672, 
				   2.0694,  2.0716,  2.0737,  2.0759, 
				   2.0781,  2.0803,  2.0824,  2.0846, 
				   2.0868,  2.0890,  2.0912,  2.0934, 
				   2.0956,  2.0978,  2.1000,  2.1022, 
				   2.1044,  2.1067,  2.1089,  2.1111, 
				   2.1133,  2.1156,  2.1178,  2.1201, 
				   2.1223,  2.1245,  2.1268,  2.1290, 
				   2.1313,  2.1336,  2.1358,  2.1381, 
				   2.1404,  2.1427,  2.1449,  2.1472, 
				   2.1495,  2.1518,  2.1541,  2.1564, 
				   2.1587,  2.1610,  2.1633,  2.1656, 
				   2.1679,  2.1703,  2.1726,  2.1749, 
				   2.1773,  2.1796,  2.1819,  2.1843, 
				   2.1866,  2.1890,  2.1913,  2.1937, 
				   2.1961,  2.1984,  2.2008,  2.2032, 
				   2.2056,  2.2080,  2.2103,  2.2127, 
				   2.2151,  2.2175,  2.2200,  2.2224, 
				   2.2248,  2.2272,  2.2296,  2.2321, 
				   2.2345,  2.2369,  2.2394,  2.2418, 
				   2.2443,  2.2467,  2.2492,  2.2517, 
				   2.2541,  2.2566,  2.2591,  2.2616, 
				   2.2640,  2.2665,  2.2690,  2.2715, 
				   2.2740,  2.2766,  2.2791,  2.2816, 
				   2.2841,  2.2867,  2.2892,  2.2917, 
				   2.2943,  2.2968,  2.2994,  2.3019, 
				   2.3045,  2.3071,  2.3097,  2.3122, 
				   2.3148,  2.3174,  2.3200,  2.3226, 
				   2.3252,  2.3278,  2.3305,  2.3331, 
				   2.3357,  2.3383,  2.3410,  2.3436, 
				   2.3463,  2.3489,  2.3516,  2.3543, 
				   2.3569,  2.3596,  2.3623,  2.3650, 
				   2.3677,  2.3704,  2.3731,  2.3758, 
				   2.3785,  2.3812,  2.3840,  2.3867, 
				   2.3895,  2.3922,  2.3950,  2.3977, 
				   2.4005,  2.4033,  2.4060,  2.4088, 
				   2.4116,  2.4144,  2.4172,  2.4200, 
				   2.4228,  2.4257,  2.4285,  2.4313, 
				   2.4342,  2.4370,  2.4399,  2.4427, 
				   2.4456,  2.4485,  2.4514,  2.4542, 
				   2.4571,  2.4600,  2.4630,  2.4659, 
				   2.4688,  2.4717,  2.4747,  2.4776, 
				   2.4806,  2.4835,  2.4865,  2.4895, 
				   2.4924,  2.4954,  2.4984,  2.5014, 
				   2.5044,  2.5075,  2.5105,  2.5135, 
				   2.5166,  2.5196,  2.5227,  2.5257, 
				   2.5288,  2.5319,  2.5350,  2.5381, 
				   2.5412,  2.5443,  2.5474,  2.5506, 
				   2.5537,  2.5568,  2.5600,  2.5632, 
				   2.5663,  2.5695,  2.5727,  2.5759, 
				   2.5791,  2.5823,  2.5855,  2.5888, 
				   2.5920,  2.5953,  2.5985,  2.6018, 
				   2.6051,  2.6084,  2.6117,  2.6150, 
				   2.6183,  2.6216,  2.6250,  2.6283, 
				   2.6317,  2.6350,  2.6384,  2.6418, 
				   2.6452,  2.6486,  2.6520,  2.6555, 
				   2.6589,  2.6623,  2.6658,  2.6693, 
				   2.6728,  2.6762,  2.6797,  2.6833, 
				   2.6868,  2.6903,  2.6939,  2.6974, 
				   2.7010,  2.7046,  2.7082,  2.7118, 
				   2.7154,  2.7190,  2.7227,  2.7263, 
				   2.7300,  2.7337,  2.7373,  2.7410, 
				   2.7448,  2.7485,  2.7522,  2.7560, 
				   2.7597,  2.7635,  2.7673,  2.7711, 
				   2.7749,  2.7788,  2.7826,  2.7865, 
				   2.7903,  2.7942,  2.7981,  2.8021, 
				   2.8060,  2.8099,  2.8139,  2.8179, 
				   2.8219,  2.8259,  2.8299,  2.8339, 
				   2.8380,  2.8420,  2.8461,  2.8502, 
				   2.8543,  2.8584,  2.8626,  2.8667, 
				   2.8709,  2.8751,  2.8793,  2.8836, 
				   2.8878,  2.8921,  2.8964,  2.9007, 
				   2.9050,  2.9093,  2.9137,  2.9180, 
				   2.9224,  2.9268,  2.9313,  2.9357, 
				   2.9402,  2.9447,  2.9492,  2.9537, 
				   2.9582,  2.9628,  2.9674,  2.9720, 
				   2.9766,  2.9813,  2.9860,  2.9907, 
				   2.9954,  3.0001,  3.0049,  3.0097, 
				   3.0145,  3.0193,  3.0242,  3.0290, 
				   3.0339,  3.0389,  3.0438,  3.0488, 
				   3.0538,  3.0588,  3.0639,  3.0690, 
				   3.0741,  3.0792,  3.0844,  3.0896, 
				   3.0948,  3.1000,  3.1053,  3.1106, 
				   3.1159,  3.1213,  3.1267,  3.1321, 
				   3.1376,  3.1430,  3.1486,  3.1541, 
				   3.1597,  3.1653,  3.1709,  3.1766, 
				   3.1823,  3.1881,  3.1939,  3.1997, 
				   3.2056,  3.2114,  3.2174,  3.2233, 
				   3.2294,  3.2354,  3.2415,  3.2476, 
				   3.2538,  3.2600,  3.2662,  3.2725, 
				   3.2789,  3.2853,  3.2917,  3.2982, 
				   3.3047,  3.3112,  3.3179,  3.3245, 
				   3.3312,  3.3380,  3.3448,  3.3517, 
				   3.3586,  3.3656,  3.3726,  3.3797, 
				   3.3868,  3.3940,  3.4013,  3.4086, 
				   3.4160,  3.4234,  3.4309,  3.4385, 
				   3.4462,  3.4539,  3.4616,  3.4695, 
				   3.4774,  3.4854,  3.4935,  3.5016, 
				   3.5098,  3.5181,  3.5265,  3.5350, 
				   3.5435,  3.5521,  3.5609,  3.5697, 
				   3.5786,  3.5876,  3.5967,  3.6059, 
				   3.6152,  3.6246,  3.6342,  3.6438, 
				   3.6535,  3.6634,  3.6734,  3.6835, 
				   3.6937,  3.7041,  3.7146,  3.7252, 
				   3.7360,  3.7470,  3.7580,  3.7693, 
				   3.7807,  3.7922,  3.8040,  3.8159, 
				   3.8280,  3.8402,  3.8527,  3.8654, 
				   3.8783,  3.8914,  3.9047,  3.9183, 
				   3.9321,  3.9462,  3.9605,  3.9751, 
				   3.9900,  4.0052,  4.0207,  4.0366, 
				   4.0527,  4.0693,  4.0862,  4.1035, 
				   4.1212,  4.1393,  4.1579,  4.1770, 
				   4.1966,  4.2168,  4.2375,  4.2588, 
				   4.2808,  4.3035,  4.3270,  4.3512, 
				   4.3763,  4.4024,  4.4294,  4.4576, 
				   4.4869,  4.5176,  4.5497,  4.5835, 
				   4.6191,  4.6566,  4.6965,  4.7390, 
				   4.7845,  4.8336,  4.8868,  4.9450, 
				   5.0093,  5.0814,  5.1634,  5.2590, 
				   5.3738,  5.5185,  5.7164,  6.0393};

const unsigned int GaussianSQ::Rmax = 12;
const float* GaussianSQ::yk[] = {yk0, yk1, yk2, yk3, yk4, yk5, yk6,
                                 yk7, yk8, yk9, yk10, yk11, yk12 };


