// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/


// $Id: LaplacianSQ.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $

#include <math.h>

#include "ImageLib.h"
#include "LaplacianSQ.h"


LaplacianSQ::LaplacianSQ(float imu, float isigma, const QRate& qr) :
  ScalarQuantiser(qr)
{
  reset_shape(imu, isigma);
  reset_rate(qr);
}

LaplacianSQ::~LaplacianSQ()
{
  delete [] xk;
}

void LaplacianSQ::reset_shape(float imu, float isigma)
{
  mu = imu;
  sigma = isigma;
}

void LaplacianSQ::reset_rate(const QRate& qr)
{
  uRate = qr.Rate();
  ASSERT_FATAL(uRate <= Rmax);
  if (uRate > 0)
    uRightLevels = 1 << (uRate-1);
  else
    uRightLevels = 1;
  xk = new float [uRightLevels];
  xk[0] = 0.0;
  unsigned int i;
  for (i = 1; i < uRightLevels; i++)
    xk[i] = (yk[uRate][i-1] + yk[uRate][i])/2.0;
}

int LaplacianSQ::Index(float x) const
{
  float stdx = (x - mu)/sigma;
  float pstdx = fabs(stdx);
  unsigned int i = 1;
  while (pstdx > xk[i] && i < uRightLevels)
    i++;
  return (stdx < 0.0)?(uRightLevels-i):(i-1+uRightLevels);
}

float LaplacianSQ::Reconstruction(unsigned int idx) const
{
  ASSERT_FATAL(idx < 2*uRightLevels);
  float sign = (idx < uRightLevels)?-1:+1;
  int rcnst = (sign < 0)?(uRightLevels - idx - 1):(idx - uRightLevels);
  return sigma*sign*yk[uRate][rcnst] + mu;
}

//These values as in Jain89 pg. 108
const float LaplacianSQ::yk0[] = { 0.0000 };
const float LaplacianSQ::yk1[] = { 0.7071 };
const float LaplacianSQ::yk2[] = { 0.4198, 1.8340 };
const float LaplacianSQ::yk3[] = { 0.2334, 0.8330, 1.6725, 3.0868 };
const float LaplacianSQ::yk4[] = { 0.1240, 0.4049, 0.7288, 1.1111,
                                   1.5780, 2.1776, 3.0171, 4.4314 };
//Remaining values computed using OptQuantL - some inaccuracy
const float LaplacianSQ::yk5[] = { 0.0640,  0.2004,  0.3461,  0.5025, 
				   0.6714,  0.8550,  1.0559,  1.2779, 
				   1.5259,  1.8067,  2.1306,  2.5129, 
				   2.9797,  3.5793,  4.4188,  5.8330 };
const float LaplacianSQ::yk6[] = { 0.0326,  0.0998,  0.1692,  0.2409, 
				   0.3152,  0.3922,  0.4720,  0.5550, 
				   0.6414,  0.7314,  0.8254,  0.9238, 
				   1.0270,  1.1354,  1.2497,  1.3705, 
				   1.4986,  1.6350,  1.7807,  1.9371, 
				   2.1060,  2.2896,  2.4905,  2.7125, 
				   2.9605,  3.2413,  3.5651,  3.9475, 
				   4.4143,  5.0138,  5.8533,  7.2676 };
const float LaplacianSQ::yk7[] = { 0.0164,  0.0498,  0.0837,  0.1182, 
				   0.1532,  0.1888,  0.2250,  0.2619, 
				   0.2994,  0.3376,  0.3764,  0.4160, 
				   0.4564,  0.4975,  0.5395,  0.5823, 
				   0.6260,  0.6706,  0.7161,  0.7627, 
				   0.8103,  0.8590,  0.9088,  0.9598, 
				   1.0121,  1.0657,  1.1208,  1.1772, 
				   1.2352,  1.2949,  1.3563,  1.4195, 
				   1.4846,  1.5518,  1.6212,  1.6930, 
				   1.7673,  1.8442,  1.9241,  2.0071, 
				   2.0934,  2.1835,  2.2775,  2.3759, 
				   2.4790,  2.5875,  2.7018,  2.8226, 
				   2.9507,  3.0870,  3.2327,  3.3891, 
				   3.5581,  3.7416,  3.9425,  4.1645, 
				   4.4125,  4.6934,  5.0172,  5.3995, 
				   5.8663,  6.4659,  7.3054,  8.7196 };
const float LaplacianSQ::yk8[] = { 0.0082,  0.0249,  0.0416,  0.0585, 
				   0.0756,  0.0927,  0.1100,  0.1275, 
				   0.1451,  0.1628,  0.1807,  0.1988, 
				   0.2169,  0.2353,  0.2538,  0.2725, 
				   0.2913,  0.3104,  0.3295,  0.3489, 
				   0.3684,  0.3882,  0.4081,  0.4282, 
				   0.4484,  0.4689,  0.4896,  0.5105, 
				   0.5316,  0.5529,  0.5744,  0.5962, 
				   0.6181,  0.6403,  0.6627,  0.6854, 
				   0.7083,  0.7315,  0.7549,  0.7786, 
				   0.8026,  0.8268,  0.8513,  0.8761, 
				   0.9012,  0.9265,  0.9522,  0.9782, 
				   1.0046,  1.0312,  1.0582,  1.0856, 
				   1.1133,  1.1414,  1.1698,  1.1986, 
				   1.2279,  1.2575,  1.2876,  1.3181, 
				   1.3490,  1.3804,  1.4123,  1.4446, 
				   1.4775,  1.5108,  1.5448,  1.5792, 
				   1.6142,  1.6498,  1.6861,  1.7229, 
				   1.7604,  1.7986,  1.8375,  1.8771, 
				   1.9174,  1.9586,  2.0005,  2.0433, 
				   2.0870,  2.1316,  2.1772,  2.2237, 
				   2.2713,  2.3200,  2.3698,  2.4209, 
				   2.4732,  2.5268,  2.5818,  2.6383, 
				   2.6963,  2.7559,  2.8173,  2.8805, 
				   2.9457,  3.0129,  3.0823,  3.1540, 
				   3.2283,  3.3053,  3.3851,  3.4681, 
				   3.5545,  3.6445,  3.7385,  3.8369, 
				   3.9401,  4.0485,  4.1628,  4.2836, 
				   4.4117,  4.5480,  4.6937,  4.8502, 
				   5.0191,  5.2026,  5.4036,  5.6256, 
				   5.8735,  6.1544,  6.4782,  6.8605, 
				   7.3273,  7.9269,  8.7664, 10.1806 };
const float LaplacianSQ::yk9[] = { 0.0041,  0.0124,  0.0208,  0.0291, 
				   0.0375,  0.0460,  0.0544,  0.0629, 
				   0.0715,  0.0800,  0.0886,  0.0973, 
				   0.1059,  0.1146,  0.1234,  0.1322, 
				   0.1410,  0.1498,  0.1587,  0.1676, 
				   0.1766,  0.1856,  0.1947,  0.2037, 
				   0.2129,  0.2220,  0.2312,  0.2405, 
				   0.2497,  0.2591,  0.2684,  0.2778, 
				   0.2873,  0.2967,  0.3063,  0.3159, 
				   0.3255,  0.3351,  0.3448,  0.3546, 
				   0.3644,  0.3742,  0.3841,  0.3940, 
				   0.4040,  0.4140,  0.4241,  0.4342, 
				   0.4444,  0.4546,  0.4649,  0.4752, 
				   0.4856,  0.4960,  0.5064,  0.5170, 
				   0.5275,  0.5382,  0.5489,  0.5596, 
				   0.5704,  0.5812,  0.5921,  0.6031, 
				   0.6141,  0.6252,  0.6363,  0.6475, 
				   0.6587,  0.6700,  0.6814,  0.6928, 
				   0.7043,  0.7159,  0.7275,  0.7392, 
				   0.7509,  0.7627,  0.7746,  0.7866, 
				   0.7986,  0.8107,  0.8228,  0.8350, 
				   0.8473,  0.8597,  0.8721,  0.8846, 
				   0.8972,  0.9099,  0.9226,  0.9354, 
				   0.9483,  0.9613,  0.9743,  0.9874, 
				   1.0006,  1.0139,  1.0273,  1.0408, 
				   1.0543,  1.0680,  1.0817,  1.0955, 
				   1.1094,  1.1234,  1.1375,  1.1517, 
				   1.1659,  1.1803,  1.1948,  1.2093, 
				   1.2240,  1.2388,  1.2537,  1.2687, 
				   1.2837,  1.2989,  1.3142,  1.3297, 
				   1.3452,  1.3608,  1.3766,  1.3925, 
				   1.4085,  1.4246,  1.4409,  1.4572, 
				   1.4737,  1.4903,  1.5071,  1.5240, 
				   1.5410,  1.5582,  1.5755,  1.5929, 
				   1.6105,  1.6283,  1.6462,  1.6642, 
				   1.6824,  1.7008,  1.7193,  1.7380, 
				   1.7568,  1.7758,  1.7950,  1.8144, 
				   1.8339,  1.8536,  1.8735,  1.8936, 
				   1.9139,  1.9344,  1.9551,  1.9759, 
				   1.9970,  2.0183,  2.0399,  2.0616, 
				   2.0836,  2.1058,  2.1282,  2.1509, 
				   2.1738,  2.1969,  2.2204,  2.2440, 
				   2.2680,  2.2922,  2.3167,  2.3415, 
				   2.3666,  2.3920,  2.4177,  2.4437, 
				   2.4700,  2.4967,  2.5237,  2.5510, 
				   2.5787,  2.6068,  2.6352,  2.6641, 
				   2.6933,  2.7229,  2.7530,  2.7835, 
				   2.8144,  2.8458,  2.8777,  2.9101, 
				   2.9429,  2.9763,  3.0102,  3.0446, 
				   3.0797,  3.1153,  3.1515,  3.1883, 
				   3.2258,  3.2640,  3.3029,  3.3425, 
				   3.3829,  3.4240,  3.4659,  3.5087, 
				   3.5524,  3.5970,  3.6426,  3.6891, 
				   3.7367,  3.7854,  3.8353,  3.8863, 
				   3.9386,  3.9922,  4.0472,  4.1037, 
				   4.1617,  4.2214,  4.2827,  4.3459, 
				   4.4111,  4.4783,  4.5477,  4.6195, 
				   4.6937,  4.7707,  4.8505,  4.9335, 
				   5.0199,  5.1099,  5.2039,  5.3023, 
				   5.4055,  5.5139,  5.6282,  5.7490, 
				   5.8771,  6.0135,  6.1592,  6.3156, 
				   6.4845,  6.6681,  6.8690,  7.0910, 
				   7.3390,  7.6198,  7.9436,  8.3260, 
				   8.7928,  9.3923, 10.2318, 11.6461 };
const float LaplacianSQ::yk10[] = { 0.0021,  0.0062,  0.0104,  0.0145, 
				    0.0187,  0.0229,  0.0271,  0.0312, 
				    0.0354,  0.0397,  0.0439,  0.0481, 
				    0.0523,  0.0566,  0.0608,  0.0651, 
				    0.0694,  0.0736,  0.0779,  0.0822, 
				    0.0865,  0.0908,  0.0952,  0.0995, 
				    0.1038,  0.1082,  0.1125,  0.1169, 
				    0.1213,  0.1256,  0.1300,  0.1344, 
				    0.1389,  0.1433,  0.1477,  0.1521, 
				    0.1566,  0.1610,  0.1655,  0.1700, 
				    0.1745,  0.1790,  0.1835,  0.1880, 
				    0.1925,  0.1970,  0.2016,  0.2061, 
				    0.2107,  0.2153,  0.2199,  0.2244, 
				    0.2290,  0.2337,  0.2383,  0.2429, 
				    0.2476,  0.2522,  0.2569,  0.2615, 
				    0.2662,  0.2709,  0.2756,  0.2803, 
				    0.2851,  0.2898,  0.2946,  0.2993, 
				    0.3041,  0.3089,  0.3136,  0.3184, 
				    0.3233,  0.3281,  0.3329,  0.3378, 
				    0.3426,  0.3475,  0.3524,  0.3572, 
				    0.3621,  0.3671,  0.3720,  0.3769, 
				    0.3819,  0.3868,  0.3918,  0.3968, 
				    0.4018,  0.4068,  0.4118,  0.4168, 
				    0.4218,  0.4269,  0.4320,  0.4370, 
				    0.4421,  0.4472,  0.4523,  0.4575, 
				    0.4626,  0.4678,  0.4729,  0.4781, 
				    0.4833,  0.4885,  0.4937,  0.4989, 
				    0.5042,  0.5094,  0.5147,  0.5200, 
				    0.5252,  0.5306,  0.5359,  0.5412, 
				    0.5466,  0.5519,  0.5573,  0.5627, 
				    0.5681,  0.5735,  0.5789,  0.5843, 
				    0.5898,  0.5953,  0.6008,  0.6063, 
				    0.6118,  0.6173,  0.6228,  0.6284, 
				    0.6340,  0.6395,  0.6451,  0.6508, 
				    0.6564,  0.6620,  0.6677,  0.6734, 
				    0.6791,  0.6848,  0.6905,  0.6962, 
				    0.7020,  0.7077,  0.7135,  0.7193, 
				    0.7251,  0.7310,  0.7368,  0.7427, 
				    0.7485,  0.7544,  0.7604,  0.7663, 
				    0.7722,  0.7782,  0.7842,  0.7902, 
				    0.7962,  0.8022,  0.8083,  0.8143, 
				    0.8204,  0.8265,  0.8326,  0.8387, 
				    0.8449,  0.8511,  0.8573,  0.8635, 
				    0.8697,  0.8759,  0.8822,  0.8885, 
				    0.8948,  0.9011,  0.9074,  0.9138, 
				    0.9202,  0.9266,  0.9330,  0.9394, 
				    0.9458,  0.9523,  0.9588,  0.9653, 
				    0.9719,  0.9784,  0.9850,  0.9916, 
				    0.9982,  1.0048,  1.0115,  1.0181, 
				    1.0248,  1.0316,  1.0383,  1.0451, 
				    1.0518,  1.0587,  1.0655,  1.0723, 
				    1.0792,  1.0861,  1.0930,  1.0999, 
				    1.1069,  1.1139,  1.1209,  1.1279, 
				    1.1350,  1.1421,  1.1492,  1.1563, 
				    1.1634,  1.1706,  1.1778,  1.1850, 
				    1.1923,  1.1995,  1.2068,  1.2142, 
				    1.2215,  1.2289,  1.2363,  1.2437, 
				    1.2512,  1.2586,  1.2661,  1.2737, 
				    1.2812,  1.2888,  1.2964,  1.3041, 
				    1.3117,  1.3194,  1.3271,  1.3349, 
				    1.3427,  1.3505,  1.3583,  1.3662, 
				    1.3741,  1.3820,  1.3899,  1.3979, 
				    1.4059,  1.4140,  1.4221,  1.4302, 
				    1.4383,  1.4465,  1.4547,  1.4629, 
				    1.4712,  1.4795,  1.4878,  1.4962, 
				    1.5046,  1.5130,  1.5214,  1.5299, 
				    1.5385,  1.5470,  1.5556,  1.5643, 
				    1.5729,  1.5816,  1.5904,  1.5992, 
				    1.6080,  1.6168,  1.6257,  1.6346, 
				    1.6436,  1.6526,  1.6616,  1.6707, 
				    1.6798,  1.6890,  1.6982,  1.7074, 
				    1.7167,  1.7260,  1.7354,  1.7448, 
				    1.7542,  1.7637,  1.7732,  1.7828, 
				    1.7924,  1.8021,  1.8118,  1.8215, 
				    1.8313,  1.8412,  1.8510,  1.8610, 
				    1.8710,  1.8810,  1.8910,  1.9012, 
				    1.9113,  1.9216,  1.9318,  1.9421, 
				    1.9525,  1.9629,  1.9734,  1.9839, 
				    1.9945,  2.0051,  2.0158,  2.0265, 
				    2.0373,  2.0482,  2.0591,  2.0700, 
				    2.0810,  2.0921,  2.1032,  2.1144, 
				    2.1257,  2.1370,  2.1483,  2.1598, 
				    2.1712,  2.1828,  2.1944,  2.2061, 
				    2.2178,  2.2296,  2.2415,  2.2535, 
				    2.2655,  2.2776,  2.2897,  2.3019, 
				    2.3142,  2.3266,  2.3390,  2.3515, 
				    2.3641,  2.3768,  2.3895,  2.4023, 
				    2.4152,  2.4282,  2.4412,  2.4543, 
				    2.4675,  2.4808,  2.4942,  2.5077, 
				    2.5212,  2.5348,  2.5486,  2.5624, 
				    2.5763,  2.5903,  2.6044,  2.6185, 
				    2.6328,  2.6472,  2.6617,  2.6762, 
				    2.6909,  2.7057,  2.7205,  2.7355, 
				    2.7506,  2.7658,  2.7811,  2.7965, 
				    2.8121,  2.8277,  2.8435,  2.8594, 
				    2.8754,  2.8915,  2.9077,  2.9241, 
				    2.9406,  2.9572,  2.9740,  2.9909, 
				    3.0079,  3.0251,  3.0424,  3.0598, 
				    3.0774,  3.0951,  3.1130,  3.1311, 
				    3.1493,  3.1676,  3.1861,  3.2048, 
				    3.2237,  3.2427,  3.2619,  3.2812, 
				    3.3008,  3.3205,  3.3404,  3.3605, 
				    3.3808,  3.4012,  3.4219,  3.4428, 
				    3.4639,  3.4852,  3.5067,  3.5285, 
				    3.5504,  3.5726,  3.5951,  3.6177, 
				    3.6406,  3.6638,  3.6872,  3.7109, 
				    3.7349,  3.7591,  3.7836,  3.8084, 
				    3.8335,  3.8588,  3.8845,  3.9105, 
				    3.9369,  3.9635,  3.9905,  4.0179, 
				    4.0456,  4.0736,  4.1021,  4.1309, 
				    4.1602,  4.1898,  4.2199,  4.2504, 
				    4.2813,  4.3127,  4.3446,  4.3769, 
				    4.4098,  4.4431,  4.4770,  4.5115, 
				    4.5465,  4.5821,  4.6184,  4.6552, 
				    4.6927,  4.7309,  4.7698,  4.8094, 
				    4.8497,  4.8909,  4.9328,  4.9756, 
				    5.0193,  5.0639,  5.1094,  5.1560, 
				    5.2036,  5.2523,  5.3021,  5.3532, 
				    5.4055,  5.4591,  5.5141,  5.5706, 
				    5.6286,  5.6882,  5.7496,  5.8128, 
				    5.8779,  5.9451,  6.0146,  6.0863, 
				    6.1606,  6.2375,  6.3174,  6.4004, 
				    6.4868,  6.5768,  6.6708,  6.7692, 
				    6.8724,  6.9808,  7.0951,  7.2159, 
				    7.3440,  7.4803,  7.6260,  7.7825, 
				    7.9514,  8.1349,  8.3359,  8.5579, 
				    8.8058,  9.0867,  9.4105,  9.7928, 
				    10.2596, 10.8592, 11.6987, 13.1129 };
const float LaplacianSQ::yk11[] = { 0.0008,  0.0023,  0.0039,  0.0055, 
				    0.0070,  0.0086,  0.0101,  0.0117, 
				    0.0133,  0.0148,  0.0164,  0.0180, 
				    0.0195,  0.0211,  0.0227,  0.0243, 
				    0.0258,  0.0274,  0.0290,  0.0306, 
				    0.0322,  0.0337,  0.0353,  0.0369, 
				    0.0385,  0.0401,  0.0417,  0.0433, 
				    0.0448,  0.0464,  0.0480,  0.0496, 
				    0.0512,  0.0528,  0.0544,  0.0560, 
				    0.0576,  0.0592,  0.0608,  0.0624, 
				    0.0640,  0.0656,  0.0672,  0.0689, 
				    0.0705,  0.0721,  0.0737,  0.0753, 
				    0.0769,  0.0785,  0.0802,  0.0818, 
				    0.0834,  0.0850,  0.0867,  0.0883, 
				    0.0899,  0.0915,  0.0932,  0.0948, 
				    0.0964,  0.0981,  0.0997,  0.1014, 
				    0.1030,  0.1046,  0.1063,  0.1079, 
				    0.1096,  0.1112,  0.1129,  0.1145, 
				    0.1162,  0.1178,  0.1195,  0.1211, 
				    0.1228,  0.1244,  0.1261,  0.1278, 
				    0.1294,  0.1311,  0.1328,  0.1344, 
				    0.1361,  0.1378,  0.1395,  0.1411, 
				    0.1428,  0.1445,  0.1462,  0.1478, 
				    0.1495,  0.1512,  0.1529,  0.1546, 
				    0.1563,  0.1580,  0.1597,  0.1614, 
				    0.1631,  0.1648,  0.1665,  0.1682, 
				    0.1699,  0.1716,  0.1733,  0.1750, 
				    0.1767,  0.1784,  0.1801,  0.1818, 
				    0.1836,  0.1853,  0.1870,  0.1887, 
				    0.1904,  0.1922,  0.1939,  0.1956, 
				    0.1974,  0.1991,  0.2008,  0.2026, 
				    0.2043,  0.2061,  0.2078,  0.2095, 
				    0.2113,  0.2130,  0.2148,  0.2165, 
				    0.2183,  0.2201,  0.2218,  0.2236, 
				    0.2253,  0.2271,  0.2289,  0.2306, 
				    0.2324,  0.2342,  0.2359,  0.2377, 
				    0.2395,  0.2413,  0.2431,  0.2448, 
				    0.2466,  0.2484,  0.2502,  0.2520, 
				    0.2538,  0.2556,  0.2574,  0.2592, 
				    0.2610,  0.2628,  0.2646,  0.2664, 
				    0.2682,  0.2700,  0.2719,  0.2737, 
				    0.2755,  0.2773,  0.2791,  0.2810, 
				    0.2828,  0.2846,  0.2864,  0.2883, 
				    0.2901,  0.2920,  0.2938,  0.2956, 
				    0.2975,  0.2993,  0.3012,  0.3030, 
				    0.3049,  0.3068,  0.3086,  0.3105, 
				    0.3123,  0.3142,  0.3161,  0.3179, 
				    0.3198,  0.3217,  0.3236,  0.3255, 
				    0.3273,  0.3292,  0.3311,  0.3330, 
				    0.3349,  0.3368,  0.3387,  0.3406, 
				    0.3425,  0.3444,  0.3463,  0.3482, 
				    0.3501,  0.3520,  0.3539,  0.3559, 
				    0.3578,  0.3597,  0.3616,  0.3636, 
				    0.3655,  0.3674,  0.3694,  0.3713, 
				    0.3733,  0.3752,  0.3772,  0.3791, 
				    0.3811,  0.3830,  0.3850,  0.3869, 
				    0.3889,  0.3909,  0.3928,  0.3948, 
				    0.3968,  0.3988,  0.4007,  0.4027, 
				    0.4047,  0.4067,  0.4087,  0.4107, 
				    0.4127,  0.4147,  0.4167,  0.4187, 
				    0.4207,  0.4227,  0.4247,  0.4267, 
				    0.4288,  0.4308,  0.4328,  0.4348, 
				    0.4369,  0.4389,  0.4409,  0.4430, 
				    0.4450,  0.4471,  0.4491,  0.4512, 
				    0.4532,  0.4553,  0.4573,  0.4594, 
				    0.4615,  0.4635,  0.4656,  0.4677, 
				    0.4698,  0.4719,  0.4739,  0.4760, 
				    0.4781,  0.4802,  0.4823,  0.4844, 
				    0.4865,  0.4886,  0.4907,  0.4929, 
				    0.4950,  0.4971,  0.4992,  0.5013, 
				    0.5035,  0.5056,  0.5078,  0.5099, 
				    0.5120,  0.5142,  0.5163,  0.5185, 
				    0.5206,  0.5228,  0.5250,  0.5271, 
				    0.5293,  0.5315,  0.5337,  0.5359, 
				    0.5380,  0.5402,  0.5424,  0.5446, 
				    0.5468,  0.5490,  0.5512,  0.5534, 
				    0.5557,  0.5579,  0.5601,  0.5623, 
				    0.5645,  0.5668,  0.5690,  0.5713, 
				    0.5735,  0.5757,  0.5780,  0.5802, 
				    0.5825,  0.5848,  0.5870,  0.5893, 
				    0.5916,  0.5939,  0.5961,  0.5984, 
				    0.6007,  0.6030,  0.6053,  0.6076, 
				    0.6099,  0.6122,  0.6145,  0.6169, 
				    0.6192,  0.6215,  0.6238,  0.6262, 
				    0.6285,  0.6308,  0.6332,  0.6355, 
				    0.6379,  0.6403,  0.6426,  0.6450, 
				    0.6474,  0.6497,  0.6521,  0.6545, 
				    0.6569,  0.6593,  0.6617,  0.6641, 
				    0.6665,  0.6689,  0.6713,  0.6737, 
				    0.6761,  0.6786,  0.6810,  0.6834, 
				    0.6859,  0.6883,  0.6908,  0.6932, 
				    0.6957,  0.6981,  0.7006,  0.7031, 
				    0.7056,  0.7080,  0.7105,  0.7130, 
				    0.7155,  0.7180,  0.7205,  0.7230, 
				    0.7255,  0.7281,  0.7306,  0.7331, 
				    0.7356,  0.7382,  0.7407,  0.7433, 
				    0.7458,  0.7484,  0.7510,  0.7535, 
				    0.7561,  0.7587,  0.7613,  0.7638, 
				    0.7664,  0.7690,  0.7716,  0.7743, 
				    0.7769,  0.7795,  0.7821,  0.7847, 
				    0.7874,  0.7900,  0.7927,  0.7953, 
				    0.7980,  0.8006,  0.8033,  0.8060, 
				    0.8086,  0.8113,  0.8140,  0.8167, 
				    0.8194,  0.8221,  0.8248,  0.8275, 
				    0.8303,  0.8330,  0.8357,  0.8385, 
				    0.8412,  0.8440,  0.8467,  0.8495, 
				    0.8522,  0.8550,  0.8578,  0.8606, 
				    0.8634,  0.8662,  0.8690,  0.8718, 
				    0.8746,  0.8774,  0.8802,  0.8831, 
				    0.8859,  0.8887,  0.8916,  0.8945, 
				    0.8973,  0.9002,  0.9031,  0.9059, 
				    0.9088,  0.9117,  0.9146,  0.9175, 
				    0.9204,  0.9234,  0.9263,  0.9292, 
				    0.9322,  0.9351,  0.9381,  0.9410, 
				    0.9440,  0.9469,  0.9499,  0.9529, 
				    0.9559,  0.9589,  0.9619,  0.9649, 
				    0.9679,  0.9710,  0.9740,  0.9770, 
				    0.9801,  0.9831,  0.9862,  0.9893, 
				    0.9923,  0.9954,  0.9985,  1.0016, 
				    1.0047,  1.0078,  1.0109,  1.0140, 
				    1.0172,  1.0203,  1.0235,  1.0266, 
				    1.0298,  1.0329,  1.0361,  1.0393, 
				    1.0425,  1.0457,  1.0489,  1.0521, 
				    1.0553,  1.0585,  1.0618,  1.0650, 
				    1.0683,  1.0715,  1.0748,  1.0781, 
				    1.0814,  1.0847,  1.0879,  1.0913, 
				    1.0946,  1.0979,  1.1012,  1.1046, 
				    1.1079,  1.1113,  1.1146,  1.1180, 
				    1.1214,  1.1248,  1.1282,  1.1316, 
				    1.1350,  1.1384,  1.1418,  1.1453, 
				    1.1487,  1.1522,  1.1556,  1.1591, 
				    1.1626,  1.1661,  1.1696,  1.1731, 
				    1.1766,  1.1801,  1.1837,  1.1872, 
				    1.1908,  1.1943,  1.1979,  1.2015, 
				    1.2051,  1.2087,  1.2123,  1.2159, 
				    1.2196,  1.2232,  1.2268,  1.2305, 
				    1.2342,  1.2378,  1.2415,  1.2452, 
				    1.2489,  1.2526,  1.2564,  1.2601, 
				    1.2638,  1.2676,  1.2714,  1.2751, 
				    1.2789,  1.2827,  1.2865,  1.2903, 
				    1.2942,  1.2980,  1.3018,  1.3057, 
				    1.3096,  1.3134,  1.3173,  1.3212, 
				    1.3251,  1.3291,  1.3330,  1.3369, 
				    1.3409,  1.3449,  1.3488,  1.3528, 
				    1.3568,  1.3608,  1.3648,  1.3689, 
				    1.3729,  1.3770,  1.3810,  1.3851, 
				    1.3892,  1.3933,  1.3974,  1.4015, 
				    1.4057,  1.4098,  1.4140,  1.4181, 
				    1.4223,  1.4265,  1.4307,  1.4349, 
				    1.4392,  1.4434,  1.4477,  1.4519, 
				    1.4562,  1.4605,  1.4648,  1.4691, 
				    1.4735,  1.4778,  1.4822,  1.4865, 
				    1.4909,  1.4953,  1.4997,  1.5041, 
				    1.5086,  1.5130,  1.5175,  1.5219, 
				    1.5264,  1.5309,  1.5355,  1.5400, 
				    1.5445,  1.5491,  1.5536,  1.5582, 
				    1.5628,  1.5674,  1.5721,  1.5767, 
				    1.5814,  1.5860,  1.5907,  1.5954, 
				    1.6001,  1.6048,  1.6096,  1.6143, 
				    1.6191,  1.6239,  1.6287,  1.6335, 
				    1.6384,  1.6432,  1.6481,  1.6529, 
				    1.6578,  1.6627,  1.6677,  1.6726, 
				    1.6776,  1.6825,  1.6875,  1.6925, 
				    1.6975,  1.7026,  1.7076,  1.7127, 
				    1.7178,  1.7229,  1.7280,  1.7332, 
				    1.7383,  1.7435,  1.7487,  1.7539, 
				    1.7591,  1.7643,  1.7696,  1.7749, 
				    1.7802,  1.7855,  1.7908,  1.7961, 
				    1.8015,  1.8069,  1.8123,  1.8177, 
				    1.8231,  1.8286,  1.8341,  1.8396, 
				    1.8451,  1.8506,  1.8562,  1.8617, 
				    1.8673,  1.8729,  1.8786,  1.8842, 
				    1.8899,  1.8956,  1.9013,  1.9070, 
				    1.9128,  1.9185,  1.9243,  1.9301, 
				    1.9360,  1.9418,  1.9477,  1.9536, 
				    1.9595,  1.9654,  1.9714,  1.9774, 
				    1.9834,  1.9894,  1.9954,  2.0015, 
				    2.0076,  2.0137,  2.0199,  2.0260, 
				    2.0322,  2.0384,  2.0446,  2.0509, 
				    2.0572,  2.0635,  2.0698,  2.0761, 
				    2.0825,  2.0889,  2.0953,  2.1018, 
				    2.1082,  2.1147,  2.1212,  2.1278, 
				    2.1343,  2.1409,  2.1476,  2.1542, 
				    2.1609,  2.1676,  2.1743,  2.1811, 
				    2.1878,  2.1946,  2.2015,  2.2083, 
				    2.2152,  2.2221,  2.2291,  2.2361, 
				    2.2431,  2.2501,  2.2571,  2.2642, 
				    2.2713,  2.2785,  2.2857,  2.2929, 
				    2.3001,  2.3074,  2.3147,  2.3220, 
				    2.3293,  2.3367,  2.3442,  2.3516, 
				    2.3591,  2.3666,  2.3741,  2.3817, 
				    2.3893,  2.3970,  2.4047,  2.4124, 
				    2.4201,  2.4279,  2.4357,  2.4435, 
				    2.4514,  2.4593,  2.4673,  2.4753, 
				    2.4833,  2.4914,  2.4995,  2.5076, 
				    2.5158,  2.5240,  2.5322,  2.5405, 
				    2.5488,  2.5572,  2.5656,  2.5740, 
				    2.5825,  2.5910,  2.5995,  2.6081, 
				    2.6168,  2.6255,  2.6342,  2.6429, 
				    2.6517,  2.6606,  2.6695,  2.6784, 
				    2.6874,  2.6964,  2.7055,  2.7146, 
				    2.7237,  2.7329,  2.7422,  2.7515, 
				    2.7608,  2.7702,  2.7796,  2.7891, 
				    2.7986,  2.8082,  2.8178,  2.8275, 
				    2.8372,  2.8470,  2.8569,  2.8667, 
				    2.8767,  2.8867,  2.8967,  2.9068, 
				    2.9169,  2.9271,  2.9374,  2.9477, 
				    2.9581,  2.9685,  2.9790,  2.9895, 
				    3.0002,  3.0108,  3.0215,  3.0323, 
				    3.0432,  3.0541,  3.0650,  3.0761, 
				    3.0872,  3.0983,  3.1096,  3.1208, 
				    3.1322,  3.1436,  3.1551,  3.1667, 
				    3.1783,  3.1900,  3.2018,  3.2136, 
				    3.2255,  3.2375,  3.2496,  3.2617, 
				    3.2740,  3.2863,  3.2986,  3.3111, 
				    3.3236,  3.3362,  3.3489,  3.3617, 
				    3.3746,  3.3875,  3.4005,  3.4136, 
				    3.4269,  3.4401,  3.4535,  3.4670, 
				    3.4806,  3.4942,  3.5080,  3.5219, 
				    3.5358,  3.5499,  3.5640,  3.5783, 
				    3.5926,  3.6071,  3.6216,  3.6363, 
				    3.6511,  3.6660,  3.6810,  3.6961, 
				    3.7113,  3.7267,  3.7421,  3.7577, 
				    3.7734,  3.7893,  3.8052,  3.8213, 
				    3.8375,  3.8539,  3.8703,  3.8870, 
				    3.9037,  3.9206,  3.9376,  3.9548, 
				    3.9721,  3.9896,  4.0072,  4.0250, 
				    4.0430,  4.0611,  4.0793,  4.0977, 
				    4.1163,  4.1351,  4.1540,  4.1732, 
				    4.1925,  4.2119,  4.2316,  4.2515, 
				    4.2715,  4.2918,  4.3122,  4.3329, 
				    4.3537,  4.3748,  4.3961,  4.4176, 
				    4.4394,  4.4614,  4.4836,  4.5061, 
				    4.5288,  4.5517,  4.5749,  4.5984, 
				    4.6222,  4.6462,  4.6705,  4.6951, 
				    4.7200,  4.7452,  4.7707,  4.7965, 
				    4.8227,  4.8492,  4.8760,  4.9032, 
				    4.9307,  4.9586,  4.9869,  5.0155, 
				    5.0446,  5.0741,  5.1040,  5.1343, 
				    5.1651,  5.1964,  5.2281,  5.2603, 
				    5.2930,  5.3262,  5.3600,  5.3943, 
				    5.4292,  5.4647,  5.5008,  5.5375, 
				    5.5748,  5.6129,  5.6517,  5.6911, 
				    5.7314,  5.7724,  5.8142,  5.8569, 
				    5.9005,  5.9450,  5.9904,  6.0369, 
				    6.0844,  6.1330,  6.1827,  6.2337, 
				    6.2859,  6.3394,  6.3943,  6.4507, 
				    6.5086,  6.5682,  6.6295,  6.6926, 
				    6.7577,  6.8248,  6.8942,  6.9659, 
				    7.0401,  7.1170,  7.1968,  7.2797, 
				    7.3660,  7.4560,  7.5499,  7.6483, 
				    7.7514,  7.8598,  7.9740,  8.0948, 
				    8.2229,  8.3592,  8.5048,  8.6612, 
				    8.8301,  9.0136,  9.2146,  9.4365, 
				    9.6845,  9.9653, 10.2891, 10.6714, 
				    11.1382, 11.7378, 12.5773, 13.9915};
const float LaplacianSQ::yk12[] = { 0.0004,  0.0011,  0.0018,  0.0025, 
				    0.0032,  0.0039,  0.0047,  0.0054, 
				    0.0061,  0.0068,  0.0075,  0.0083, 
				    0.0090,  0.0097,  0.0104,  0.0111, 
				    0.0119,  0.0126,  0.0133,  0.0140, 
				    0.0147,  0.0155,  0.0162,  0.0169, 
				    0.0176,  0.0183,  0.0191,  0.0198, 
				    0.0205,  0.0212,  0.0220,  0.0227, 
				    0.0234,  0.0241,  0.0249,  0.0256, 
				    0.0263,  0.0270,  0.0278,  0.0285, 
				    0.0292,  0.0299,  0.0307,  0.0314, 
				    0.0321,  0.0329,  0.0336,  0.0343, 
				    0.0350,  0.0358,  0.0365,  0.0372, 
				    0.0380,  0.0387,  0.0394,  0.0402, 
				    0.0409,  0.0416,  0.0423,  0.0431, 
				    0.0438,  0.0445,  0.0453,  0.0460, 
				    0.0467,  0.0475,  0.0482,  0.0489, 
				    0.0497,  0.0504,  0.0511,  0.0519, 
				    0.0526,  0.0534,  0.0541,  0.0548, 
				    0.0556,  0.0563,  0.0570,  0.0578, 
				    0.0585,  0.0592,  0.0600,  0.0607, 
				    0.0615,  0.0622,  0.0629,  0.0637, 
				    0.0644,  0.0652,  0.0659,  0.0666, 
				    0.0674,  0.0681,  0.0689,  0.0696, 
				    0.0704,  0.0711,  0.0718,  0.0726, 
				    0.0733,  0.0741,  0.0748,  0.0756, 
				    0.0763,  0.0771,  0.0778,  0.0785, 
				    0.0793,  0.0800,  0.0808,  0.0815, 
				    0.0823,  0.0830,  0.0838,  0.0845, 
				    0.0853,  0.0860,  0.0868,  0.0875, 
				    0.0883,  0.0890,  0.0898,  0.0905, 
				    0.0913,  0.0920,  0.0928,  0.0935, 
				    0.0943,  0.0950,  0.0958,  0.0965, 
				    0.0973,  0.0980,  0.0988,  0.0996, 
				    0.1003,  0.1011,  0.1018,  0.1026, 
				    0.1033,  0.1041,  0.1049,  0.1056, 
				    0.1064,  0.1071,  0.1079,  0.1086, 
				    0.1094,  0.1102,  0.1109,  0.1117, 
				    0.1124,  0.1132,  0.1140,  0.1147, 
				    0.1155,  0.1163,  0.1170,  0.1178, 
				    0.1185,  0.1193,  0.1201,  0.1208, 
				    0.1216,  0.1224,  0.1231,  0.1239, 
				    0.1247,  0.1254,  0.1262,  0.1270, 
				    0.1277,  0.1285,  0.1293,  0.1300, 
				    0.1308,  0.1316,  0.1323,  0.1331, 
				    0.1339,  0.1347,  0.1354,  0.1362, 
				    0.1370,  0.1377,  0.1385,  0.1393, 
				    0.1401,  0.1408,  0.1416,  0.1424, 
				    0.1432,  0.1439,  0.1447,  0.1455, 
				    0.1463,  0.1470,  0.1478,  0.1486, 
				    0.1494,  0.1501,  0.1509,  0.1517, 
				    0.1525,  0.1533,  0.1540,  0.1548, 
				    0.1556,  0.1564,  0.1572,  0.1579, 
				    0.1587,  0.1595,  0.1603,  0.1611, 
				    0.1619,  0.1626,  0.1634,  0.1642, 
				    0.1650,  0.1658,  0.1666,  0.1674, 
				    0.1681,  0.1689,  0.1697,  0.1705, 
				    0.1713,  0.1721,  0.1729,  0.1737, 
				    0.1745,  0.1752,  0.1760,  0.1768, 
				    0.1776,  0.1784,  0.1792,  0.1800, 
				    0.1808,  0.1816,  0.1824,  0.1832, 
				    0.1840,  0.1848,  0.1856,  0.1864, 
				    0.1872,  0.1880,  0.1887,  0.1895, 
				    0.1903,  0.1911,  0.1919,  0.1927, 
				    0.1935,  0.1943,  0.1951,  0.1959, 
				    0.1967,  0.1975,  0.1984,  0.1992, 
				    0.2000,  0.2008,  0.2016,  0.2024, 
				    0.2032,  0.2040,  0.2048,  0.2056, 
				    0.2064,  0.2072,  0.2080,  0.2088, 
				    0.2096,  0.2104,  0.2112,  0.2121, 
				    0.2129,  0.2137,  0.2145,  0.2153, 
				    0.2161,  0.2169,  0.2177,  0.2185, 
				    0.2194,  0.2202,  0.2210,  0.2218, 
				    0.2226,  0.2234,  0.2243,  0.2251, 
				    0.2259,  0.2267,  0.2275,  0.2283, 
				    0.2292,  0.2300,  0.2308,  0.2316, 
				    0.2324,  0.2333,  0.2341,  0.2349, 
				    0.2357,  0.2365,  0.2374,  0.2382, 
				    0.2390,  0.2398,  0.2407,  0.2415, 
				    0.2423,  0.2431,  0.2440,  0.2448, 
				    0.2456,  0.2465,  0.2473,  0.2481, 
				    0.2489,  0.2498,  0.2506,  0.2514, 
				    0.2523,  0.2531,  0.2539,  0.2548, 
				    0.2556,  0.2564,  0.2573,  0.2581, 
				    0.2589,  0.2598,  0.2606,  0.2614, 
				    0.2623,  0.2631,  0.2640,  0.2648, 
				    0.2656,  0.2665,  0.2673,  0.2682, 
				    0.2690,  0.2698,  0.2707,  0.2715, 
				    0.2724,  0.2732,  0.2741,  0.2749, 
				    0.2757,  0.2766,  0.2774,  0.2783, 
				    0.2791,  0.2800,  0.2808,  0.2817, 
				    0.2825,  0.2834,  0.2842,  0.2851, 
				    0.2859,  0.2868,  0.2876,  0.2885, 
				    0.2893,  0.2902,  0.2910,  0.2919, 
				    0.2927,  0.2936,  0.2945,  0.2953, 
				    0.2962,  0.2970,  0.2979,  0.2987, 
				    0.2996,  0.3005,  0.3013,  0.3022, 
				    0.3030,  0.3039,  0.3048,  0.3056, 
				    0.3065,  0.3074,  0.3082,  0.3091, 
				    0.3099,  0.3108,  0.3117,  0.3125, 
				    0.3134,  0.3143,  0.3151,  0.3160, 
				    0.3169,  0.3178,  0.3186,  0.3195, 
				    0.3204,  0.3212,  0.3221,  0.3230, 
				    0.3239,  0.3247,  0.3256,  0.3265, 
				    0.3274,  0.3282,  0.3291,  0.3300, 
				    0.3309,  0.3317,  0.3326,  0.3335, 
				    0.3344,  0.3353,  0.3361,  0.3370, 
				    0.3379,  0.3388,  0.3397,  0.3406, 
				    0.3414,  0.3423,  0.3432,  0.3441, 
				    0.3450,  0.3459,  0.3468,  0.3476, 
				    0.3485,  0.3494,  0.3503,  0.3512, 
				    0.3521,  0.3530,  0.3539,  0.3548, 
				    0.3557,  0.3565,  0.3574,  0.3583, 
				    0.3592,  0.3601,  0.3610,  0.3619, 
				    0.3628,  0.3637,  0.3646,  0.3655, 
				    0.3664,  0.3673,  0.3682,  0.3691, 
				    0.3700,  0.3709,  0.3718,  0.3727, 
				    0.3736,  0.3745,  0.3754,  0.3763, 
				    0.3773,  0.3782,  0.3791,  0.3800, 
				    0.3809,  0.3818,  0.3827,  0.3836, 
				    0.3845,  0.3854,  0.3864,  0.3873, 
				    0.3882,  0.3891,  0.3900,  0.3909, 
				    0.3918,  0.3928,  0.3937,  0.3946, 
				    0.3955,  0.3964,  0.3973,  0.3983, 
				    0.3992,  0.4001,  0.4010,  0.4020, 
				    0.4029,  0.4038,  0.4047,  0.4056, 
				    0.4066,  0.4075,  0.4084,  0.4094, 
				    0.4103,  0.4112,  0.4121,  0.4131, 
				    0.4140,  0.4149,  0.4159,  0.4168, 
				    0.4177,  0.4187,  0.4196,  0.4205, 
				    0.4215,  0.4224,  0.4233,  0.4243, 
				    0.4252,  0.4261,  0.4271,  0.4280, 
				    0.4290,  0.4299,  0.4308,  0.4318, 
				    0.4327,  0.4337,  0.4346,  0.4356, 
				    0.4365,  0.4375,  0.4384,  0.4394, 
				    0.4403,  0.4412,  0.4422,  0.4431, 
				    0.4441,  0.4450,  0.4460,  0.4470, 
				    0.4479,  0.4489,  0.4498,  0.4508, 
				    0.4517,  0.4527,  0.4536,  0.4546, 
				    0.4555,  0.4565,  0.4575,  0.4584, 
				    0.4594,  0.4604,  0.4613,  0.4623, 
				    0.4632,  0.4642,  0.4652,  0.4661, 
				    0.4671,  0.4681,  0.4690,  0.4700, 
				    0.4710,  0.4719,  0.4729,  0.4739, 
				    0.4749,  0.4758,  0.4768,  0.4778, 
				    0.4787,  0.4797,  0.4807,  0.4817, 
				    0.4826,  0.4836,  0.4846,  0.4856, 
				    0.4866,  0.4875,  0.4885,  0.4895, 
				    0.4905,  0.4915,  0.4924,  0.4934, 
				    0.4944,  0.4954,  0.4964,  0.4974, 
				    0.4984,  0.4994,  0.5003,  0.5013, 
				    0.5023,  0.5033,  0.5043,  0.5053, 
				    0.5063,  0.5073,  0.5083,  0.5093, 
				    0.5103,  0.5113,  0.5123,  0.5133, 
				    0.5143,  0.5153,  0.5163,  0.5173, 
				    0.5183,  0.5193,  0.5203,  0.5213, 
				    0.5223,  0.5233,  0.5243,  0.5253, 
				    0.5263,  0.5273,  0.5283,  0.5293, 
				    0.5303,  0.5314,  0.5324,  0.5334, 
				    0.5344,  0.5354,  0.5364,  0.5374, 
				    0.5385,  0.5395,  0.5405,  0.5415, 
				    0.5425,  0.5435,  0.5446,  0.5456, 
				    0.5466,  0.5476,  0.5487,  0.5497, 
				    0.5507,  0.5517,  0.5528,  0.5538, 
				    0.5548,  0.5558,  0.5569,  0.5579, 
				    0.5589,  0.5600,  0.5610,  0.5620, 
				    0.5631,  0.5641,  0.5651,  0.5662, 
				    0.5672,  0.5683,  0.5693,  0.5703, 
				    0.5714,  0.5724,  0.5735,  0.5745, 
				    0.5755,  0.5766,  0.5776,  0.5787, 
				    0.5797,  0.5808,  0.5818,  0.5829, 
				    0.5839,  0.5850,  0.5860,  0.5871, 
				    0.5881,  0.5892,  0.5902,  0.5913, 
				    0.5924,  0.5934,  0.5945,  0.5955, 
				    0.5966,  0.5976,  0.5987,  0.5998, 
				    0.6008,  0.6019,  0.6030,  0.6040, 
				    0.6051,  0.6062,  0.6072,  0.6083, 
				    0.6094,  0.6104,  0.6115,  0.6126, 
				    0.6136,  0.6147,  0.6158,  0.6169, 
				    0.6179,  0.6190,  0.6201,  0.6212, 
				    0.6223,  0.6233,  0.6244,  0.6255, 
				    0.6266,  0.6277,  0.6288,  0.6298, 
				    0.6309,  0.6320,  0.6331,  0.6342, 
				    0.6353,  0.6364,  0.6375,  0.6385, 
				    0.6396,  0.6407,  0.6418,  0.6429, 
				    0.6440,  0.6451,  0.6462,  0.6473, 
				    0.6484,  0.6495,  0.6506,  0.6517, 
				    0.6528,  0.6539,  0.6550,  0.6561, 
				    0.6572,  0.6583,  0.6595,  0.6606, 
				    0.6617,  0.6628,  0.6639,  0.6650, 
				    0.6661,  0.6672,  0.6684,  0.6695, 
				    0.6706,  0.6717,  0.6728,  0.6739, 
				    0.6751,  0.6762,  0.6773,  0.6784, 
				    0.6796,  0.6807,  0.6818,  0.6829, 
				    0.6841,  0.6852,  0.6863,  0.6875, 
				    0.6886,  0.6897,  0.6909,  0.6920, 
				    0.6931,  0.6943,  0.6954,  0.6965, 
				    0.6977,  0.6988,  0.7000,  0.7011, 
				    0.7022,  0.7034,  0.7045,  0.7057, 
				    0.7068,  0.7080,  0.7091,  0.7103, 
				    0.7114,  0.7126,  0.7137,  0.7149, 
				    0.7160,  0.7172,  0.7183,  0.7195, 
				    0.7207,  0.7218,  0.7230,  0.7241, 
				    0.7253,  0.7265,  0.7276,  0.7288, 
				    0.7300,  0.7311,  0.7323,  0.7335, 
				    0.7346,  0.7358,  0.7370,  0.7382, 
				    0.7393,  0.7405,  0.7417,  0.7429, 
				    0.7440,  0.7452,  0.7464,  0.7476, 
				    0.7488,  0.7499,  0.7511,  0.7523, 
				    0.7535,  0.7547,  0.7559,  0.7571, 
				    0.7582,  0.7594,  0.7606,  0.7618, 
				    0.7630,  0.7642,  0.7654,  0.7666, 
				    0.7678,  0.7690,  0.7702,  0.7714, 
				    0.7726,  0.7738,  0.7750,  0.7762, 
				    0.7774,  0.7786,  0.7798,  0.7810, 
				    0.7823,  0.7835,  0.7847,  0.7859, 
				    0.7871,  0.7883,  0.7895,  0.7908, 
				    0.7920,  0.7932,  0.7944,  0.7956, 
				    0.7969,  0.7981,  0.7993,  0.8005, 
				    0.8018,  0.8030,  0.8042,  0.8055, 
				    0.8067,  0.8079,  0.8092,  0.8104, 
				    0.8116,  0.8129,  0.8141,  0.8154, 
				    0.8166,  0.8178,  0.8191,  0.8203, 
				    0.8216,  0.8228,  0.8241,  0.8253, 
				    0.8266,  0.8278,  0.8291,  0.8303, 
				    0.8316,  0.8328,  0.8341,  0.8353, 
				    0.8366,  0.8379,  0.8391,  0.8404, 
				    0.8417,  0.8429,  0.8442,  0.8455, 
				    0.8467,  0.8480,  0.8493,  0.8505, 
				    0.8518,  0.8531,  0.8544,  0.8556, 
				    0.8569,  0.8582,  0.8595,  0.8607, 
				    0.8620,  0.8633,  0.8646,  0.8659, 
				    0.8672,  0.8685,  0.8698,  0.8710, 
				    0.8723,  0.8736,  0.8749,  0.8762, 
				    0.8775,  0.8788,  0.8801,  0.8814, 
				    0.8827,  0.8840,  0.8853,  0.8866, 
				    0.8879,  0.8892,  0.8906,  0.8919, 
				    0.8932,  0.8945,  0.8958,  0.8971, 
				    0.8984,  0.8998,  0.9011,  0.9024, 
				    0.9037,  0.9050,  0.9064,  0.9077, 
				    0.9090,  0.9103,  0.9117,  0.9130, 
				    0.9143,  0.9157,  0.9170,  0.9183, 
				    0.9197,  0.9210,  0.9224,  0.9237, 
				    0.9250,  0.9264,  0.9277,  0.9291, 
				    0.9304,  0.9318,  0.9331,  0.9345, 
				    0.9358,  0.9372,  0.9385,  0.9399, 
				    0.9412,  0.9426,  0.9440,  0.9453, 
				    0.9467,  0.9481,  0.9494,  0.9508, 
				    0.9522,  0.9535,  0.9549,  0.9563, 
				    0.9577,  0.9590,  0.9604,  0.9618, 
				    0.9632,  0.9645,  0.9659,  0.9673, 
				    0.9687,  0.9701,  0.9715,  0.9729, 
				    0.9743,  0.9756,  0.9770,  0.9784, 
				    0.9798,  0.9812,  0.9826,  0.9840, 
				    0.9854,  0.9868,  0.9882,  0.9897, 
				    0.9911,  0.9925,  0.9939,  0.9953, 
				    0.9967,  0.9981,  0.9995,  1.0010, 
				    1.0024,  1.0038,  1.0052,  1.0067, 
				    1.0081,  1.0095,  1.0109,  1.0124, 
				    1.0138,  1.0152,  1.0167,  1.0181, 
				    1.0195,  1.0210,  1.0224,  1.0239, 
				    1.0253,  1.0268,  1.0282,  1.0296, 
				    1.0311,  1.0326,  1.0340,  1.0355, 
				    1.0369,  1.0384,  1.0398,  1.0413, 
				    1.0428,  1.0442,  1.0457,  1.0472, 
				    1.0486,  1.0501,  1.0516,  1.0530, 
				    1.0545,  1.0560,  1.0575,  1.0589, 
				    1.0604,  1.0619,  1.0634,  1.0649, 
				    1.0664,  1.0679,  1.0694,  1.0708, 
				    1.0723,  1.0738,  1.0753,  1.0768, 
				    1.0783,  1.0798,  1.0813,  1.0829, 
				    1.0844,  1.0859,  1.0874,  1.0889, 
				    1.0904,  1.0919,  1.0934,  1.0950, 
				    1.0965,  1.0980,  1.0995,  1.1011, 
				    1.1026,  1.1041,  1.1056,  1.1072, 
				    1.1087,  1.1103,  1.1118,  1.1133, 
				    1.1149,  1.1164,  1.1180,  1.1195, 
				    1.1211,  1.1226,  1.1242,  1.1257, 
				    1.1273,  1.1288,  1.1304,  1.1320, 
				    1.1335,  1.1351,  1.1366,  1.1382, 
				    1.1398,  1.1414,  1.1429,  1.1445, 
				    1.1461,  1.1477,  1.1492,  1.1508, 
				    1.1524,  1.1540,  1.1556,  1.1572, 
				    1.1588,  1.1604,  1.1620,  1.1636, 
				    1.1652,  1.1668,  1.1684,  1.1700, 
				    1.1716,  1.1732,  1.1748,  1.1764, 
				    1.1780,  1.1797,  1.1813,  1.1829, 
				    1.1845,  1.1861,  1.1878,  1.1894, 
				    1.1910,  1.1927,  1.1943,  1.1959, 
				    1.1976,  1.1992,  1.2009,  1.2025, 
				    1.2042,  1.2058,  1.2075,  1.2091, 
				    1.2108,  1.2124,  1.2141,  1.2157, 
				    1.2174,  1.2191,  1.2207,  1.2224, 
				    1.2241,  1.2258,  1.2274,  1.2291, 
				    1.2308,  1.2325,  1.2342,  1.2358, 
				    1.2375,  1.2392,  1.2409,  1.2426, 
				    1.2443,  1.2460,  1.2477,  1.2494, 
				    1.2511,  1.2528,  1.2545,  1.2563, 
				    1.2580,  1.2597,  1.2614,  1.2631, 
				    1.2649,  1.2666,  1.2683,  1.2700, 
				    1.2718,  1.2735,  1.2752,  1.2770, 
				    1.2787,  1.2805,  1.2822,  1.2840, 
				    1.2857,  1.2875,  1.2892,  1.2910, 
				    1.2927,  1.2945,  1.2963,  1.2980, 
				    1.2998,  1.3016,  1.3033,  1.3051, 
				    1.3069,  1.3087,  1.3105,  1.3123, 
				    1.3140,  1.3158,  1.3176,  1.3194, 
				    1.3212,  1.3230,  1.3248,  1.3266, 
				    1.3284,  1.3302,  1.3321,  1.3339, 
				    1.3357,  1.3375,  1.3393,  1.3412, 
				    1.3430,  1.3448,  1.3467,  1.3485, 
				    1.3503,  1.3522,  1.3540,  1.3559, 
				    1.3577,  1.3596,  1.3614,  1.3633, 
				    1.3651,  1.3670,  1.3689,  1.3707, 
				    1.3726,  1.3745,  1.3763,  1.3782, 
				    1.3801,  1.3820,  1.3839,  1.3857, 
				    1.3876,  1.3895,  1.3914,  1.3933, 
				    1.3952,  1.3971,  1.3990,  1.4010, 
				    1.4029,  1.4048,  1.4067,  1.4086, 
				    1.4105,  1.4125,  1.4144,  1.4163, 
				    1.4183,  1.4202,  1.4222,  1.4241, 
				    1.4260,  1.4280,  1.4299,  1.4319, 
				    1.4339,  1.4358,  1.4378,  1.4398, 
				    1.4417,  1.4437,  1.4457,  1.4477, 
				    1.4496,  1.4516,  1.4536,  1.4556, 
				    1.4576,  1.4596,  1.4616,  1.4636, 
				    1.4656,  1.4676,  1.4696,  1.4716, 
				    1.4737,  1.4757,  1.4777,  1.4797, 
				    1.4818,  1.4838,  1.4859,  1.4879, 
				    1.4899,  1.4920,  1.4940,  1.4961, 
				    1.4982,  1.5002,  1.5023,  1.5043, 
				    1.5064,  1.5085,  1.5106,  1.5127, 
				    1.5147,  1.5168,  1.5189,  1.5210, 
				    1.5231,  1.5252,  1.5273,  1.5294, 
				    1.5315,  1.5337,  1.5358,  1.5379, 
				    1.5400,  1.5422,  1.5443,  1.5464, 
				    1.5486,  1.5507,  1.5529,  1.5550, 
				    1.5572,  1.5593,  1.5615,  1.5636, 
				    1.5658,  1.5680,  1.5702,  1.5723, 
				    1.5745,  1.5767,  1.5789,  1.5811, 
				    1.5833,  1.5855,  1.5877,  1.5899, 
				    1.5921,  1.5944,  1.5966,  1.5988, 
				    1.6010,  1.6033,  1.6055,  1.6077, 
				    1.6100,  1.6122,  1.6145,  1.6167, 
				    1.6190,  1.6213,  1.6235,  1.6258, 
				    1.6281,  1.6304,  1.6327,  1.6349, 
				    1.6372,  1.6395,  1.6418,  1.6441, 
				    1.6465,  1.6488,  1.6511,  1.6534, 
				    1.6557,  1.6581,  1.6604,  1.6627, 
				    1.6651,  1.6674,  1.6698,  1.6722, 
				    1.6745,  1.6769,  1.6793,  1.6816, 
				    1.6840,  1.6864,  1.6888,  1.6912, 
				    1.6936,  1.6960,  1.6984,  1.7008, 
				    1.7032,  1.7056,  1.7081,  1.7105, 
				    1.7129,  1.7154,  1.7178,  1.7203, 
				    1.7227,  1.7252,  1.7276,  1.7301, 
				    1.7326,  1.7350,  1.7375,  1.7400, 
				    1.7425,  1.7450,  1.7475,  1.7500, 
				    1.7525,  1.7550,  1.7576,  1.7601, 
				    1.7626,  1.7652,  1.7677,  1.7703, 
				    1.7728,  1.7754,  1.7779,  1.7805, 
				    1.7831,  1.7856,  1.7882,  1.7908, 
				    1.7934,  1.7960,  1.7986,  1.8012, 
				    1.8038,  1.8065,  1.8091,  1.8117, 
				    1.8144,  1.8170,  1.8197,  1.8223, 
				    1.8250,  1.8276,  1.8303,  1.8330, 
				    1.8357,  1.8384,  1.8411,  1.8438, 
				    1.8465,  1.8492,  1.8519,  1.8546, 
				    1.8573,  1.8601,  1.8628,  1.8656, 
				    1.8683,  1.8711,  1.8739,  1.8766, 
				    1.8794,  1.8822,  1.8850,  1.8878, 
				    1.8906,  1.8934,  1.8962,  1.8990, 
				    1.9018,  1.9047,  1.9075,  1.9104, 
				    1.9132,  1.9161,  1.9189,  1.9218, 
				    1.9247,  1.9276,  1.9305,  1.9334, 
				    1.9363,  1.9392,  1.9421,  1.9450, 
				    1.9480,  1.9509,  1.9539,  1.9568, 
				    1.9598,  1.9627,  1.9657,  1.9687, 
				    1.9717,  1.9747,  1.9777,  1.9807, 
				    1.9837,  1.9867,  1.9898,  1.9928, 
				    1.9958,  1.9989,  2.0020,  2.0050, 
				    2.0081,  2.0112,  2.0143,  2.0174, 
				    2.0205,  2.0236,  2.0267,  2.0298, 
				    2.0330,  2.0361,  2.0393,  2.0424, 
				    2.0456,  2.0488,  2.0520,  2.0552, 
				    2.0584,  2.0616,  2.0648,  2.0680, 
				    2.0712,  2.0745,  2.0777,  2.0810, 
				    2.0842,  2.0875,  2.0908,  2.0941, 
				    2.0974,  2.1007,  2.1040,  2.1073, 
				    2.1107,  2.1140,  2.1174,  2.1207, 
				    2.1241,  2.1275,  2.1308,  2.1342, 
				    2.1376,  2.1411,  2.1445,  2.1479, 
				    2.1513,  2.1548,  2.1583,  2.1617, 
				    2.1652,  2.1687,  2.1722,  2.1757, 
				    2.1792,  2.1827,  2.1863,  2.1898, 
				    2.1934,  2.1969,  2.2005,  2.2041, 
				    2.2077,  2.2113,  2.2149,  2.2185, 
				    2.2221,  2.2258,  2.2294,  2.2331, 
				    2.2368,  2.2405,  2.2441,  2.2478, 
				    2.2516,  2.2553,  2.2590,  2.2628, 
				    2.2665,  2.2703,  2.2741,  2.2779, 
				    2.2817,  2.2855,  2.2893,  2.2931, 
				    2.2970,  2.3008,  2.3047,  2.3086, 
				    2.3125,  2.3164,  2.3203,  2.3242, 
				    2.3282,  2.3321,  2.3361,  2.3400, 
				    2.3440,  2.3480,  2.3520,  2.3561, 
				    2.3601,  2.3641,  2.3682,  2.3723, 
				    2.3763,  2.3804,  2.3845,  2.3887, 
				    2.3928,  2.3969,  2.4011,  2.4053, 
				    2.4094,  2.4136,  2.4178,  2.4221, 
				    2.4263,  2.4306,  2.4348,  2.4391, 
				    2.4434,  2.4477,  2.4520,  2.4563, 
				    2.4607,  2.4650,  2.4694,  2.4738, 
				    2.4782,  2.4826,  2.4870,  2.4915, 
				    2.4959,  2.5004,  2.5049,  2.5094, 
				    2.5139,  2.5184,  2.5230,  2.5275, 
				    2.5321,  2.5367,  2.5413,  2.5459, 
				    2.5506,  2.5552,  2.5599,  2.5646, 
				    2.5693,  2.5740,  2.5787,  2.5834, 
				    2.5882,  2.5930,  2.5978,  2.6026, 
				    2.6074,  2.6123,  2.6171,  2.6220, 
				    2.6269,  2.6318,  2.6367,  2.6417, 
				    2.6466,  2.6516,  2.6566,  2.6616, 
				    2.6666,  2.6717,  2.6768,  2.6818, 
				    2.6870,  2.6921,  2.6972,  2.7024, 
				    2.7075,  2.7127,  2.7180,  2.7232, 
				    2.7284,  2.7337,  2.7390,  2.7443, 
				    2.7496,  2.7550,  2.7604,  2.7657, 
				    2.7711,  2.7766,  2.7820,  2.7875, 
				    2.7930,  2.7985,  2.8040,  2.8096, 
				    2.8151,  2.8207,  2.8263,  2.8320, 
				    2.8376,  2.8433,  2.8490,  2.8547, 
				    2.8605,  2.8662,  2.8720,  2.8778, 
				    2.8836,  2.8895,  2.8954,  2.9013, 
				    2.9072,  2.9131,  2.9191,  2.9251, 
				    2.9311,  2.9372,  2.9432,  2.9493, 
				    2.9554,  2.9616,  2.9677,  2.9739, 
				    2.9801,  2.9863,  2.9926,  2.9989, 
				    3.0052,  3.0115,  3.0179,  3.0243, 
				    3.0307,  3.0372,  3.0436,  3.0501, 
				    3.0566,  3.0632,  3.0698,  3.0764, 
				    3.0830,  3.0897,  3.0964,  3.1031, 
				    3.1098,  3.1166,  3.1234,  3.1302, 
				    3.1371,  3.1440,  3.1509,  3.1579, 
				    3.1649,  3.1719,  3.1789,  3.1860, 
				    3.1931,  3.2002,  3.2074,  3.2146, 
				    3.2218,  3.2291,  3.2364,  3.2437, 
				    3.2511,  3.2585,  3.2659,  3.2734, 
				    3.2809,  3.2884,  3.2960,  3.3036, 
				    3.3112,  3.3189,  3.3266,  3.3344, 
				    3.3421,  3.3500,  3.3578,  3.3657, 
				    3.3736,  3.3816,  3.3896,  3.3976, 
				    3.4057,  3.4138,  3.4220,  3.4302, 
				    3.4384,  3.4467,  3.4550,  3.4634, 
				    3.4718,  3.4803,  3.4887,  3.4973, 
				    3.5058,  3.5145,  3.5231,  3.5318, 
				    3.5406,  3.5493,  3.5582,  3.5671, 
				    3.5760,  3.5850,  3.5940,  3.6030, 
				    3.6122,  3.6213,  3.6305,  3.6398, 
				    3.6491,  3.6584,  3.6679,  3.6773, 
				    3.6868,  3.6964,  3.7060,  3.7157, 
				    3.7254,  3.7351,  3.7450,  3.7548, 
				    3.7648,  3.7748,  3.7848,  3.7949, 
				    3.8051,  3.8153,  3.8256,  3.8359, 
				    3.8463,  3.8568,  3.8673,  3.8778, 
				    3.8885,  3.8992,  3.9099,  3.9208, 
				    3.9317,  3.9426,  3.9536,  3.9647, 
				    3.9759,  3.9871,  3.9984,  4.0098, 
				    4.0212,  4.0327,  4.0443,  4.0560, 
				    4.0677,  4.0795,  4.0913,  4.1033, 
				    4.1153,  4.1274,  4.1396,  4.1519, 
				    4.1642,  4.1767,  4.1892,  4.2018, 
				    4.2144,  4.2272,  4.2400,  4.2530, 
				    4.2660,  4.2791,  4.2923,  4.3056, 
				    4.3190,  4.3325,  4.3461,  4.3598, 
				    4.3736,  4.3874,  4.4014,  4.4155, 
				    4.4297,  4.4440,  4.4584,  4.4729, 
				    4.4875,  4.5022,  4.5171,  4.5320, 
				    4.5471,  4.5623,  4.5776,  4.5930, 
				    4.6086,  4.6242,  4.6400,  4.6560, 
				    4.6720,  4.6882,  4.7045,  4.7210, 
				    4.7376,  4.7544,  4.7712,  4.7883, 
				    4.8055,  4.8228,  4.8403,  4.8579, 
				    4.8757,  4.8937,  4.9118,  4.9301, 
				    4.9485,  4.9672,  4.9860,  5.0050, 
				    5.0241,  5.0435,  5.0630,  5.0827, 
				    5.1027,  5.1228,  5.1431,  5.1637, 
				    5.1844,  5.2054,  5.2265,  5.2479, 
				    5.2696,  5.2914,  5.3135,  5.3359, 
				    5.3585,  5.3813,  5.4044,  5.4278, 
				    5.4514,  5.4754,  5.4996,  5.5241, 
				    5.5489,  5.5739,  5.5993,  5.6251, 
				    5.6511,  5.6775,  5.7042,  5.7313, 
				    5.7587,  5.7865,  5.8147,  5.8433, 
				    5.8723,  5.9017,  5.9315,  5.9617, 
				    5.9924,  6.0236,  6.0552,  6.0874, 
				    6.1200,  6.1531,  6.1868,  6.2211, 
				    6.2559,  6.2913,  6.3273,  6.3639, 
				    6.4012,  6.4392,  6.4779,  6.5173, 
				    6.5575,  6.5984,  6.6402,  6.6828, 
				    6.7264,  6.7708,  6.8162,  6.8626, 
				    6.9100,  6.9586,  7.0082,  7.0591, 
				    7.1113,  7.1648,  7.2196,  7.2760, 
				    7.3339,  7.3934,  7.4546,  7.5177, 
				    7.5827,  7.6498,  7.7191,  7.7908, 
				    7.8650,  7.9418,  8.0216,  8.1045, 
				    8.1908,  8.2807,  8.3746,  8.4730, 
				    8.5761,  8.6844,  8.7987,  8.9194, 
				    9.0474,  9.1837,  9.3294,  9.4858, 
				    9.6546,  9.8381, 10.0390, 10.2610, 
				    10.5089, 10.7898, 11.1135, 11.4959, 
				    11.9626, 12.5622, 13.4017, 14.8159};

const unsigned int LaplacianSQ::Rmax = 12;
const float* LaplacianSQ::yk[] = {yk0, yk1, yk2, yk3, yk4, yk5, yk6,
                                 yk7, yk8, yk9, yk10, yk11, yk12 };
