// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

// $Id: SQuantise.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $

#include <math.h>
#include <values.h>

#include "ImageLib.h"
#include "SQuantise.h"


ScalarQuantiser::ScalarQuantiser(const QLevels& ql)
{
  uLevels = (unsigned int)ql;
  reset_distortion();
}

ScalarQuantiser::~ScalarQuantiser()
{
}

int ScalarQuantiser::IndexD(float x)
{
  float sample_diff;
  int idx;

  idx = Index(x);
  sample_diff = x - Reconstruction(idx);
  fDistortion += sample_diff*sample_diff;
  uSamples++;
  return idx;
}

float ScalarQuantiser::Q(float x)
{
  return Reconstruction(Index(x));
}


UniformSQ::UniformSQ(float ia, float ib, const QLevels& ql) :
  ScalarQuantiser(ql)
{
  reset_interval(ia, ib);
}

void UniformSQ::reset_interval(float ia, float ib)
{
  a = ia;
  b = ib;
  delta = (b - a)/(float)uLevels;
}

void UniformSQ::reset_rate(const QLevels& ql)
{
  uLevels = (unsigned int)ql;
}

int UniformSQ::Index(float x) const
{
  if (x >= a && x <= b)
    return (unsigned int)floor((x - a)/delta);
  else
    return -1;
}

float UniformSQ::Reconstruction(unsigned int idx) const
{
  return a + (float)idx*delta + delta/2;
}


StandardSQ::StandardSQ(const QLevels& ql) : ScalarQuantiser(ql)
{
  rlevel = new float [uLevels];
  for (unsigned int i = 0; i < uLevels; i++)
    rlevel[i] = 0.0;
}

StandardSQ::~StandardSQ()
{
  delete [] rlevel;
}

int StandardSQ::Index(float x) const
{
  float dist, mindist = MAXFLOAT;
  unsigned int minidx = 0;
  // Search could be more efficient
  for (unsigned int i = 0; i < uLevels; i++)
    {
      dist = fabs(x - rlevel[i]);
      if (dist < mindist)
	{
	  mindist = dist;
	  minidx = i;
	}
    }
  return minidx;
}

float StandardSQ::Reconstruction(unsigned int idx) const
{
  return rlevel[idx];
}


float StandardSQ::LloydAlgorithm(unsigned int tsize, float *training,
		                 float threshold,
				 unsigned int iteration_limit)
{
  unsigned int iteration_count = 0;
  float distortion = MAXFLOAT;
  float previous_distortion, fractional_change;

  //Check stopping criteria
  bool no_threshold = bool(threshold == 0.0);
  bool no_iteration_limit = bool(iteration_limit == 0);
  ASSERT_FATAL( !(no_threshold && no_iteration_limit) );

  //Create initial codebooks
  ASSERT_FATAL( tsize >= uLevels);
  initialise_codebook(tsize, training);
  
  do
    {
      previous_distortion = distortion;
      distortion = LloydIteration(tsize, training);
      fractional_change = (previous_distortion - distortion)/
	previous_distortion;
    }
  while ((no_threshold || (fractional_change > threshold )) &&
	 (no_iteration_limit || (++iteration_count < iteration_limit)));

  return distortion;
}

void StandardSQ::initialise_codebook(unsigned int tsize, float *training)
{
  unsigned int increment = (unsigned int)floor((float)tsize/(float)uLevels);
  for (unsigned int i = 0; i < uLevels; i++)
    rlevel[i] = training[i*increment];
  qsort(rlevel, uLevels, sizeof(float), cmp);
}
  
float StandardSQ::LloydIteration(unsigned int tsize, float *training)
{
  //Initialise centroids
  float *centroids = new float [uLevels];
  unsigned int *cell_members = new unsigned int [uLevels];
  for (unsigned int i = 0; i < uLevels; i++)
    {
      centroids[i] = 0.0;
      cell_members[i] = 0;
    }

  //Loop through training points
  unsigned int bestidx = 0;
  reset_distortion();
#if defined(NEW_FOR_LOOP_SCOPE)
  for (unsigned int i = 0; i < tsize; i++)
#else
  for (i = 0; i < tsize; i++)
#endif
    {
      //Find closest reconstruction level
      bestidx = IndexD(training[i]);
      centroids[bestidx] += training[i];
      cell_members[bestidx]++;
    }

  //Assign centroids as new reconstruction levels
#if defined(NEW_FOR_LOOP_SCOPE)
  for (unsigned int i = 0; i < uLevels; i++)
#else
  for (i = 0; i < uLevels; i++)
#endif
    rlevel[i] = centroids[i]/(float)cell_members[i];

  delete [] centroids;
  delete [] cell_members;

  return Distortion();
}


int StandardSQ::cmp(const void * ap, const void * bp)
{
  float a = *(float *)ap;
  float b = *(float *)bp;

  if (a > b)
    return 1;
  if (a < b)
    return -1;
  return 0;
}


bool StandardSQ::save(char *filename) const
{
  ofstream ofs(filename, IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  bool status = save(ofs);
  ofs.close();
  return status;
}

bool StandardSQ::save(ofstream& ofs) const
{
  if (!ofs.write((char *)&uLevels, sizeof(unsigned int)))
    return false;
  else
    {
      if (!ofs.write((char *)rlevel, uLevels*sizeof(float)))
	return false;
      else
	return true;
    }
}

bool StandardSQ::load(char *filename)
{
  ifstream ifs(filename, IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;

  bool status = load(ifs);
  ifs.close();
  return status;
}

bool StandardSQ::load(ifstream& ifs)
{
  unsigned int tmpLevels;
  if (!ifs.read((char *)&tmpLevels, sizeof(unsigned int)))
    return false;
  else
    {
      float *tmprlevel = new float [tmpLevels];
      if (!ifs.read((char *)tmprlevel, sizeof(float)*tmpLevels))
	{
	  delete [] tmprlevel;
	  return false;
	}
      else
	{
	  delete [] rlevel;
	  rlevel = tmprlevel;
	  uLevels = tmpLevels;
	  return true;
	}
    }
}
