// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

// $Id: SQuantise.h,v 1.2 2005/10/06 14:46:02 brendt Exp $

#ifndef _ScalarQuantiser_h
#define _ScalarQuantiser_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
using std::ofstream;
using std::ifstream;
using std::cerr;
using std::endl;
#else
#include <iostream.h>
#include <fstream.h>
#endif

#include "QLevels.h"

//! unit="Quantisation"
//! module="Scalar Quantisation" 
//! level = 1

//: Interface for Scalar Quantisers
// ScalarQuantiser provides an interface for all scalar quantisers.
//   The number of reconstruction levels are specified on construction by
//   a parameter of type QLevels.
//   Rate and Levels return the bit rate and number of reconstruction
//   levels respectively.
//   Index returns the index of the closest reconstruction level, while
//   Reconstruction returns the reconstruction value for a specified
//   index. Q returns the quantised value of its parameter.
//   IndexD is the same as Index except that a record of quantisation
//   distortion is maintained. Distortion returns the current average
//   value of this distortion, while reset_distortion resets the record.

class ScalarQuantiser {
private:
  unsigned int uSamples;
  float fDistortion;
  
protected:
  unsigned int uLevels;
  
public:
  ScalarQuantiser(const QLevels& ql);
  virtual ~ScalarQuantiser();
  // Reset the average distortion count maintained by IndexD
  void reset_distortion() { uSamples = 0; fDistortion = 0.0; }
  // Return the bit rate of the quantiser
  float Rate(void) const { return log2((float)uLevels); }
  // Return the number of reconstruction levels of the quantiser  
  unsigned int Levels(void) const { return uLevels; }
  // Return the index of the nearest reconstruction level to x  
  virtual int Index(float x) const = 0;
  // Same as Index, but maintains a record of average distortion  
  virtual int IndexD(float x);
  // Return quantised value of x  
  virtual float Q(float x);
  // Return reconstruction level corresponding to index idx  
  virtual float Reconstruction(unsigned int idx) const = 0;
  // Return average distortion in calls to IndexD  
  float Distortion() const { return fDistortion/(float)uSamples; }
    
};

//! level = 2

//: Uniform Scalar Quantiser
// UniformSQ is a uniform scalar quantiser. Left and right interval
//   bounds and number of reconstruction levels are specified on
//   construction

class UniformSQ : public ScalarQuantiser {
private:
  float a;
  float b;
  float delta;

public:
  //!param: ia - left interval bound
  //!param: ib - right interval bound
  //!param: ql - number of quantisation levels
  UniformSQ(float ia = -sqrt(3.0), float ib = sqrt(3.0),
	    const QLevels& ql = QRate(1));
  ~UniformSQ() {}
  // Reset quantisation interval to (ia,ib)
  void reset_interval(float ia, float ib);
  // Reset coding rate
  void reset_rate(const QLevels& ql); 
  int Index(float x) const;
  float Reconstruction(unsigned int idx) const;
};


//: Lloyd-Max Scalar Quantiser
// StandardSQ is a Lloyd-Max scalar quantiser. The number of
//   reconstruction levels is specified on construction, and
//   LloydAlgorithm allows the quantiser to be trained for efficient
//   quatisation of a distribution represented by a training set. The
//   save and load members allow a trained quantiser to be saved to disk
//   for subsequent re-use.


class StandardSQ : public ScalarQuantiser {
private:
  float *rlevel;

  static int cmp(const void *, const void *);

public:
  StandardSQ(const QLevels& ql);
  ~StandardSQ();
  int Index(float x) const;
  float Reconstruction(unsigned int idx) const;
  // Use Lloyd Algorithm to generate a codebook from the training data
  float LloydAlgorithm(unsigned int tsize, float *training,
		       float theshold, unsigned int iteration_limit =0);
  void initialise_codebook(unsigned int tsize, float *training);
  // Single iteration of the Lloyd Algorithm
  float LloydIteration(unsigned int tsize, float *training);
  bool save(char *filename) const;
  bool save(ofstream& ofs) const;
  bool load(char *filename);
  bool load(ifstream& ifs);
};

#endif
