// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

#include <values.h>
#include <math.h>

#include "VQuantise.h"

VectorQuantiser::VectorQuantiser()
{
  uDimension = 0;
  uLevels = 0;
  reset_distortion();
}

VectorQuantiser::VectorQuantiser(const QLevels& ql, unsigned int d)
{
  uDimension = d;
  uLevels = (unsigned int)ql;
  reset_distortion();
}

VectorQuantiser::VectorQuantiser(const QRate& qr, unsigned int d)
{
  uDimension = d;
  uLevels = 1 << (d*qr.Rate());
  reset_distortion();
}


VectorQuantiser::~VectorQuantiser()
{
}

int VectorQuantiser::IndexD(const floatVector& x)
{
  int idx = Index(x);
  fDistortion += MSE(x, Reconstruction(idx));
  uSamples++;
  return idx;
}


floatVectorRef VectorQuantiser::Q(const floatVector& x)
{
  return Reconstruction(Index(x));
}


floatVectorList::floatVectorList(unsigned int maxsize)
{
  list_alloc = maxsize;
  list_size = 0;
  list = new floatVector* [list_alloc];
}

floatVectorList::~floatVectorList()
{
  unsigned int i;
  for (i = 0; i < list_size; i++)
    delete list[i];
  delete [] list;
}

int floatVectorList::insert(const floatVector &fv)
{
  if (list_size >= list_alloc)
    return -1;
  else
    {
      floatVector *newvec = new floatVector;
      (*newvec) = fv;
      list[list_size++] = newvec;
      return list_size;
    }
}

void floatVectorList::remove(unsigned int index)
{
  if (list_size > 0 && index < list_size)
    {
      delete list[index];
      if (index < list_size-1)   
	list[index] = list[list_size-1];
      list_size--;
    }
}

floatVector& floatVectorList::entry(unsigned int index) const
{
  ASSERT_FATAL( index < list_size );
  return *list[index];
}

unsigned int floatVectorList::nearest_neighbour(const floatVector &fv)
const
{
  float distortion, best_distortion = MAXFLOAT;
  unsigned int i,bestindex = 0;
  for (i = 0; i < list_size; i++)
    	if ((distortion = MSE(fv, (*list[i])))
	    < best_distortion)
	  {
	    best_distortion = distortion;
	    bestindex = i;
	  }
  return bestindex;
}


bool floatVectorList::save(ofstream& ofs) const
{
  unsigned int header[2];
  header[0] = list_size;
  header[1] = entry(0).vlength();

  if (!ofs.write((char *)header, sizeof(header)))
    {
      ofs.close();
      return false;
    }

  for (unsigned int i = 0; i < list_size; i++)
    if (!entry(i).raw_write(ofs))
      {
      ofs.close();
      return false;
    }
  
  ofs.close();
  return true;
}

bool floatVectorList::save(char *filename) const
{
  ofstream ofs(filename, IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  bool status = save(ofs);
  ofs.close();
  return status;
}

bool floatVectorList::load(ifstream& ifs)
{
  unsigned int header[2];
  if (!ifs.read((char *)header, sizeof(header)))
    {
      ifs.close();
      return false;
    }

  floatVector **load_list = new floatVector* [header[0]];
  if (!load_list)
    return false;

  unsigned int i;
  for (i = 0; i < header[0]; i++)
    {
      floatVector *newvec = new floatVector(header[1]);
      if (!newvec->raw_read(ifs))
	{
	  delete newvec;
	  for (unsigned int j = 0; j < i; j++)
	    delete load_list[j];
	  delete [] load_list;
	  return false;
	}
      load_list[i] = newvec;
    }

  for (i = 0; i < list_size; i++)
    delete list[i];
  delete [] list;

  list_size = header[0];
  list_alloc = header[0];
  list = load_list;
  
  ifs.close();
  return true;
}

bool floatVectorList::load(char *filename)
{
  ifstream ifs(filename, IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;

  bool status = load(ifs);
  ifs.close();
  return status;
}


float StandardVQ::GLA(floatVectorList &training, float threshold,
		      unsigned int iteration_limit)
{
  unsigned int iteration_count = 0;
  float distortion = MAXFLOAT;
  float previous_distortion, fractional_change;

  //Check dimensionality
  for (unsigned int i = 0; i < training.entries(); i++)
    {
      ASSERT_FATAL( training.entry(i).vlength() == uDimension );
    }
  
  //Check stopping criteria
  bool no_threshold = bool(threshold == 0.0);
  bool no_iteration_limit = bool(iteration_limit == 0);
  ASSERT_FATAL( !(no_threshold && no_iteration_limit) );

  //Create initial codebook
  ASSERT_FATAL( training.entries() >= uLevels);
  unsigned int increment = (unsigned int)
    floor((float)training.entries()/(float)uLevels);
#if defined(NEW_FOR_LOOP_SCOPE)
  for (unsigned int i = 0; i < uLevels; i++)
#else
  for (i = 0; i < uLevels; i++)
#endif
    rlevel.insert(training.entry(i*increment));

  //Repeat Lloyd iteration
  do
    {
      previous_distortion = distortion;
      distortion = LloydIteration(training);
      fractional_change = (previous_distortion - distortion)/
	previous_distortion;
    }
  while ((no_threshold || (fractional_change > threshold )) &&
	 (no_iteration_limit || (++iteration_count < iteration_limit)));

  return distortion;
}


float StandardVQ::LloydIteration(floatVectorList &training)
{
  bool empty_cell_detected = false;
  unsigned int i,j;
  unsigned int cells = rlevel.entries();
  float distortion = 0.0;
  floatVector* centroids = new floatVector [cells];
  unsigned int* cell_members = new unsigned int [cells];

  do {
    distortion = 0.0;
    float worsttraindist = 0.0;
    unsigned int worsttrainidx = 0;
    
  //Initialise centroid array
  for (i = 0; i < cells; i++)
    {
      cell_members[i] = 0;
      centroids[i] = floatVector(rlevel.entry(i).vlength(), 0.0);
    }

  //Loop through all training vectors
  for (i = 0; i < training.entries(); i++)
    {
      //Find closest codebook vector
      float dist, bestdist = MAXFLOAT;
      unsigned int bestindex = 0;
      for (j = 0; j < cells; j++)
	if ((dist = MSE(training.entry(i), rlevel.entry(j)))
	    < bestdist)
	  {
	    bestdist = dist;
	    bestindex = j;
	  }
      
      //Update distortion and index for worst represented training vector
	if (bestdist > worsttraindist)
	  {
	    worsttraindist = bestdist;
	    worsttrainidx = i;
	  }

      //Update distortion and centroids
      distortion += bestdist;
      cell_members[bestindex]++;
      centroids[bestindex] = centroids[bestindex] + training.entry(i);
    }

  //Check for empty cells
    empty_cell_detected = false;
    for (j = 0; j < cells; j++)
      if ( cell_members[j] == 0)
	{
	  empty_cell_detected = true;
	  rlevel.entry(j) = training.entry(worsttrainidx);
	  cout << "Warning: Empty cell detected for index " << j << endl;
	  break;
	}

  } while (empty_cell_detected);
  
  //Assign centroids as new codevectors
  for (j = 0; j < cells; j++)
    rlevel.entry(j) = (1.0f/(float)cell_members[j]) * centroids[j];

  delete [] centroids;
  delete [] cell_members;
  
  cout << "Distortion: " << setw(14) << setprecision(1)
       << distortion << endl;
  return distortion;
}


bool StandardVQ::save(char *filename) const
{
  return rlevel.save(filename);
}

bool StandardVQ::save(ofstream& ofs) const
{
  return rlevel.save(ofs);
}

bool StandardVQ::load(char *filename)
{
  if (rlevel.load(filename))
    {
      uLevels = rlevel.entries();
      uDimension = rlevel.entry(0).vlength(); //kludge
      return true;
    }
  else
    return false;
}

bool StandardVQ::load(ifstream& ifs)
{
  if (rlevel.load(ifs))
    {
      uLevels = rlevel.entries();
      uDimension = rlevel.entry(0).vlength(); //kludge
      return true;
    }
  else
    return false;
}


float NormalisedVQ::GLA(floatVectorList &training, float threshold,
		       unsigned int iteration_limit)
{
  unsigned int iteration_count = 0;
  float distortion = MAXFLOAT;
  float previous_distortion, fractional_change;

  //Check dimensionality
  for (unsigned int i = 0; i < training.entries(); i++)
    {
      ASSERT_FATAL( training.entry(i).vlength() == uDimension );
    }
  
  //Check stopping criteria
  bool no_threshold = bool(threshold == 0.0);
  bool no_iteration_limit = bool(iteration_limit == 0);
  ASSERT_FATAL( !(no_threshold && no_iteration_limit) );

  //Normalise training vectors
#if defined(NEW_FOR_LOOP_SCOPE)
  for (unsigned int i = 0; i < training.entries(); i++)
#else
  for (i = 0; i < training.entries(); i++)
#endif
    training.entry(i) =
      (1.0f/training.entry(i).normL2())*
      training.entry(i);

   //Create initial codebook
  ASSERT_FATAL( training.entries() >= uLevels);
  unsigned int increment = (unsigned int)
    floor((float)training.entries()/(float)uLevels);
#if defined(NEW_FOR_LOOP_SCOPE)
  for (unsigned int i = 0; i < uLevels; i++)
#else
  for (i = 0; i < uLevels; i++)
#endif
    rlevel.insert(training.entry(i*increment));
  
  do
    {
      previous_distortion = distortion;
      distortion = LloydIteration(training);
      fractional_change = fabs(distortion - previous_distortion)/
	previous_distortion;
    }
  while ((no_threshold || (fractional_change > threshold )) &&
	 (no_iteration_limit || (++iteration_count < iteration_limit)));

  return distortion;
}


float NormalisedVQ::LloydIteration(floatVectorList &training) 
{
  bool empty_cell_detected = false;
  unsigned int i,j;
  unsigned int cells = rlevel.entries();
  floatVector* centroids = new floatVector [cells];
  unsigned int* cell_members = new unsigned int [cells];
  float distortion = 0.0;
    
  do {
    distortion = 0.0;
    float worsttraindist = 1.0;
    unsigned int worsttrainidx = 0;
  
    //Initialise centroid array
    for (i = 0; i < cells; i++)
      {
	cell_members[i] = 0;
	centroids[i] = floatVector(rlevel.entry(i).vlength(), 0.0);
      }

    //Loop through all training vectors
    for (i = 0; i < training.entries(); i++)
      {
	//Find closest codebook vector
	float dist, bestdist = -1.0;
	unsigned int bestindex = 0;
	for (j = 0; j < cells; j++) //fabs here doesn't matter - as good
	  //as squaring
	  if ((dist = fabs(correlation(training.entry(i), rlevel.entry(j))))
	      > bestdist)
	    {
	      bestdist = dist;
	      bestindex = j;
	    }
	//Update distortion and index for worst represented training vector
	if (bestdist < worsttraindist)
	  {
	    worsttraindist = bestdist;
	    worsttrainidx = i;
	  }
      
	//Update distortion and centroids
	distortion += bestdist;
	cell_members[bestindex]++;

	float alpha = correlation(rlevel.entry(bestindex), training.entry(i));
	centroids[bestindex] = centroids[bestindex] + alpha*training.entry(i);
      }

    //Check for empty cells
    empty_cell_detected = false;
    for (j = 0; j < cells; j++)
      if ( cell_members[j] == 0)
	{
	  empty_cell_detected = true;
	  rlevel.entry(j) = training.entry(worsttrainidx);
	  cout << "Warning: Empty cell detected for index " << j << endl;
	  break;
	}
  } while (empty_cell_detected);

  //Assign normalised centroids as new codevectors
  for (j = 0; j < cells; j++)
    rlevel.entry(j) = (1.0f/centroids[j].normL2()) * centroids[j];
  
  cout << "Angular Distortion: " << distortion/((float)training.entries())
       << endl;

  delete [] centroids;
  delete [] cell_members;
  
  return distortion;
}



float NormalisedVQ::correlation(const floatVector &a, const floatVector &b)
{
  float ab = inproduct(a, b);
  float an = a.normL2();
  float bn = b.normL2();

  if (ab == 0.0)
    return 0.0;
  else
  return ab/(an*bn);
}


int NormalisedVQ::Index(const floatVector& x) const
{
  float corr, maxcorr = 0.0;
  unsigned int bestidx = 0;
  for (unsigned int i = 0; i < rlevel.entries(); i++)
    {
      corr = correlation(x, rlevel.entry(i));
      if (corr * corr > maxcorr)
	{
	  bestidx = i;
	  maxcorr = corr*corr;
	}
    }
  return bestidx;
}


bool NormalisedVQ::save(char *filename) const
{
  return rlevel.save(filename);
}

bool NormalisedVQ::save(ofstream& ofs) const
{
  return rlevel.save(ofs);
}

bool NormalisedVQ::load(char *filename)
{
  if (rlevel.load(filename))
    {
      uLevels = rlevel.entries();
      uDimension = rlevel.entry(0).vlength(); //kludge
      return true;
    }
  else
    return false;
}

bool NormalisedVQ::load(ifstream& ifs)
{
  if (rlevel.load(ifs))
    {
      uLevels = rlevel.entries();
      uDimension = rlevel.entry(0).vlength(); //kludge
      return true;
    }
  else
    return false;
}


unsigned int ShapeGainVQ::Levels(void) const
{
  return scalarq.Levels() * NormalisedVQ::Levels();
}

float ShapeGainVQ::Rate(void) const
{
  return IndexBits()/(float)uDimension;
}

float ShapeGainVQ::IndexBits(void) const
{
  return scalarq.Rate() + NormalisedVQ::IndexBits();
}

int ShapeGainVQ::Index(const floatVector& x) const
{
  //See Gersho & Gray pp. 441-450
  unsigned int sindex = NormalisedVQ::Index(x);
  float scalar = inproduct(x, NormalisedVQ::Reconstruction(sindex));
  unsigned int gindex = scalarq.Index(scalar);
  unsigned int div = (1 << (unsigned int)scalarq.Rate());
  return gindex + sindex*div;
}

floatVectorRef ShapeGainVQ::Reconstruction(unsigned int idx) const
{
  //Extract separate shape and gain indices
  unsigned int div = (1 << (unsigned int)scalarq.Rate());
  unsigned int gindex = idx % div;
  unsigned int sindex = idx / div;
  return scalarq.Reconstruction(gindex) *
    NormalisedVQ::Reconstruction(sindex);
}

float ShapeGainVQ::GLA(floatVectorList &training, float threshold,
		       unsigned int iteration_limit)
{
  unsigned int iteration_count = 0;
  float distortion = MAXFLOAT;
  float previous_distortion, fractional_change;

  //Check dimensionality
  for (unsigned int i = 0; i < training.entries(); i++)
    {
      ASSERT_FATAL( training.entry(i).vlength() == uDimension );
    }
  
  //Check stopping criteria
  bool no_threshold = bool(threshold == 0.0);
  bool no_iteration_limit = bool(iteration_limit == 0);
  ASSERT_FATAL( !(no_threshold && no_iteration_limit) );

  //Normalise training vectors and create gain training set
  float *gtraining = new float [training.entries()];
#if defined(NEW_FOR_LOOP_SCOPE)
  for (unsigned int i = 0; i < training.entries(); i++)
#else
  for (i = 0; i < training.entries(); i++)
#endif
    {
      gtraining[i] = training.entry(i).normL2();
      training.entry(i) = training.entry(i)/gtraining[i];
    }

   //Create initial shape codebook
  ASSERT_FATAL( training.entries() >= uLevels);
  unsigned int increment = (unsigned int)
    floor((float)training.entries()/(float)uLevels);
#if defined(NEW_FOR_LOOP_SCOPE)
  for (unsigned int i = 0; i < uLevels; i++)
#else
  for (i = 0; i < uLevels; i++)
#endif
    ASSERT_FATAL( rlevel.insert(training.entry(i*increment)) != -1);

  //Create initial gain codebook
  ASSERT_FATAL( training.entries() >= scalarq.Levels() );
  scalarq.initialise_codebook(training.entries(), gtraining);
  
  do
    {
      previous_distortion = distortion; 
      distortion = NormalisedVQ::LloydIteration(training);
      scalarq.LloydIteration(training.entries(), gtraining);
      fractional_change = fabs(distortion - previous_distortion)/
	previous_distortion;
    }
  while ((no_threshold || (fractional_change > threshold )) &&
	 (no_iteration_limit || (++iteration_count < iteration_limit)));

  delete [] gtraining;
  
  return distortion;
}


bool ShapeGainVQ::save(char *filename) const
{
  ofstream ofs(filename, IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  bool status = save(ofs);
  ofs.close();
  return status;
}

bool ShapeGainVQ::save(ofstream& ofs) const
{
  if (!scalarq.save(ofs))
    return false;
  else
    return NormalisedVQ::save(ofs);
}

bool ShapeGainVQ::load(char *filename)
{
  ifstream ifs(filename, IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;

  bool status = load(ifs);
  ifs.close();
  return status;
}

bool ShapeGainVQ::load(ifstream& ifs)
{
  if (!scalarq.load(ifs))
    return false;
  else
    return NormalisedVQ::load(ifs);
}



float ZeroMeanNormalisedVQ::GLA(floatVectorList &training, float threshold,
				unsigned int iteration_limit)
{
  unsigned int iteration_count = 0;
  float distortion = MAXFLOAT;
  float previous_distortion, fractional_change;

  //Check dimensionality
  for (unsigned int i = 0; i < training.entries(); i++)
    {
      ASSERT_FATAL( training.entry(i).vlength() == uDimension );
    }
  
  //Check stopping criteria
  bool no_threshold = bool(threshold == 0.0);
  bool no_iteration_limit = bool(iteration_limit == 0);
  ASSERT_FATAL( !(no_threshold && no_iteration_limit) );

  //Remove mean from training vectors
#if defined(NEW_FOR_LOOP_SCOPE)
  for (unsigned int i = 0; i < training.entries(); i++)
#else
  for (i = 0; i < training.entries(); i++)
#endif
    {
      training.entry(i) = training.entry(i) -
	training.entry(i).origin_moment(1);
      float norm = training.entry(i).normL2();
      if (norm > 0.0)
	{
	  training.entry(i) /= norm;
	  i++;
	}
      else
	training.remove(i);
    }

   //Create initial codebook
  ASSERT_FATAL( training.entries() >= uLevels);
  unsigned int increment = (unsigned int)
    floor((float)training.entries()/(float)uLevels);
#if defined(NEW_FOR_LOOP_SCOPE)
  for (unsigned int i = 0; i < uLevels; i++)
#else
  for (i = 0; i < uLevels; i++)
#endif
    //NB: Should ensure that all initial codebook entries are unique
    if (training.entry(i*increment).normL2() > 0.0) //Poor kludge
      rlevel.insert(training.entry(i*increment));
    else
      {
	cout << "Warning: entry " << i << " had zero norm\n";
	rlevel.insert(training.entry(i*increment+1));
      }
  
  do
    {
      previous_distortion = distortion;
      distortion = LloydIteration(training);
      fractional_change = fabs(distortion - previous_distortion)/
	previous_distortion;
    }
  while ((no_threshold || (fractional_change > threshold )) &&
	 (no_iteration_limit || (++iteration_count < iteration_limit)));

  cout << "Done\n";

  return distortion;
}


float ZeroMeanNormalisedVQ::LloydIteration(floatVectorList &training) 
{
  bool empty_cell_detected = false;
  unsigned int i,j;
  unsigned int cells = rlevel.entries();
  floatVector* centroids = new floatVector [cells];
  unsigned int* cell_members = new unsigned int [cells];
  float distortion = 0.0;
    
  do {
    distortion = 0.0;
    float worsttraindist = 1.0;
    unsigned int worsttrainidx = 0;

    //Initialise centroid array
    for (i = 0; i < cells; i++)
      {
	cell_members[i] = 0;
	centroids[i] = floatVector(rlevel.entry(i).vlength(), 0.0);
      }

    //Loop through all training vectors
    for (i = 0; i < training.entries(); i++)
      {
	//Find closest codebook vector
	float dist, bestdist = -1.0;
	unsigned int bestindex = 0;
	for (j = 0; j < cells; j++) //fabs here doesn't matter - as good
	  //as squaring
	  if ((dist = fabs(correlation(rlevel.entry(j), training.entry(i))))
	      > bestdist)
	    {
	      bestdist = dist;
	      bestindex = j;
	    }

	//Update distortion and index for worst represented training vector
	if (bestdist < worsttraindist)
	  {
	    worsttraindist = bestdist;
	    worsttrainidx = i;
	  }
      
	//Update distortion and centroids
	distortion += bestdist;
	cell_members[bestindex]++;

	float alpha = correlation(rlevel.entry(bestindex), training.entry(i));
	centroids[bestindex] += alpha*training.entry(i);
      }

    //Check for empty cells
    empty_cell_detected = false;
    for (j = 0; j < cells; j++)
      if ( cell_members[j] == 0)
	{
	  empty_cell_detected = true;
	  rlevel.entry(j) = training.entry(worsttrainidx);
	  cout << "Warning: Empty cell detected for index " << j 
	       << " assigning training index " << worsttrainidx
	       << endl;
	  break;
	}
  } while (empty_cell_detected);

  //Remove means, normalise and assign centroids as new codevectors
  for (j = 0; j < cells; j++)
    {
      centroids[j] = centroids[j] - centroids[j].origin_moment(1);
      centroids[j] = (1.0f/centroids[j].normL2()) * centroids[j];
      rlevel.entry(j) = centroids[j];
    }

  cout << "Angular Distortion: " << distortion/((float)training.entries())
       << endl;

  delete [] centroids;
  delete [] cell_members;
  
  return distortion;
}



float ZeroMeanNormalisedVQ::correlation(const floatVector &a, 
					const floatVector &b)
{
  floatVector u,v;

  u = a - a.origin_moment(1);
  v = b - b.origin_moment(1);

  float uv = inproduct(u, v);
  float un = u.normL2();
  float vn = v.normL2();

  if (uv == 0.0)
    return 0.0;
  else
  return uv/(un*vn);
}


int ZeroMeanNormalisedVQ::Index(const floatVector& x) const
{
  float corr, maxcorr = 0.0;
  unsigned int bestidx = 0;
  for (unsigned int i = 0; i < rlevel.entries(); i++)
    {
      corr = correlation(x, rlevel.entry(i));
      if (corr * corr > maxcorr)
	{
	  bestidx = i;
	  maxcorr = corr*corr;
	}
    }
  return bestidx;
}


bool ZeroMeanNormalisedVQ::save(char *filename) const
{
  return rlevel.save(filename);
}

bool ZeroMeanNormalisedVQ::save(ofstream& ofs) const
{
  return rlevel.save(ofs);
}

bool ZeroMeanNormalisedVQ::load(char *filename)
{
  if (rlevel.load(filename))
    {
      uLevels = rlevel.entries();
      uDimension = rlevel.entry(0).vlength(); //kludge
      return true;
    }
  else
    return false;
}

bool ZeroMeanNormalisedVQ::load(ifstream& ifs)
{
  if (rlevel.load(ifs))
    {
      uLevels = rlevel.entries();
      uDimension = rlevel.entry(0).vlength(); //kludge
      return true;
    }
  else
    return false;
}
