// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
using std::ofstream;
using std::ifstream;
using std::cout;
using std::cerr;
using std::endl;
#else
#include <iostream.h>
#include <fstream.h>
#endif

#include "floatVector.h"
#include "QLevels.h"
#include "SQuantise.h"

//! unit="Quantisation"
//! module="Vector Quantisation" 
//! level = 1


//: Abstract interface for Vector Quantisers
class VectorQuantiser {
private:
  unsigned int uSamples;
  float fDistortion;
  
protected:
  unsigned int uDimension;
  unsigned int uLevels;
  
public:
  VectorQuantiser();
  VectorQuantiser(const QLevels& ql, unsigned int d);
  VectorQuantiser(const QRate& qr, unsigned int d);
  virtual ~VectorQuantiser();
  void reset_distortion() { uSamples = 0; fDistortion = 0.0; }
  virtual unsigned int Levels(void) const { return uLevels; }
  virtual float Rate(void) const
   { return (log2((float)uLevels))/(float)uDimension; }
  virtual float IndexBits(void) const
   { return log2((float)uLevels); }
  virtual int Index(const floatVector& x) const = 0;
  virtual int IndexD(const floatVector& x);
  virtual floatVectorRef Q(const floatVector& x);
  virtual floatVectorRef Reconstruction(unsigned int idx) const = 0;
  float Distortion() const { return fDistortion/(float)uSamples; }
};

//! level = 0

class floatVectorList {
private:
  floatVector **list;
  unsigned int list_alloc;
  unsigned int list_size;

public:
  floatVectorList(unsigned int maxsize);
  ~floatVectorList();

  int insert(const floatVector &fv);
  void remove(unsigned int index);
  floatVector& entry(unsigned int index) const;
  unsigned int entries() const { return list_size; }
  unsigned int nearest_neighbour(const floatVector &fv) const;

  bool save(char *filename) const;
  bool save(ofstream& ofs) const;
  bool load(char *filename);
  bool load(ifstream& ifs);
};

//! level = 3

//: Standard VQ
class StandardVQ : public VectorQuantiser {
private:
  floatVectorList rlevel;
  
public:
  StandardVQ() : VectorQuantiser(), rlevel(0) {}
  StandardVQ(const QLevels& ql, unsigned int d) :
    VectorQuantiser(ql, d), rlevel((unsigned int)ql) {}
  StandardVQ(const QRate& qr, unsigned int d) :
    VectorQuantiser(qr, d), rlevel(1 << (d*qr.Rate())) {}
  int Index(const floatVector& x) const
  { return rlevel.nearest_neighbour(x); }
  floatVectorRef Reconstruction(unsigned int idx) const
  {
    floatVector rec = rlevel.entry(idx);
    return rec;
  }
  float GLA(floatVectorList &training, float threshold,
	    unsigned int iteration_limit = 0);
  float LloydIteration(floatVectorList &training);
  bool save(char *filename) const;
  bool save(ofstream& ofs) const;
  bool load(char *filename);
  bool load(ifstream& ifs);
};


//: Normalised VQ - ShapeGain without qain quantisation
class NormalisedVQ : public VectorQuantiser {
protected:
  floatVectorList rlevel;

  static float correlation(const floatVector &a, const floatVector &b);
  
public:
  NormalisedVQ() : VectorQuantiser(), rlevel(0) {}
  NormalisedVQ(const QLevels& ql, unsigned int d) :
    VectorQuantiser(ql, d), rlevel((unsigned int)ql) {};
  NormalisedVQ(const QRate& qr, unsigned int d) :
    VectorQuantiser(qr, d), rlevel(1 << (d*qr.Rate())) {};
  int Index(const floatVector& x) const;
  floatVectorRef Reconstruction(unsigned int idx) const
  {
    floatVector rec = rlevel.entry(idx);
    return rec;
  }
  float GLA(floatVectorList &training, float threshold,
	    unsigned int iteration_limit = 0);
  float LloydIteration(floatVectorList &training);
  bool save(char *filename) const;
  bool save(ofstream& ofs) const;
  bool load(char *filename);
  bool load(ifstream& ifs);
};


//: Shape Gain VQ 
class ShapeGainVQ : public NormalisedVQ {
private:
  StandardSQ scalarq;
  
public:
  ShapeGainVQ() : NormalisedVQ(), scalarq((QLevels)0) {}
  ShapeGainVQ(const QIndexBits& shapebits,
	      const QIndexBits& gainbits, unsigned int shaped) :
    NormalisedVQ(shapebits, shaped), scalarq(gainbits) {}
  unsigned int Levels(void) const;
  float Rate(void) const;
  float IndexBits(void) const;
  int Index(const floatVector& x) const;
  floatVectorRef Reconstruction(unsigned int idx) const;
  float GLA(floatVectorList &training, float threshold,
	    unsigned int iteration_limit = 0);
  bool save(char *filename) const;
  bool save(ofstream& ofs) const;
  bool load(char *filename);
  bool load(ifstream& ifs);
};



//: Zero-Mean Normalised VQ - MRSGVQ without mean or qain quantisation
class ZeroMeanNormalisedVQ : public VectorQuantiser {
protected:
  floatVectorList rlevel;

  static float correlation(const floatVector &a, const floatVector &b);

public:
  ZeroMeanNormalisedVQ() : VectorQuantiser(), rlevel(0) {}
  ZeroMeanNormalisedVQ(const QLevels& ql, unsigned int d) :
    VectorQuantiser(ql, d), rlevel((unsigned int)ql) {};
  ZeroMeanNormalisedVQ(const QRate& qr, unsigned int d) :
    VectorQuantiser(qr, d), rlevel(1 << (d*qr.Rate())) {};
  int Index(const floatVector& x) const;
  floatVectorRef Reconstruction(unsigned int idx) const
  {
    floatVector rec = rlevel.entry(idx);
    return rec;
  }
  float GLA(floatVectorList &training, float threshold,
	    unsigned int iteration_limit = 0);
  float LloydIteration(floatVectorList &training);
  bool save(char *filename) const;
  bool save(ofstream& ofs) const;
  bool load(char *filename);
  bool load(ifstream& ifs);
};
