// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: Histogram.cc,v 1.2 2005/10/06 14:46:06 brendt Exp $
  
*/


#include <math.h>
#include <values.h>

#include "ImageLib.h"
#include "Histogram.h"

void Histogram::initialise()
{
  Ndata = 0;
  rhxmin = MAXFLOAT;
  rhxmax = -MAXFLOAT;
  xsum = 0.0;
  xsum2 = 0.0;
}

int Histogram::set(float sxmin, float sxmax, float sxh)
{
  xmin = sxmin;
  xmax = sxmax;
  xh = sxh;
  xNbins = (unsigned long)ceil((xmax - xmin)/xh);
  xh = (xmax - xmin)/(float)xNbins;

  initialise();

  data = new unsigned long int [xNbins];
  if (data == 0)
    return 0;
  for (unsigned long int i = 0; i < xNbins; i++)
    data[i] = 0;
  return 1;
}

int Histogram::set(float sxmin, float sxmax, unsigned long int sNbins)
{
  xmin = sxmin;
  xmax = sxmax;
  xNbins = sNbins;
  xh = (xmax - xmin)/(float)xNbins;

  initialise();

  data = new unsigned long int [xNbins];
  if (data == 0)
    return 0;
  for (unsigned long int i = 0; i < xNbins; i++)
    data[i] = 0;
  return 1;
}

int Histogram::reset(float sxmin, float sxmax, float sxh)
{
  delete [] data;
  return set(sxmin, sxmax, sxh);
}

int Histogram::reset(float sxmin, float sxmax, unsigned long int sNbins)
{
  delete [] data;
  return set(sxmin, sxmax, sNbins);
}

Histogram::Histogram(float sxmin, float sxmax, float sxh)
{
  ASSERT_FATAL(set(sxmin, sxmax, sxh));
}

Histogram::Histogram(float sxmin, float sxmax, unsigned long int sNbins)
{
  ASSERT_FATAL(set(sxmin, sxmax, sNbins));
}

Histogram::~Histogram()
{
  delete [] data;
}

int Histogram::insert(float x)
{
  if (x < rhxmin) rhxmin = x;
  if (x > rhxmax) rhxmax = x;
  xsum += x;
  xsum2 += x*x;
  Ndata++;
  long int bin = (long int)floor((x - xmin)/xh);
  if ((bin < 0) || (bin >= (long int)xNbins))
    return 0;
  else
    {
      data[bin]++;
      return 1;
    }
}

float Histogram::mean() const
{
  return xsum/(double)Ndata;
}

float Histogram::variance() const
{
  float n = (double)Ndata;
  return (n*xsum2 - xsum*xsum)/(n*n);
}

float Histogram::Entropy() const
{
  float* px = new float [xNbins];
  unsigned long int i;
  for (i = 0; i < xNbins; i++)
    px[i] = (1.0/(float)Ndata)*(float)data[i];

  float entropy = 0.0;
  for (i = 0; i < xNbins; i++)
    if (px[i] != 0.0)
      entropy -= px[i]*log2(px[i]);

  delete [] px;
  
  return entropy;
}

int Histogram::save(const char* filename) const
{
  ofstream data_file(filename, IOS_OUT);
  if (!STREAM_OPEN(data_file))
    return 0;
  data_file << "# " << xNbins << " bins of size " << xh
  	    << " on [" << xmin << "," << xmax << "]" << endl
            << "# mean: " << mean() << "  variance: " << variance() << endl
            << "# Entropy: " << Entropy() << endl
            << "#        x        N(x)          P(x)          W(x)\n";     
  return savebins(data_file);
}

int Histogram::savebins(ofstream& s) const
{
  unsigned long int i;
  float integral = 0.0;

  //Could use more accurate integral
  for (i = 0; i < xNbins; i++)
    integral += (float)data[i];
  integral *= xh;
  
  for (i = 0; i < xNbins; i++)
    s  << setw(10) << xmin + xh*(float)i
       << setw(12) << data[i]
       << setw(14) << (float)data[i]/integral
       << setw(14) << 1.0/(1.0+(float)data[i]) << endl;
  return xNbins;
}



long int Histogram::bin(unsigned long int x) const
{
  return data[x];
}




JointHistogram::JointHistogram(float sxmin, float sxmax, float
				     symin, float symax, float sxh,
				     float syh)
{
  ASSERT_FATAL(set(sxmin, sxmax, symin, symax, sxh, syh));
}

JointHistogram::JointHistogram(float sxmin, float sxmax,
			       float symin, float symax,
			       unsigned int sxN, unsigned int syN)
{
  ASSERT_FATAL(set(sxmin, sxmax, symin, symax, sxN, syN));
}

JointHistogram::~JointHistogram()
{
  delete [] data;
}

void JointHistogram::initialise()
{
  Ndata = 0;
  rhxmin = MAXFLOAT;
  rhxmax = -MAXFLOAT;
  rhymin = MAXFLOAT;
  rhymax = -MAXFLOAT;
  xsum = 0.0;
  xsum2 = 0.0;
  ysum = 0.0;
  ysum2 = 0.0;
  xysum = 0.0;
}

int JointHistogram::set(float sxmin, float sxmax, float symin,
			float symax, float sxh, float syh)
{
  xmin = sxmin;
  xmax = sxmax;
  ymin = symin;
  ymax = symax;
  xNbins = (unsigned int)ceil((xmax - xmin)/sxh);
  yNbins = (unsigned int)ceil((ymax - ymin)/syh);
  xh = (xmax - xmin)/(float)xNbins;
  yh = (ymax - ymin)/(float)yNbins;

  initialise();
  
  data = new unsigned long int [xNbins*yNbins];
  if (data == 0)
    return 0;
  unsigned long int i;
  for (i = 0; i < xNbins*yNbins; i++)
    data[i] = 0;
  return 1;
}

int JointHistogram::set(float sxmin, float sxmax, float symin,
			float symax, unsigned int sxN, unsigned int syN)
{
  xmin = sxmin;
  xmax = sxmax;
  ymin = symin;
  ymax = symax;
  xNbins = sxN;
  yNbins = syN;
  xh = (xmax - xmin)/(float)xNbins;
  yh = (ymax - ymin)/(float)yNbins;

  initialise();
  
  data = new unsigned long int [xNbins*yNbins];
  if (data == 0)
    return 0;
  unsigned long int i;
  for (i = 0; i < xNbins*yNbins; i++)
    data[i] = 0;
  return 1;
}

int JointHistogram::reset(float sxmin, float sxmax, float symin,
			      float symax, float sxh, float syh)
{
  delete [] data;
  return set(sxmin, sxmax, symin, symax, sxh, syh);
}

int JointHistogram::reset(float sxmin, float sxmax, float symin, float symax,
	                   unsigned int sxN, unsigned int syN)
{
  delete [] data;
  return set(sxmin, sxmax, symin, symax, sxN, syN);
}

int JointHistogram::insert(float x, float y)
{
  long int xpos, ypos;

  Ndata++;
  xsum += x;
  ysum += y;
  xsum2 += x*x;
  ysum2 += y*y;
  xysum += x*y;
  if (x < rhxmin) rhxmin = x;
  if (x > rhxmax) rhxmax = x;
  if (y < rhymin) rhymin = y;
  if (y > rhymax) rhymax = y;
  
  xpos = (int)floor((x - xmin)/xh);
  ypos = (int)floor((y - ymin)/yh);
  if (xpos < 0 || xpos >= (long int)xNbins || 
      ypos < 0 || ypos >= (long int)yNbins)
    return 0;
  data[xpos + ypos*xNbins]++;
  
  return 1;
}

long int JointHistogram::bin(unsigned int x, unsigned int y) const
{
  return data[x + y*xNbins];
}

float JointHistogram::MutualInformation() const
{
  // See Cover91 Ch. 9
  unsigned int i,j;
  float pxy;
  float* py = new float[yNbins];
  float* px = new float[xNbins];
  float mi = 0.0;
  
  for (j = 0; j < yNbins; j++)
    {
      py[j] = 0.0;
      for (i = 0; i < xNbins; i++)
	py[j] += data[i + j*xNbins];
      py[j] /= (float)Ndata;
    }

  for (i = 0; i < xNbins; i++)
    {
      px[i] = 0.0;
      for (j = 0; j < yNbins; j++)
	px[i] += data[i + j*xNbins];
      px[i] /= (float)Ndata;
    }

  for (i = 0; i < xNbins; i++)
    for (j = 0; j < yNbins; j++)
      {
	pxy = data[i + j*xNbins]/(float)Ndata;
	if (px[i]*py[j] != 0.0 && pxy != 0.0)
	  mi += pxy*log2(pxy/(px[i]*py[j]));
      }

  delete [] px;
  delete [] py;
  
  return mi;
}

float JointHistogram::xmean() const
{
  return xsum/(double)Ndata;
}

float JointHistogram::ymean() const
{
  return ysum/(double)Ndata;
}

float JointHistogram::xvariance() const
{
  float n = (double)Ndata;
  return (n*xsum2 - xsum*xsum)/(n*n);
}

float JointHistogram::yvariance() const
{
  float n = (double)Ndata;
  return (n*ysum2 - ysum*ysum)/(n*n);
}

float JointHistogram::covariance() const
{
  float n = (double)Ndata;
  return (n*xysum - xsum*ysum)/(n*n);
}

float JointHistogram::normcovariance() const
{
  return covariance()/(sqrt(xvariance()*yvariance()));
}

int JointHistogram::save(const char *filename) const
{
  ofstream data_file(filename, IOS_OUT);
  if (!STREAM_OPEN(data_file))
    return 0;
  savebins(data_file);
  data_file.close();
  return 1;
}


int JointHistogram::savebins(ofstream& s) const
{
  unsigned int i,j;
  float integral = 0.0;
  for (i = 0; i < xNbins; i++)
    for (j = 0; j < yNbins; j++)
      integral += (float)data[i + j*xNbins]; //Rough estimate of integral
  integral *= xh*yh;
  
  s << "# xmin: " << xmin << "  xmax: " << xmax << endl
    << "# ymin: " << ymin << "  ymax: " << ymax << endl
    << "# xh: " << xh << "  yh: " << yh << endl
    << "# xNbins: " << xNbins << "  yNbins: " << yNbins << endl
    << "# N: " << Ndata << endl
    << "# xmean: " << xmean() << "  xvariance: " << xvariance() << endl
    << "# ymean: " << ymean() << "  yvariance: " << yvariance() << endl
    << "# covariance: " << covariance()
    << "  correlation: " << normcovariance() << endl;
    
  for (i = 0; i < xNbins; i++)
    {
      for (j = 0; j < yNbins; j++)
	s << setw(14) << data[i + j*xNbins]
	  << setw(14) << data[i + j*xNbins]/integral << endl;
      s << endl;
    }
  return 1;
}
