// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: Histogram.h,v 1.1 2003/02/01 17:46:23 brendt Exp $
  
*/


#ifndef _Histogram_h
#define _Histogram_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
#include <iomanip>
using std::ofstream;
using std::ifstream;
using std::setw;
#else
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#endif

#include "floatImage.h"


//! unit="Statistics"
//! module="Histograms" 
//! level = 1


//: Class implementing histogram operations.
// Note that samples lying outside of the specified interval are still 
// included in the mean and variance calculations.
class Histogram {
private:
  float xmin;  
  float xmax;
  float xh;  /* bin size */
  unsigned long int xNbins;
  unsigned long int *data;

  unsigned long int Ndata;
  float rhxmin;
  float rhxmax;
  double xsum;
  double xsum2;

  void initialise();
  int set(float sxmin, float sxmax, float sxh);
  int set(float sxmin, float sxmax, unsigned long int sNbins);

public:
  Histogram() { data = 0; xNbins = 0; initialise(); }
  //Histogram on interval (sxmin, sxmax) with bins of width sxh
  Histogram(float sxmin, float sxmax, float sxh);
  //Histogram with sNbins on interval (sxmin, sxmax) 
  Histogram(float sxmin, float sxmax, unsigned long int sNbins);
  ~Histogram();
  //Reset histogram to interval (sxmin, sxmax) with bins of width sxh
  int reset(float sxmin, float sxmax, float sxh);
  //Reset histogram to have sNbins on interval (sxmin, sxmax)
  int reset(float sxmin, float sxmax, unsigned long int sNbins);
  //Add sample to sample set
  int insert(float x);
  //Return number of samples in bin x
  long int bin(unsigned long int x) const;
  //Return sample size
  float N() const { return Ndata; }
  //Return minimum sample value
  float hxmin() const { return rhxmin; }
  //Return maximum sample value
  float hxmax() const { return rhxmax; }
  //Return sample mean
  float mean() const;
  //Return sample variance
  float variance() const;
  //Estimate entropy of associated pdf
  float Entropy() const;
  
  int save(const char *filename) const;
  int savebins(ofstream& s) const;
};


//: Class implementing joint histogram operations. 
class JointHistogram {
private:
  float xmin;  
  float xmax;
  float ymin;
  float ymax;
  float xh;  /* x bin size */
  float yh;  /* y bin size */
  unsigned long int xNbins;
  unsigned long int yNbins;
  unsigned long int *data;

  unsigned long int Ndata;
  float rhxmin;
  float rhxmax;
  float rhymin;
  float rhymax;
  double xsum;
  double xsum2;
  double ysum;
  double ysum2;
  double xysum;

  void initialise();
  int set(float sxmin, float sxmax, float ymin, float ymax,
	  float sxh, float syh);
  int set(float sxmin, float sxmax, float ymin, float ymax,
	  unsigned int sxN, unsigned int syN);
public:
  JointHistogram() { data = 0;xNbins = 0;yNbins = 0;initialise(); }
  JointHistogram(float sxmin, float sxmax, float ymin, float ymax,
	    float sxh, float syh);
  JointHistogram(float sxmin, float sxmax, float ymin, float ymax,
	    unsigned int sxN, unsigned int syN);
  ~JointHistogram();
  int reset(float sxmin, float sxmax, float ymin, float ymax,
	    float sxh, float syh);
  int reset(float sxmin, float sxmax, float ymin, float ymax,
	    unsigned int sxN, unsigned int syN);

  unsigned long int xbins() const { return xNbins; }
  unsigned long int ybins() const { return yNbins; }
  float xhval() const { return xh; }
  float yhval() const { return yh; }
  
  int insert(float x, float y);
  long int bin(unsigned int x, unsigned int y) const;
  float N() const { return Ndata; }
  float hxmin() const { return rhxmin; }
  float hxmax() const { return rhxmax; }
  float hymin() const { return rhymin; }
  float hymax() const { return rhymax; }
  float xmean() const;
  float ymean() const;
  float xvariance() const;
  float yvariance() const;
  float covariance() const;
  //Normalised covariance (correlation coefficient)
  float normcovariance() const;

  float MutualInformation() const;
  
  int save(const char *filename) const;
  int savebins(ofstream& s) const;
};

#endif
