// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip1.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: dct.h,v 1.3 2005/10/06 14:10:19 brendt Exp $
  
*/


#ifndef _dct_h
#define _dct_h

#include <math.h>

#include "ImageLib.h"
#include "floatImage.h"
#include "floatVector.h"

//! unit="Transforms"
//! module="DCT" 
//! level = 1


/* Slow DCT functions */

static inline real alpha(unsigned int u, unsigned int N)
{
  if (u == 0)
    return sqrt(1/double(N));
  else if ((u > 0) && (u < N))
    return sqrt(2/double(N));
  else
    return 0.0;

  // Silence strange "control may reach end of non-void function" warning
  return 0.0;
}

//: Perform DCT on src, result in dest
void dct(const floatImage& src, floatImage& dest);
//: Perform Inverse DCT on src, result in dest
void idct(const floatImage& src, floatImage& dest);
//: Return DCT of im
floatImageRef dct(const floatImage& im);
//: Return Inverse DCT of im
floatImageRef idct(const floatImage& im); 


/***************************************************************************

  class ChanDCT is an encapsulation of sofware (URL:
  ftp://etro.vub.ac.be/pub/COMPRESSION/DCT_ALGORITHMS/) written by
  Charilaos A. Christopoulos (Email:chchrist@etro.vub.ac.be), based on
  the paper:

  S. C. Chan and K. L. Ho, "A new two-dimensional fast cosine transform 
  algorithm", IEEE Trans. on Signal Processing, Vol. 39, No. 2, pp. 481-485,
  Feb. 1991.

***************************************************************************/

//: Class implementing Fast DCT
class ChanDCT {
private:
  int N;
  int m;
  float *cosines;

  float scaleDC;
  float scaleMix;
  float scaleAC;

  void makecosinetable();
  void columnspostadditions(floatImage& fi) const;
  void rowspostadditions(floatImage& fi) const;
  void rowsbitreversal(floatImage& fi) const;
  void columnsbitreversal(floatImage& fi) const;
  void columnsinputmapping(floatImage& fi) const;
  void rowsinputmapping(floatImage& fi) const;

public:
  ChanDCT(unsigned int size);
  ~ChanDCT();
  
  void dct_in_place(floatImage& im) const;
  floatImageRef operator()(const floatImage& im) const
  { return dct(im); }
  floatImageRef dct(const floatImage& im) const;
  void dct(const floatImage& src, floatImage& dest) const;
};


/***************************************************************************

  class VecRadDCT is an encapsulation of sofware (URL:
  ftp://etro.vub.ac.be/pub/COMPRESSION/DCT_ALGORITHMS/) written by
  Charilaos A. Christopoulos (Email:chchrist@etro.vub.ac.be), based on
  the papers:

  C.A. Christopoulos, J. Bormans, A.N. Skodras and J. Cornelis, "Efficient 
  computation of the two-dimensional fast cosine transform", SPIE Hybrid Image 
  and Signal Processing IV, 5-8 April 1994, Orlando, Florida, USA. Accepted.

  and

  C.A. Christopoulos and A.N. Skodras, "Pruning the two-dimensional fast cosine 
  transform", To be presented at the VII European Signal Processing Conference ,
  September 13-16, 1994, Edinburgh, Scotland.

***************************************************************************/

#define PIO2 1.5707966327

//: Class implementing Fast DCT
class VecRadDCT {
private:
  int N;
  int N0;
  int m;
  float *ct;
  float *ct2d;
  unsigned int **r;
  float *cosine_array;
  unsigned int MASK[2];

  float scaleDC;
  float scaleMix;
  float scaleAC;

  void lut_of_cos();
  void expand1d_lookup_table();
  void make2Darray();
  void bitreversalrows() const;
  void bitreversalcolumns() const;
  void firo3(floatImage& fi) const;
  void post_adds(floatImage& fi) const;

public:
  VecRadDCT(unsigned int size, unsigned int pts);
  VecRadDCT();
  ~VecRadDCT();
  void initialise(unsigned int size, unsigned int pts);
  
  void dct_in_place(floatImage& im) const;
  floatImageRef operator()(const floatImage& im) const
  { return dct(im); }
  floatImageRef dct(const floatImage& im) const;
  void dct(const floatImage& src, floatImage& dest) const;
};


#endif
