// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: CoefAccess.h,v 1.2 2005/10/06 14:46:06 brendt Exp $

 */

#if !defined(_CoefAccess_h)
#define _CoefAccess_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
using std::ofstream;
using std::ifstream;
using std::endl;
using std::ios;
#else
#include <iostream.h>
#include <fstream.h>
#endif

#include "floatVector.h"

//! unit="Wavelets"
//! module="Coefficient Access" 
//! level = 1

class CoefAccessSubtree;
class CoefTree;

//: Tree structure access to wavelet coefficients
//Note that the tree must be a balanced binary tree, and that
//overlapping tree copying using the subtree methods is unsafe.
class CoefAccess {
private:
  floatVector& tree_vec;
  unsigned tree_depth;

public:
  //Construct tree access class given vector of wavelet coefficients
  CoefAccess(floatVector& fv);
  //Tree depth - a tree with a single node has depth 1
  unsigned int depth() const { return tree_depth; }
  //Length of subband at level i in the tree
  static unsigned int subband_length(unsigned int i) { return 1 << i; }
  //Access detail coefficient d_{i,j}
  float& operator()(unsigned int i, unsigned int j) const;
  //Access subband at resolution i
  floatSubvector subband(unsigned int i);
  //Return subtree rooted at node (i,j). The result of this member
  //function may be assigned to in order to change a subtree.
  CoefAccessSubtree subtree(unsigned int i, unsigned int j);
  //Return subtree of specified depth rooted at node (i,j)
  CoefAccessSubtree subtree(unsigned int i, unsigned int j,
			    unsigned int depth);

  friend class CoefAccessSubtree;
};

inline float& CoefAccess::operator()(unsigned int i, unsigned int j) const
{
  return tree_vec( (1 << i) + j );
}


//: Subtree of a coefficient tree
//This class supports the subtree copying mechanism of class CoefAccess
class CoefAccessSubtree {
private:
  unsigned int ri;
  unsigned int rj;
  unsigned int stdepth;
  const CoefAccess& ptree;

public:
  CoefAccessSubtree(const CoefAccess& tree, unsigned int i = 0,
		    unsigned int j = 0);
  CoefAccessSubtree(const CoefAccess& tree, unsigned int i,
		    unsigned int j, unsigned int depth);
  const CoefAccessSubtree& operator=(const CoefAccessSubtree& subtree);
  const CoefTree& operator=(const CoefTree& vecrep);
  const CoefAccess& operator=(const CoefAccess& tree);

  float& operator()(unsigned int i, unsigned int j) const;

  friend ostream& operator<<(ostream &s, const CoefAccessSubtree& t);
  
  friend class CoefAccess;
  friend class CoefTree;
};


inline float& CoefAccessSubtree::operator()(unsigned int i,
					    unsigned int j) const
{
  return ptree(ri + i, rj*(1 << i) + j);
}


//: Vector of wavelet coefficients supporting tree-structured access
class CoefTree : public floatVector {
private:
  unsigned int tree_depth;

public:
  CoefTree(const CoefAccessSubtree& stree);
  CoefTree(const floatVector& fv);
  CoefTree(const floatVector& fv, unsigned int depth);
  CoefTree(unsigned int depth, float f = 0.0);

  float& operator()(unsigned int i, unsigned int j) const;

  friend class CoefAccessSubtree;
};

inline CoefTree::CoefTree(const CoefAccessSubtree& stree) :
  floatVector( (1 << stree.stdepth) - 1), tree_depth(stree.stdepth)
{
  unsigned int i,j,repc = 0;
  for (i = 0; i < tree_depth; i++)
    for (j = 0; j < CoefAccess::subband_length(i); j++)
      floatVector::operator()(repc++) = stree(i,j);
}


inline CoefTree::CoefTree(const floatVector& fv) : floatVector(fv)
{
  float fdepth = log2(fv.vlength()+1);
  assert( fdepth == floor(fdepth) );
  tree_depth = (unsigned int) floor(fdepth);
}

inline CoefTree::CoefTree(const floatVector& fv, unsigned int depth) :
  floatVector(fv), tree_depth(depth)
{
  float fdepth = log2(fv.vlength()+1);
  assert( fdepth == floor(fdepth) );
  assert( tree_depth <= floor(fdepth) );
}

inline CoefTree::CoefTree(unsigned int depth, float f) :
  floatVector((1 << depth) - 1, f), tree_depth(depth)
{ }

inline float& CoefTree::operator()(unsigned int i, unsigned int j) const
{
  return floatVector::operator()( (1 << i) + j - 1 );
}

#endif


