// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: NstdCoefAccess.h,v 1.2 2005/10/06 14:46:06 brendt Exp $

 */

#if !defined(_NstdCoefAccess_h)
#define _NstdCoefAccess_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
using std::ofstream;
using std::ifstream;
using std::endl;
using std::ios;
#else
#include <iostream.h>
#include <fstream.h>
#endif

#include "floatVector.h"
#include "floatImage.h"

//! unit="Wavelets"
//! module="Coefficient Access" 
//! level = 1

class NstdCoefAccessDrctSubtree;
class NstdCoefAccessFullSubtree;
class NstdCoefDrctTree;
class NstdCoefFullTree;

//: Tree structure access to wavelet coefficients in a non-standard decomposition
//Note that the tree must be a balanced quad tree, and that
//overlapping tree copying using the subtree methods is unsafe.
class NstdCoefAccess {
private:
  floatImage& tree_im;
  unsigned tree_depth;

public:
  //The directional subbands in a non-standard decomposition
  enum Direction {Horizontal, Vertical, Diagonal};
  //Construct tree access class given image of wavelet coefficients
  NstdCoefAccess(floatImage& fi);
  //Tree depth - a tree with a single node has depth 1
  unsigned int depth() const { return tree_depth; }
  //Length of subband at level i in the tree
  static unsigned int subband_length(unsigned int i) { return 1 << i; }
  //Access detail coefficient d_{i;j,k} in the specified directional subband
  float& operator()(unsigned int i,
		    unsigned int j, unsigned int k,
		    Direction drn) const;
  //Access specified directional subband at resolution i
  floatSubimage subband(unsigned int i, Direction drn);
  //Return subtree rooted at node (i,j,k) in the specified directional 
  //subband. The result of this member function may be assigned to in
  //order to change a subtree.
  NstdCoefAccessDrctSubtree subtree(unsigned int i,
				    unsigned int j, unsigned int k,
				    Direction drn);
  //Return subtree of specified depth rooted at node (i,j,k) in the specified directional 
  //subband.
  NstdCoefAccessDrctSubtree subtree(unsigned int i,
				    unsigned int j, unsigned int k,
				    Direction drn,
				    unsigned int depth);
  //Return combined subtrees rooted at nodes (i,j,k) in all three
  //directional subbands
  NstdCoefAccessFullSubtree subtree(unsigned int i,
				    unsigned int j, unsigned int k);
  //Return combined subtrees of specified depth rooted at nodes (i,j,k) in all three
  //directional subbands
  NstdCoefAccessFullSubtree subtree(unsigned int i,
				    unsigned int j, unsigned int k,
				    unsigned int depth);

  friend class NstdCoefAccessDrctSubtree;
  friend class NstdCoefAccessFullSubtree;
};

inline float& NstdCoefAccess::operator()(unsigned int i,
					 unsigned int j, unsigned int k,
					 Direction drn) const
{
  unsigned int c = 1 << i;
  unsigned int a = (drn == Horizontal)?0:c;
  unsigned int b = (drn == Vertical)?0:c;
  return tree_im( a + j, b + k);
}


//: Subtree of a coefficient tree
//This class supports the subtree copying mechanism of class NstdCoefAccess
class NstdCoefAccessDrctSubtree {
private:
  unsigned int ri;
  unsigned int rj;
  unsigned int rk;
  unsigned int stdepth;
  NstdCoefAccess::Direction stdrn;
  const NstdCoefAccess& ptree;

public:
  NstdCoefAccessDrctSubtree(const NstdCoefAccess& tree, unsigned int i,
			    unsigned int j, unsigned int k,
			    NstdCoefAccess::Direction drn);
  NstdCoefAccessDrctSubtree(const NstdCoefAccess& tree, unsigned int i,
			    unsigned int j, unsigned int k,
			    NstdCoefAccess::Direction drn,
			    unsigned int depth);
  NstdCoefAccessDrctSubtree(const NstdCoefAccessFullSubtree& stree,
			    NstdCoefAccess::Direction drn);
  const NstdCoefAccessDrctSubtree&
     operator=(const NstdCoefAccessDrctSubtree& subtree);
  const NstdCoefDrctTree&
     operator=(const NstdCoefDrctTree& vecrep);

  float& operator()(unsigned int i,
		    unsigned int j, unsigned int k) const;

  unsigned int depth() { return stdepth; }

  friend class NstdCoefAccess;
  friend class NstdCoefAccessFullSubtree;
  friend class NstdCoefDrctTree;
};


inline float& NstdCoefAccessDrctSubtree::operator()(unsigned int i,
						    unsigned int j,
						    unsigned int k) const
{
  return ptree(ri + i, rj*(1 << i) + j, rk*(1 << i) + k, stdrn);
}

//: Combined directional subtrees of a coefficient tree
//This class supports the subtree copying mechanism of class NstdCoefAccess
class NstdCoefAccessFullSubtree {
private:
  unsigned int ri;
  unsigned int rj;
  unsigned int rk;
  unsigned int stdepth;
  const NstdCoefAccess& ptree;

public:
  NstdCoefAccessFullSubtree(const NstdCoefAccess& tree, unsigned int i,
			    unsigned int j, unsigned int k);
  NstdCoefAccessFullSubtree(const NstdCoefAccess& tree, unsigned int i,
			    unsigned int j, unsigned int k,
			    unsigned int depth);
  const NstdCoefAccessFullSubtree&
     operator=(const NstdCoefAccessFullSubtree& subtree);
  const NstdCoefFullTree&
      operator=(const NstdCoefFullTree& vecrep);

  float& operator()(unsigned int i,
		    unsigned int j, unsigned int k,
		    NstdCoefAccess::Direction drn) const;

  unsigned int depth() { return stdepth; }
  
  friend class NstdCoefAccess;
  friend class NstdCoefAccessDrctSubtree;
  friend class NstdCoefFullTree;
};


inline float& NstdCoefAccessFullSubtree::operator()(unsigned int i,
						    unsigned int j,
						    unsigned int k,
					  NstdCoefAccess::Direction drn) const
{
  return ptree(ri + i, rj*(1 << i) + j, rk*(1 << i) + k, drn);
}


/* NB: Should define assignment operator */
//: Image of wavelet coefficients supporting tree-structured access to 
//: a single directional subband
class NstdCoefDrctTree : public floatVector {
private:
  unsigned int tree_depth;

public:
  NstdCoefDrctTree(const NstdCoefAccessDrctSubtree& stree);
  NstdCoefDrctTree(const floatVector& fv);
  NstdCoefDrctTree(const floatVector& fv, unsigned int depth);
  NstdCoefDrctTree(unsigned int depth, float f = 0.0);

  unsigned int depth() const { return tree_depth; }
  static unsigned int tree_size(unsigned int depth);

  NstdCoefDrctTree subtree(unsigned int ri,
			   unsigned int rj, unsigned int rk,
			   unsigned int depth) const;
  NstdCoefDrctTree subtree(unsigned int ri,
			   unsigned int rj, unsigned int rk) const;

  float &operator()(unsigned int i, unsigned int j, unsigned int k) const;

  friend class NstdCoefAccessDrctSubtree;
};

inline NstdCoefDrctTree NstdCoefDrctTree::subtree(unsigned int ri,
						  unsigned int rj,
						  unsigned int rk) const
{
  return subtree(ri,rj,rk,tree_depth-ri);
}

/* Note that $\sum_{i = 0}^{n-1} 4^i = \frac{4^n - 1}{3}$ */

inline unsigned int NstdCoefDrctTree::tree_size(unsigned int depth)
{
  return ((1 << (2*depth)) - 1)/3;
}
 
inline
NstdCoefDrctTree::NstdCoefDrctTree(const NstdCoefAccessDrctSubtree&
				   stree) :
  floatVector(tree_size(stree.stdepth)),
  tree_depth(stree.stdepth)
{
  unsigned int i,j,k,repc = 0;
  for (i = 0; i < tree_depth; i++)
    for (j = 0; j < NstdCoefAccess::subband_length(i); j++)
      for (k = 0; k < NstdCoefAccess::subband_length(i); k++)
        floatVector::operator()(repc++) = stree(i,j,k);
}

inline NstdCoefDrctTree::NstdCoefDrctTree(const floatVector& fv) :
  floatVector(fv)
{
  float fdepth = log2(3*fv.vlength() + 1)/2.0;
  assert( fdepth == floor(fdepth) );
  tree_depth = (unsigned int)floor(fdepth);
}

inline NstdCoefDrctTree::NstdCoefDrctTree(const floatVector& fv,
					  unsigned int depth) :
  floatVector(fv), tree_depth(depth)
{
  float fdepth = log2(3*fv.vlength() + 1)/2.0;
  assert( fdepth == floor(fdepth) );
  assert( tree_depth <= floor(fdepth) );
}

inline NstdCoefDrctTree::NstdCoefDrctTree(unsigned int depth, float f) :
  floatVector(tree_size(depth), f), tree_depth(depth)
{ }

inline float& NstdCoefDrctTree::operator()(unsigned int i,
					   unsigned int j,
					   unsigned int k) const
{
  unsigned int depthoffset = ( (1 << (2*i)) - 1 )/3;
  unsigned int posoffset = (1 << i)*j + k;
  return floatVector::operator()(depthoffset + posoffset);
}

//: Image of wavelet coefficients supporting tree-structured access
class NstdCoefFullTree : public floatVector {
private:
  unsigned int tree_depth;

public:
  NstdCoefFullTree(const NstdCoefAccessFullSubtree& stree);
  NstdCoefFullTree(const floatVector& fv);
  NstdCoefFullTree(const floatVector& fv, unsigned int depth);
  NstdCoefFullTree(unsigned int depth, float f = 0.0);

  unsigned int depth() const { return tree_depth; }
  static unsigned int tree_size(unsigned int depth);

  NstdCoefFullTree subtree(unsigned int ri,
			   unsigned int rj, unsigned int rk,
			   unsigned int depth) const;
  NstdCoefFullTree subtree(unsigned int ri,
			   unsigned int rj, unsigned int rk) const;
  NstdCoefDrctTree subtree(unsigned int ri,
			   unsigned int rj, unsigned int rk,
			   NstdCoefAccess::Direction drn,
			   unsigned int depth) const;
  NstdCoefDrctTree subtree(unsigned int ri,
			   unsigned int rj, unsigned int rk,
			   NstdCoefAccess::Direction drn) const;
  
  float& operator()(unsigned int i, unsigned int j, unsigned int k,
		    NstdCoefAccess::Direction drn) const;
  
  friend class NstdCoefAccessFullSubtree;
};

inline NstdCoefFullTree NstdCoefFullTree::subtree(unsigned int ri,
						  unsigned int rj,
						  unsigned int rk) const
{
  return subtree(ri,rj,rk,tree_depth-ri);
}

inline NstdCoefDrctTree NstdCoefFullTree::subtree(unsigned int ri,
						  unsigned int rj,
						  unsigned int rk,
					 NstdCoefAccess::Direction drn) const
{
  return subtree(ri,rj,rk,drn,tree_depth-ri);
}

inline unsigned int NstdCoefFullTree::tree_size(unsigned int depth)
{
  return ((1 << (2*depth)) - 1);
}

inline NstdCoefFullTree::NstdCoefFullTree(const
					  NstdCoefAccessFullSubtree& stree) :
  floatVector(tree_size(stree.stdepth)),
  tree_depth(stree.stdepth)
{
  unsigned int i,j,k,drn,repc = 0;
  for (drn = (unsigned int)NstdCoefAccess::Horizontal;
       drn <= (unsigned int)NstdCoefAccess::Diagonal; drn++)
  for (i = 0; i < tree_depth; i++)
    for (j = 0; j < NstdCoefAccess::subband_length(i); j++)
      for (k = 0; k < NstdCoefAccess::subband_length(i); k++)
        floatVector::operator()(repc++) = stree(i,j,k,
					      (NstdCoefAccess::Direction)drn);
}

inline NstdCoefFullTree::NstdCoefFullTree(const floatVector& fv) :
  floatVector(fv)
{
  float fdepth = log2(fv.vlength() + 1)/2.0;
  assert( fdepth == floor(fdepth) );
  tree_depth = (unsigned int)floor(fdepth);
}

inline NstdCoefFullTree::NstdCoefFullTree(const floatVector& fv,
					  unsigned int depth) :
  floatVector(fv), tree_depth(depth)
{
  float fdepth = log2(fv.vlength() + 1)/2.0;
  assert( fdepth == floor(fdepth) );
  assert( tree_depth <= floor(fdepth) );
}

inline NstdCoefFullTree::NstdCoefFullTree(unsigned int depth, float f) :
  floatVector(tree_size(depth), f), tree_depth(depth)
{ }

inline float &NstdCoefFullTree::operator()(unsigned int i,
					   unsigned int j,
					   unsigned int k,
					   NstdCoefAccess::Direction drn) const
{
  unsigned int depthoffset = ( (1 << (2*i)) - 1 )/3;
  unsigned int posoffset = (1 << i)*j + k;
  unsigned int diroffset = (((1 << (2*tree_depth)) - 1)/3) * (unsigned int)drn;
  return floatVector::operator()(diroffset + depthoffset + posoffset);
}

#endif
