// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: Sequence.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $
  
*/

#include "Sequence.h"

bool Sequence::construct(unsigned int ln, int inidx)
{
  slength = ln;
  initial_index = inidx;
  if ((elements = new double [slength]))
    return true;
  else
    return false;
}

  
Sequence::Sequence(double *elt, unsigned int ln, int inidx)
{
  ASSERT_FATAL( construct(ln, inidx) );
  for (unsigned int i = 0; i < slength; i++)
    elements[i] = elt[i];
}

Sequence::Sequence(const floatVector& fv)
{
  ASSERT_FATAL( construct(fv.vlength(), 0) );
  for (unsigned int i = 0; i < fv.vlength(); i++)
    elements[i] = fv(i);
}
 

Sequence::Sequence(const Sequence& s)
{
  ASSERT_FATAL( construct(s.length(), s.initial()) );
  for (unsigned int i = 0; i < s.length(); i++)
    elements[i] = s.elements[i];
}

Sequence Sequence::operator=(const Sequence& s)
{
  if (this == &s)
    return *this;

  delete [] elements;
  ASSERT_FATAL( construct(s.length(), s.initial()) );
  for (unsigned int i = 0; i < s.length(); i++)
    elements[i] = s.elements[i];

  return *this;
}

bool Sequence::assign(int index, double d)
{
  if (index < initial() || index > final())
    return false;
  else
    {
      elements[index - initial_index] = d;
      return true;
    }
}


Sequence Sequence::operator+(const Sequence& s) const
{
  Sequence sum;
  int start = ::min(initial(), s.initial());
  int end = ::max(final(), s.final());
  ASSERT_FATAL( sum.construct(end - start + 1, start) );
  for (int i = start; i <= end; i++)
    sum.assign(i, operator()(i) + s(i));
  return sum;
}

Sequence Sequence::operator-(const Sequence& s) const
{
  Sequence sum;
  int start = ::min(initial(), s.initial());
  int end = ::max(final(), s.final());
  ASSERT_FATAL( sum.construct(end - start + 1, start) );
  for (int i = start; i <= end; i++)
    sum.assign(i, operator()(i) - s(i));
  return sum;
}

Sequence Sequence::operator*(const Sequence& s) const
{
  Sequence con;
  ASSERT_FATAL( con.construct(slength + s.slength - 1,
			      initial_index + s.initial_index) );
  int i,j;
  for (i = 0; i < (int)con.slength; i++)
    {
      con.elements[i] = 0.0;
      for (j = 0; j < (int)slength; j++)
	con.elements[i] += elements[j]*
	  ((i-j >= 0 && i-j < (int)s.slength)?s.elements[i-j]:0.0);
    }
  return con;
}

//Periodic extension
floatVectorRef Sequence::operator*(const floatVector& fv) const
{
  floatVector cv(fv.vlength(), 0.0);
  unsigned int i,j;
  for (i = 0; i < fv.vlength(); i++)
    for (j = 0; j < slength; j++)
      cv(i) += elements[j] * fv.modindex(i-j-initial_index);
  return cv;
}

Sequence operator*(double d, const Sequence& s)
{
  Sequence m(s);
  for (unsigned int i = 0; i < m.slength; i++)
    m.elements[i] *= d;
  return m;
}

Sequence operator*(const Sequence& s, double d)
{
  Sequence m(s);
  for (unsigned int i = 0; i < m.slength; i++)
    m.elements[i] *= d;
  return m;
}

// z^n H(z)       n positive - advance   n negative - delay
Sequence shift(const Sequence& s, int d)
{
  return Sequence(s.elements, s.slength, s.initial_index - d);
}

// H(z^{-1})
Sequence transpose(const Sequence& s)
{
  Sequence tr;
  ASSERT_FATAL( tr.construct(s.slength, -s.initial_index-s.slength+1) );
  for (int i = 0; i < (int)s.slength; i++)
    tr.elements[i] = s.elements[s.slength-1-i];
  return tr;
}

// H(-z)
Sequence altsign(const Sequence& s)
{
  Sequence alt(s);
  int sign = (s.initial_index % 2) ? -1 : 1;
  for (int i = 0; i < (int)s.slength; i++)
    {
      alt.elements[i] *= sign;
      sign *= -1;
    }
  return alt;
}

// H(-z^{-1})
Sequence altflip(const Sequence& s)
{
  return altsign(transpose(s));
}


Sequence downsample(const Sequence& s)
{
  Sequence down;
  int start = (s.initial_index + (abs(s.initial_index) % 2))/2;
  int end = (s.initial_index + (int)s.slength - 1 -
	       (abs(s.initial_index + (int)s.slength - 1)%2))/2;
  ASSERT_FATAL( down.construct(end - start + 1, start) );
  for (unsigned int i = 0; i < down.slength; i++)
    down.elements[i] = s.elements[2*i + abs(s.initial_index)%2];
  return down;
}

Sequence upsample(const Sequence& s)
{
  Sequence up;
  int start = 2*s.initial_index;
  int end = 2*(s.initial_index + s.slength - 1);
  ASSERT_FATAL( up.construct(end - start + 1, start) );
  for (unsigned int i = 0; i < up.slength; i++)
    up.elements[i] = (i%2)? 0.0 : s.elements[i/2];
  return up;
}


floatVectorRef Sequence::downscale(const floatVector& fv) const
{
  ASSERT_FATAL( fv.vlength()%2 == 0 );
  floatVector down(fv.vlength()/2, 0.0);
  unsigned int i,j;
  for (i = 0; i < fv.vlength(); i+=2)
    for (j = 0; j < slength; j++)
      down(i/2) += elements[j] * fv.modindex(i-j-initial_index);
  return down;
}

floatVectorRef Sequence::upscale(const floatVector& fv) const
{
  floatVector up(2*fv.vlength(), 0.0);
  unsigned int i,j;
  for (i = 0; i < up.vlength(); i++)
      for (j = 0; j < slength; j++)
	if (!((i-j-initial_index)%2))
	  up(i) += elements[j] * fv.modindex((i-j-initial_index)/2);
 
  return up;
}

int Sequence::argmax() const
{
  unsigned int idxmax = 0;
  for (unsigned int i = 0; i < slength; i++)
    if (elements[i] > elements[idxmax])
      idxmax = i;
  return (int)idxmax + initial_index;
}

// Returns 0 for no symmetry, 1 for symmetry, -1 for anti-symmetry
int Sequence::symmetry() const
{
  double sign = (elements[0]/elements[slength-1] < 0)?-1.0:1.0;
  int i;
  for (i = 0; i < (int)slength/2; i++)
    if (fabs(sign*elements[i] - elements[slength-1-i]) > 1e-10)
      return 0;

  return (int)sign;
}

ostream& operator<<(ostream &s, const Sequence& seq)
{
  for (int i = 0; i < (int)seq.slength; i++)
    s << setw(6) << i + seq.initial_index
      << setw(14) << setprecision(8) << seq.elements[i] << endl;
  return s; 
}

SeqFrequencyResponse Sequence::fresponse(unsigned int p) const
{
  return SeqFrequencyResponse(*this, p);
}

SeqScaledDomain Sequence::samplewidth(double d) const
{
  return SeqScaledDomain(*this, d);
}


ostream& operator<<(ostream &s, const SeqFrequencyResponse& sfrs)
{
  double delta = PI/(double)sfrs.points;
  for (double w = 0.0; w <= PI; w+=delta)
    {
      double rsum = 0.0, isum = 0.0;
      for (int n = sfrs.s.initial(); n <= sfrs.s.final(); n++)
	{
	  rsum += sfrs.s(n) * cos(-double(n)*w);
	  isum += sfrs.s(n) * sin(-double(n)*w);
	}
      s << setw(14) << setprecision(6) << w
	<< setw(14) << setprecision(6) << hypot(rsum, isum)
	<< setw(14) << setprecision(6) << atan2(isum, rsum)
	<< endl;
    }
  return s;
}



ostream& operator<<(ostream &s, const SeqScaledDomain& ssd)
{
  for (int i = 0; i < (int)ssd.s.length(); i++)
    s << setw(14) << setprecision(6) << ssd.width*(i + ssd.s.initial())
      << setw(14) << setprecision(6) << ssd.s(i + ssd.s.initial()) << endl;
  return s;
}
