// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: Sequence.h,v 1.1 2003/02/01 17:46:23 brendt Exp $

 */

#ifndef _Sequence_h
#define _Sequence_h

//! unit="Wavelets"
//! module="Support" 
//! level = 1


#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
#include <iomanip>
using std::cerr;
using std::setw;
using std::endl;
#else
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#endif

#include "floatVector.h"

class SeqFrequencyResponse;
class SeqScaledDomain;

//: Finite sequence class
class Sequence {
protected:
  int initial_index;
  unsigned int slength;
  double *elements;

  bool construct(unsigned int ln, int inidx = 0);
  
public:
  //Null sequence
  Sequence(void) : initial_index(0), slength(0), elements(0) {}
  //Construct sequence from elements of array elt, with length ln and
  //initial index inidx
  Sequence(double *elt, unsigned int ln, int inidx = 0);
  //Construct sequence from floatVector
  Sequence(const floatVector& fv);
  virtual ~Sequence() { delete [] elements; }

  Sequence(const Sequence& s);
  Sequence operator=(const Sequence& s);

  //Index of first non-zero element
  int initial() const { return initial_index; }
  //Index of last non-zero element
  int final() const { return initial_index + (int)slength - 1; }
  //Length of sequence
  unsigned int length() const { return slength; }

  //Element at index (index must be between initial and final)
  double coefficient(unsigned int index) const { return elements[index]; }
  //Element at index, return 0 if index outside index range
  double operator()(int index) const
  {
    return (index < initial() || index > final())? 0.0 :
      elements[index - initial_index];
  }
  //Assign element d to index
  bool assign(int index, double d);

  //Add sequences
  Sequence operator+(const Sequence& s) const;
  //Subtract sequences
  Sequence operator-(const Sequence& s) const;
  //Convolve sequences
  Sequence operator*(const Sequence& s) const;

  //Convolve a sequence and a floatVector
  virtual floatVectorRef operator*(const floatVector& fv) const;
  friend floatVectorRef operator*(const floatVector& fv,
				  const Sequence& s)
  { return s * fv; }

  //Multiply a sequence by a constant
  friend Sequence operator*(double d, const Sequence& s);
  friend Sequence operator*(const Sequence& s, double d);

  //Shift sequence by d indices (positive - advance, negative - delay)
  friend Sequence shift(const Sequence& s, int d);
  //Transpose sequence
  friend Sequence transpose(const Sequence& s);
  //Alternating sign of sequence
  friend Sequence altsign(const Sequence& s);
  //Alternating sign of transpose of sequence
  friend Sequence altflip(const Sequence& s);

  //Downsample 
  friend Sequence downsample(const Sequence& s);
  //Upsample
  friend Sequence upsample(const Sequence& s);

  virtual floatVectorRef downscale(const floatVector& fv) const;
  virtual floatVectorRef upscale(const floatVector& fv) const;

  //Index of maximum element
  int argmax() const;
  //Maximum element
  double max() const { return operator()(argmax()); }
  //Return true of sequence has off length
  bool odd() const { return bool(slength%2 == 1); }
  //Return true of sequence has even length
  bool even() const { return bool(slength%2 == 0); }
  // Returns 0 for no symmetry, 1 for symmetry, -1 for anti-symmetry
  int symmetry() const;

  friend ostream& operator<<(ostream &strm, const Sequence& s);
  SeqFrequencyResponse fresponse(unsigned int p) const;
  SeqScaledDomain samplewidth(double d) const;

};

//: Class supporting display of the frequency response associated with 
//: a filter sequence
class SeqFrequencyResponse {
private:
  const Sequence& s;
  const unsigned int points;
public:
  SeqFrequencyResponse(const Sequence& smrs, unsigned int p) :
    s(smrs), points(p) {}
  friend ostream& operator<<(ostream &s, const SeqFrequencyResponse& smrs);
};


class SeqScaledDomain {
private:
  const Sequence s;
  const double width;
public:
  SeqScaledDomain(const Sequence& ssd, double d) : s(ssd), width(d) {}
  friend ostream& operator<<(ostream &s, const SeqScaledDomain& ssd);
};

#endif
