// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: SymWavelet.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $

  
*/

#include "SymWavelet.h"

// a b c d --> a b c d -c -b  
inline static float oddantisymindex(const floatVector& fv, int n)
{
  unsigned int idx = rnmod(n, 2*fv.vlength()-2);
  return (idx < fv.vlength())?(fv(idx)):(-1.0*fv(2*fv.vlength() - 2 - idx));
}

// a b c  --> a b c -c -b -a
inline static float evenantisymindex(const floatVector& fv, int n)
{
   unsigned int idx = rnmod(n, 2*fv.vlength());
    return (idx < fv.vlength())?(fv(idx)):
      (-1.0*fv(2*fv.vlength() - 2 - idx+1));
}


// a b c  --> a b c 0 -c -b -a
inline static float extevenantisymindex(const floatVector& fv, int n)
{
   unsigned int idx = rnmod(n, 2*fv.vlength()+1);
    return (idx < fv.vlength())?(fv(idx)):
     ((idx == fv.vlength())?0.0:(-1.0*fv(2*fv.vlength() - idx)));
}



// a b c  --> a b c c b a
inline static const float& oddevensymindex(const floatVector& fv, int n)
{
  int mid = fv.vlength()-1;
  n = n + mid;
  unsigned int idx = rnmod(n, 2*fv.vlength()-1);
  return fv(abs(idx - mid));
}


// a b c --> a b c b a
inline static const float& evenoddsymindex(const floatVector& fv, int n)
{
  unsigned int mid = fv.vlength()-1;
  unsigned int idx = rnmod(n, 2*fv.vlength()-1);
  if (idx > mid)
    idx = 2*mid - idx;
  return fv(idx);
}


floatVectorRef SymmetricWavelet::analysislow(const floatVector &fv) const
{
  floatVector low((fv.vlength()+1)/2, 0.0);
  unsigned int i,j;
  if (h0.odd())
    {
      for (i = 0; i < fv.vlength(); i+=2)
	for (j = 0; j < h0.length(); j++)
	  low(i/2) += h0.coefficient(j) * fv.oddsymindex(i-j+h0.length()/2);
    }
  else
    {
      for (i = 0; i < fv.vlength(); i+=2)
	for (j = 0; j < h0.length(); j++)
	  low(i/2) += h0.coefficient(j) * fv.evensymindex(i-j+h0.length()/2);
    }
  return low;
}

floatVectorRef SymmetricWavelet::analysishigh(const floatVector &fv) const
{
  floatVector high(fv.vlength()/2, 0.0);
  unsigned int i,j;
  unsigned int limit = ((int)fv.vlength() - int(fv.vlength()%2));
  if (h0.odd())
    {
      for (i = 0; i < limit; i+=2)
	for (j = 0; j < h1.length(); j++)
	  high(i/2) += h1.coefficient(j)*fv.oddsymindex(i-j+h1.length()/2+1);
    }
  else
    {
      for (i = 0; i < limit; i+=2)
	for (j = 0; j < h1.length(); j++)
	  high(i/2) += h1.coefficient(j)*fv.evensymindex(i-j+h1.length()/2);
    }
  return high;
}


floatVectorRef SymmetricWavelet::synthesis(const floatVector &lowchannel,
					 const floatVector &highchannel) const
{
  int lowln = 2*lowchannel.vlength();
  int highln = 2*highchannel.vlength();
  if (lowln != highln)
    {
      lowln--;
      highln++;
    }
  floatVector lowup(lowln, 0.0);
  floatVector highup(highln, 0.0);
  unsigned int i,j;
  int idx;
  if (h0.odd())
    {
      if (lowchannel.vlength() == highchannel.vlength())
	{
	  for (i = 0; i < lowup.vlength(); i++)
	    for (j = 0; j < g0.length(); j++)
	      {
		idx = i-j+g0.length()/2;
		if (!(idx%2))
		  lowup(i) += g0.coefficient(j) *
		    oddevensymindex(lowchannel, idx/2 );
	      }
	  
	  for (i = 0; i < highup.vlength(); i++)
	    for (j = 0; j < g1.length(); j++)
	      {
		idx = i-j+g1.length()/2;    
		if (idx%2)
		  highup(i) += g1.coefficient(j) *
		    evenoddsymindex(highchannel, (idx - 1)/2);
	      }
	}
      else
	{
	  for (i = 0; i < lowup.vlength(); i++)
	    for (j = 0; j < g0.length(); j++)
	      {
		idx = i-j+g0.length()/2;
		if (!(idx%2))
		  lowup(i) += g0.coefficient(j) *
		    lowchannel.oddsymindex(idx/2);
	      }

	  for (i = 0; i < highup.vlength(); i++)
	    for (j = 0; j < g1.length(); j++)
	      {
		idx = i-j+g1.length()/2;
		if (idx%2)
		  highup(i) += g1.coefficient(j) *
		    highchannel.evensymindex((idx - 1)/2);
	      }
	}  
    }
  else
    {
      if (lowchannel.vlength() == highchannel.vlength())
	{
	  for (i = 0; i < lowup.vlength(); i++)
	    for (j = 0; j < g0.length(); j++)
	      {
		idx = i-j+g0.length()/2;
		if (idx%2)
		  lowup(i) += g0.coefficient(j) *
		    oddevensymindex(lowchannel, idx/2);
	      }
	  for (i = 0; i < highup.vlength(); i++)
	    for (j = 0; j < g1.length(); j++)
	      {
		idx = i-j+g1.length()/2;
		if (idx%2)
		  highup(i) += g1.coefficient(j) *
		    evenantisymindex(highchannel, (idx-1)/2);
	      }
	}
      else
	{
	  for (i = 0; i < lowup.vlength(); i++)
	    for (j = 0; j < g0.length(); j++)
	      {
		idx = i-j+g0.length()/2;
		if (idx%2)
		  lowup(i) += g0.coefficient(j) *
		    evenoddsymindex(lowchannel,(idx-1)/2);
	      }
	  for (i = 0; i < highup.vlength(); i++)
	    for (j = 0; j < g1.length(); j++)
	      {
		idx = i-j+g1.length()/2;
		if (idx%2)
		  highup(i) += g1.coefficient(j) *
		    extevenantisymindex(highchannel, (idx-1)/2);
	      }
	} 
    }
  
  return lowup + highup;
}
  
//NB Could also change -1.0 factors to +1.0
SymmetricWavelet::SymmetricWavelet(const Sequence& h, const Sequence& ht,
				   Sign sign, double error) :
  Wavelet(h, ht, sign, error)
{
  if (h0.symmetry() == 0 || g0.symmetry() == 0)
    {
      cerr << "h0:\n" << h0 << endl
	   << "g0:\n" << g0 << endl;
    }
  ASSERT_FATAL(h0.symmetry() != 0 && g0.symmetry() != 0);
}



SymmetricWavelet::SymmetricWavelet(double *hs, unsigned int hl,
		                   double *hts, unsigned int htl) :
  Wavelet(hs, hl, hts, htl)
{
  ASSERT_FATAL(h0.symmetry() != 0 && g0.symmetry() != 0);
}



floatVectorRef SymmetricWavelet::analyse(const floatVector &fv) const
{
  floatVector low  = analysislow(fv);
  floatVector high = analysishigh(fv);
  return concatenate(low, high);
}

floatVectorRef SymmetricWavelet::synthesise(const floatVector &fv) const
{
  return synthesis(fv(0, (fv.vlength()+1)/2),
		   fv((fv.vlength()+1)/2, fv.vlength()/2));
}

//NB: Transform implementations are elegant but inefficient
floatVectorRef SymmetricWavelet::fwt(const floatVector &fv, unsigned
				     int lev) const 
{
  int w = (int)fv.vlength();
  if (w <= (1 << lev))
    {
      floatVector tmp = fv;
      return tmp;
    }
  else
    return concatenate(fwt(analysislow(fv), lev), analysishigh(fv));
}

floatVectorRef SymmetricWavelet::ifwt(const floatVector &fv, unsigned
				      int lev) const 
{
  int w = (int)fv.vlength();
  if (w <= (1 << lev))
    {
      floatVector tmp = fv;
      return tmp;
    }
  else
    return synthesis(ifwt(fv(0, (w+1)/2), lev), (fv((w+1)/2, w/2)));
}



floatImageRef SymmetricWavelet::fwt_nstd(const floatImage &fi,
					 unsigned int lev) const
{
  int w = (int)min(fi.iwidth(), fi.iheight());
  if (w <= (1 << lev))
    {
      floatImage tmp = fi;
      return tmp;
    }
  else
    {
      floatImage tmp(fi.size());
      unsigned int lw = (fi.iwidth()+1)/2;
      unsigned int lh = (fi.iheight()+1)/2;
      unsigned int i;
      for (i = 0; i < tmp.iheight(); i++)
	tmp.row(i) = rowimage(analyse(fi.row(i)));
      for (i = 0; i < tmp.iwidth(); i++)
	tmp.col(i) = colimage(analyse(tmp.col(i)));
      tmp(0,0,lw,lh) = fwt_nstd(tmp(0,0,lw,lh), lev);
      return tmp;  
    }
}

floatImageRef SymmetricWavelet::ifwt_nstd(const floatImage &fi,
					  unsigned int lev) const
{
  int w = (int)min(fi.iwidth(), fi.iheight());
  if (w <= (1 << lev))
    {
      floatImage tmp = fi;
      return tmp;
    }
  else
    {
      floatImage tmp(fi);
      unsigned int lw = (fi.iwidth()+1)/2;
      unsigned int lh = (fi.iheight()+1)/2;
      tmp(0,0,lw,lh) = ifwt_nstd(tmp(0,0,lw,lh), lev);
      unsigned int i;
      for (i = 0; i < tmp.iwidth(); i++)
	tmp.col(i) = colimage(synthesise(tmp.col(i)));
      for (i = 0; i < tmp.iheight(); i++)
	tmp.row(i) = rowimage(synthesise(tmp.row(i)));
      return tmp;  
    }
}
  
