// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1995 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: Wavelet.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $

*/

#include "Wavelet.h"

const double Wavelet::prerror = 1e-6;

//NB: See pg. 243 Vetterli and Kovacevic
SeqScaledDomain Wavelet::scalingfn(const Sequence& h, unsigned int it)
{
  Sequence prod = h;
  Sequence up = upsample(h);
  for (unsigned int i = 1; i < it; i++)
    {
      prod = up * prod;
      up = upsample(up);
    }
  prod = pow(2.0, double(it)/2.0) * prod;
  return SeqScaledDomain(prod, pow(1.0/2.0, double(it)));
}

SeqScaledDomain Wavelet::wavelet(const Sequence& h, const Sequence& g,
			          unsigned int it)
{
  Sequence prod = h;
  Sequence uph = upsample(h);
  Sequence upg = upsample(g);
  for (unsigned int i = 1; i < it-1; i++)
    {
      prod = uph * prod;
      uph = upsample(uph);
      upg = upsample(upg);
    }
  prod = pow(2.0, double(it)/2.0)  * upg * prod;
  return SeqScaledDomain(prod, pow(1.0/2.0, double(it)));
}

bool Wavelet::calcshifts(int &h0shift, int &g0shift)
{
  Sequence evenproduct = downsample(h0*g0);   // Try P(z) + P(-z) = 2z^l l even
  int product_offset = evenproduct.argmax();
  evenproduct = shift(evenproduct, product_offset);
  if (unitsequence(evenproduct))
    {
      h0shift = product_offset;
      g0shift = product_offset;
    }
  else
    {   // Try P(z) - P(-z) = 2z^l    l odd
      Sequence oddproduct = shift(downsample(shift(h0*g0, -1)), 1);
      product_offset = oddproduct.argmax();
      oddproduct = shift(oddproduct, product_offset);
      if (unitsequence(oddproduct))
	{
	  h0shift = product_offset;
	  g0shift = product_offset+1;
	}
      else
	return false;
    }

  return true;
}

bool Wavelet::alignfilters()
{
  int h0shift = 0, g0shift = 0;
  if (calcshifts(h0shift, g0shift))
    {
      h0 = shift(h0, h0shift);
      g0 = shift(g0, g0shift);
     
      return true;
    }
  return false;
}

void Wavelet::alignerror(double e)
{
  Sequence product = h0*g0;
  cerr.setf(ios::fixed);
  cerr.setf(ios::showpoint);
  cerr << "Error in constructing biorthogonal Wavelet object with"
       << " lowpass filters:\n"
       << " h:\n" << transpose(h0) << endl 
       << "~h:\n" << g0 << endl
       << "Error margin is " << e << endl << endl;
  
  int h0shift, g0shift;
  if (calcshifts(h0shift, g0shift))
    {
      cerr << "Recommended filter shifts:\n"
           << "h :  " << -h0shift << "    ~h :  " << g0shift
	   << "\ngiving initial indices:\n"
	   << "h :  " << transpose(h0).initial() + h0shift
	   << "    ~h :  " << g0.initial() - g0shift << endl;
    }
  else
    {
      cerr << "Downsampled even shift product filter:\n"
	   << downsample(product) << endl
	   << "Downsampled odd shift product filter:\n"
	   << shift(downsample(shift(product, -1)), 1) << endl;
    }
  
  exit(1);
}

void Wavelet::orthogerror(double e)
{
  cerr.setf(ios::fixed);
  cerr.setf(ios::showpoint);
  cerr << "Error in constructing orthogonal Wavelet object with"
       << " lowpass filter:\n"
       << " h:\n" << g0 << endl
       << "Perfect Reconstruction condition, which should result "
           << "in a\nsequence with the only non-zero element having"
	   <<  " value 1 at\nindex 0, produces sequence (error margin is "
           << e <<  "):\n"
       << downsample(h0*g0) << endl;
  exit(1);  
}

bool Wavelet::unitsequence(Sequence p, double e)
{
   if (fabs(p(0)-1.0) > e)
    return false;
  else
    {
      p.assign(0, 0.0);
      if (fabs(p.max()) > e)
	return false;
      else
	return true;
    }
}

bool Wavelet::productPR(const Sequence& p, double e)
{
  return unitsequence(downsample(p), e);
}


Wavelet::Wavelet(const Sequence& h, const Sequence& ht,
		 const Sequence& g, const Sequence& gt,
		 double error) :
  h0(transpose(h)), g0(ht), h1(transpose(g)), g1(gt)
{
  //See Vetterli95 pg. 112
  Sequence pr = h0*g0 + h1*g1;
  if ((fabs(pr(0) - 2.0) > error) ||
      (pr.assign(0, 0.0), (fabs(pr.max()) > error)))
    {
      cerr << "Error in constructing Wavelet object with filters:\n"
	   << " h:\n" << h << endl << " g:\n" << g << endl
	   << "~h:\n" << ht << endl << "~g:\n" << gt << endl
	   << "Perfect Reconstruction condition, which should result "
           << "in a\nsequence with the only non-zero element having"
	   <<  " value 2 at\nindex 0, produces sequence (error margin is "
           << error <<  "):\n"
	   << pr << endl;
      exit(1);
    }

  Sequence alias = altsign(h0)*g0 + altsign(h1)*g1;
  if (fabs(alias.max()) > error)
    {
      cerr << "Error in constructing Wavelet object with filters:\n"
	   << " h:\n" << h << endl << " g:\n" << g << endl
	   << "~h:\n" << ht << endl << "~g:\n" << gt << endl
	   << "Alias Cancellation condition, which should result\n"
           << "in the zero sequence, produces sequence (error margin\nis "
	   << error << "):\n"
	   << alias << endl;
      exit(1);
    }
}


//NB Could also change -1.0 factors to +1.0
Wavelet::Wavelet(const Sequence& h, const Sequence& ht,
		 Sign sign, double error) :
  h0(transpose(h)), g0(ht),
  h1(double(sign)*shift(altsign(ht),1)), g1(double(sign)*shift(altflip(h),-1))
{
#if 0
  Sequence pr = h0*g0 + h1*g1;
  Sequence alias = altsign(h0)*g0 + altsign(h1)*g1;
  cout << "PR:\n" << pr << endl << "Alias:\n" << alias << endl;
  cout << "h0:\n" << h0 << endl
       << "g0:\n" << g0 << endl
       << "h1:\n" << h1 << endl
       << "g1:\n" << g1 << endl;
#endif
  
  if (!productPR(h0*g0, error))
    alignerror(error);
}

Wavelet::Wavelet(const Sequence& h,
		 Sign sign, double error) :
  h0(transpose(h)), g0(h),
  h1(double(sign)*shift(altsign(h),1)), g1(double(sign)*shift(altflip(h),-1))
{
  if (!productPR(h0*g0, error))
    orthogerror(error);
}

Wavelet::Wavelet(double *hs, unsigned int hl,
		 double *hts, unsigned int htl) :
  h0(transpose(Sequence(hs, hl))), g0(hts, htl)
{
  if (!alignfilters())
    alignerror();
 
  h1 = -1.0*shift(altsign(g0),1);
  g1 = -1.0*shift(altsign(h0),-1);
}



Wavelet::Wavelet(double *hs, unsigned int hl) :
  h0(transpose(Sequence(hs, hl))), g0(hs, hl)
{
  if (!productPR(h0*g0))
    orthogerror();

  h1 = -1.0*shift(altsign(g0),1);
  g1 = -1.0*shift(altflip(g0),-1);
}


Wavelet::~Wavelet() { }

floatVectorRef Wavelet::analyse(const floatVector &fv) const
{
  floatVector low  = h0.downscale(fv);
  floatVector high = h1.downscale(fv);
  return concatenate(low, high);
}

floatVectorRef Wavelet::synthesise(const floatVector &fv) const
{
  floatVector slow  = g0.upscale(fv(0, fv.vlength()/2));
  floatVector shigh = g1.upscale(fv(fv.vlength()/2, fv.vlength()/2));
  return slow + shigh;
}


//NB: Transform implementations are elegant but inefficient
floatVectorRef Wavelet::fwt(const floatVector &fv, unsigned int lev) const
{
  int w = (int)fv.vlength();
  if (w <= (1 << lev))
    {
      floatVector tmp = fv;
      return tmp;
    }
  else
    return concatenate(fwt(h0.downscale(fv), lev), h1.downscale(fv));
}

floatVectorRef Wavelet::ifwt(const floatVector &fv, unsigned int lev) const
{
  int w = (int)fv.vlength();
  if (w <= (1 << lev))
    {
      floatVector tmp = fv;
      return tmp;
    }
  else
    return g0.upscale(ifwt(fv(0, w/2), lev)) + g1.upscale(fv(w/2, w/2));
}

floatImageRef Wavelet::fwt_std(const floatImage &fi, unsigned int wlev,
		               unsigned int hlev) const
{
  floatImage wt(fi.size());
  unsigned int i;
  for (i = 0; i < wt.iheight(); i++)
      wt.row(i) = rowimage(fwt(fi.row(i), wlev));
  for (i = 0; i < wt.iwidth(); i++)
      wt.col(i) = colimage(fwt(wt.col(i), hlev));
  return wt;
}

floatImageRef Wavelet::ifwt_std(const floatImage &fi, unsigned int wlev,
		                unsigned int hlev) const
{
  floatImage iwt(fi.size());
  unsigned int i;
  for (i = 0; i < iwt.iwidth(); i++)
      iwt.col(i) = colimage(ifwt(fi.col(i), hlev));
  for (i = 0; i < iwt.iheight(); i++)
      iwt.row(i) = rowimage(ifwt(iwt.row(i), wlev));
  return iwt;
}

floatImageRef Wavelet::fwt_nstd(const floatImage &fi, unsigned int lev) const
{
  int w = (int)min(fi.iwidth(), fi.iheight());
  if (w <= (1 << lev))
    {
      floatImage tmp = fi;
      return tmp;
    }
  else
    {
      floatImage tmp(fi.size());
      unsigned int i;
      for (i = 0; i < tmp.iheight(); i++)
	tmp.row(i) = rowimage(analyse(fi.row(i)));
      for (i = 0; i < tmp.iwidth(); i++)
	tmp.col(i) = colimage(analyse(tmp.col(i)));
      tmp(0,0,fi.iwidth()/2,fi.iheight()/2)
	= fwt_nstd(tmp(0,0,fi.iwidth()/2,fi.iheight()/2), lev);
      return tmp;  
    }
}

floatImageRef Wavelet::ifwt_nstd(const floatImage &fi, unsigned int lev) const
{
  int w = (int)min(fi.iwidth(), fi.iheight());
  if (w <= (1 << lev))
    {
      floatImage tmp = fi;
      return tmp;
    }
  else
    {
      floatImage tmp(fi);
      tmp(0,0,fi.iwidth()/2,fi.iheight()/2)
	= ifwt_nstd(tmp(0,0,fi.iwidth()/2,fi.iheight()/2), lev);
      unsigned int i;
      for (i = 0; i < tmp.iwidth(); i++)
	tmp.col(i) = colimage(synthesise(tmp.col(i)));
      for (i = 0; i < tmp.iheight(); i++)
	tmp.row(i) = rowimage(synthesise(tmp.row(i)));
      return tmp;  
    }
}
  
SeqScaledDomain Wavelet::phi(unsigned int it) const
{
  return scalingfn(transpose(h0), it);
}

SeqScaledDomain Wavelet::theta(unsigned int it) const
{
  return wavelet(transpose(h0), transpose(h1), it);
}

SeqScaledDomain Wavelet::tphi(unsigned int it) const
{
  return scalingfn(g0, it);
}

SeqScaledDomain Wavelet::ttheta(unsigned int it) const
{
  return wavelet(g0, g1, it);
}

ostream& operator<<(ostream &s, const Wavelet& w)
{
  s << "h  filter:\n" << transpose(w.h0) << endl
    << "g  filter:\n" << transpose(w.h1) << endl
    << "~h filter:\n" << w.g0 << endl
    << "~g filter:\n" << w.g1 << endl;
  return s; 
}
