// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: Wavelet.h,v 1.1 2003/02/01 17:46:23 brendt Exp $

 */

#ifndef _Wavelet_h
#define _Wavelet_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
#include <iomanip>
using std::cerr;
using std::setw;
using std::endl;
using std::ios;
#else
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#endif

#include "floatVector.h"
#include "floatImage.h"
#include "Sequence.h"

//! unit="Wavelets"
//! module="Support" 
//! level = 2

enum Sign {Negative = -1, Positive = 1};

//:Wavelet class using periodic extension
class Wavelet {
protected:
  Sequence h0;
  Sequence g0;
  Sequence h1;
  Sequence g1;

  static const double prerror;

  bool calcshifts(int &h0shift, int &g0shift);
  bool alignfilters();
  void alignerror(double e = prerror);
  void orthogerror(double e = prerror);
  
  static SeqScaledDomain scalingfn(const Sequence& h, unsigned int it);
  static SeqScaledDomain wavelet(const Sequence& h, const Sequence& g,
			         unsigned int it);
  static bool unitsequence(Sequence p, double e = prerror);
  static bool productPR(const Sequence& p, double e = prerror);
  
public:
  //Construct biorthogonal wavelet pair with analysis and synthesis
  //(lowpass, highpass) pairs as (h,g) and (ht, gt). Allow up to error 
  //deviation from perfect reconstruction.
  Wavelet(const Sequence& h, const Sequence& ht,
	  const Sequence& g, const Sequence& gt,
	  double error = prerror);
  //Construct biorthogonal wavelet pair with analysis and synthesis
  //lowpass filters h and ht. Highpass filters are automatically
  //computed, with the specified initial sign.
  Wavelet(const Sequence& h, const Sequence& ht,
	  Sign sign = Negative, double error = prerror);
  //Construct orthogonal wavelet with lowpass sequence h.
  Wavelet(const Sequence& h, Sign sign = Negative, double error = prerror);
  //Construct biorthogonal wavelet pair with analysis and synthesis
  //lowpass coeffients from arrays hs and hts, lengths given by hl and 
  //htl respectively. Filter alignment is computed automatically.
  Wavelet(double *hs, unsigned int hl, double *hts, unsigned int htl);
  // Construct orthogonal wavelet with coeffients from array hs,
  // length hl.
  Wavelet(double *hs, unsigned int hl);
  virtual ~Wavelet();

  //One stage of analysis
  virtual floatVectorRef analyse(const floatVector &fv) const;
  //One stage of synethesis
  virtual floatVectorRef synthesise(const floatVector &fv) const;
  
  //Fast Wavelet Transform of fv, lev number of levels are left undone
  virtual floatVectorRef fwt(const floatVector &fv,
			     unsigned int lev = 0) const;
  //Inverse Fast Wavelet Transform of fv, lev number of levels are left undone
  virtual floatVectorRef ifwt(const floatVector &fv, 
			      unsigned int lev = 0) const;
  //Standard decomposition of an image, wlev and hlev levels are left
  //undone in the horizontal and vertical directions respectively.
  virtual floatImageRef fwt_std(const floatImage &fi, unsigned int wlev = 0,
		                unsigned int hlev = 0) const;
  //Inverse standard decomposition of an image, wlev and hlev levels are left
  //done in the horizontal and vertical directions respectively.
  virtual floatImageRef ifwt_std(const floatImage &fi, unsigned int wlev = 0,
		                 unsigned int hlev = 0) const;
  //Non-standard decomposition of an image, lev levels are left undone.
  virtual floatImageRef fwt_nstd(const floatImage &fi, unsigned int
				 lev = 0) const;
  //Inverse non-standard decomposition of an image, lev levels are left undone.
  virtual floatImageRef ifwt_nstd(const floatImage &fi, unsigned int
				  lev = 0) const;
  
  //Approximate scaling function
  SeqScaledDomain phi(unsigned int it = 8) const;
  //Approximate wavelet
  SeqScaledDomain theta(unsigned int it = 8) const;
  //Approximate dual scaling function
  SeqScaledDomain tphi(unsigned int it = 8) const;
  //Approximate dual wavelet
  SeqScaledDomain ttheta(unsigned int it = 8) const;
  
  friend ostream& operator<<(ostream &s, const Wavelet& w);
};

#endif
